@@ -5,15 +5,11 @@ use MediaWiki\Auth\UsernameAuthenticationRequest;
 use Wikimedia\TestingAccessWrapper;
 
 /**
+ * @covers CaptchaPreAuthenticationProvider
  * @group Database
  */
 class CaptchaPreAuthenticationProviderTest extends MediaWikiTestCase {
 	public function setUp() {
-		global $wgDisableAuthManager;
-		if ( !class_exists( AuthManager::class ) || $wgDisableAuthManager ) {
-			$this->markTestSkipped( 'AuthManager is disabled' );
-		}
-
 		parent::setUp();
 		$this->setMwGlobals( [
 			'wgCaptchaClass' => SimpleCaptcha::class,
