@@ -2,10 +2,6 @@
  * JavaScript for WikiEditor Toolbar
  */
 jQuery( function ( $ ) {
-	if ( !$.wikiEditor.isSupported( $.wikiEditor.modules.toolbar ) ) {
-		$( '.wikiEditor-oldToolbar' ).show();
-		return;
-	}
 	// The old toolbar is still in place and needs to be removed so there aren't two toolbars
 	$( '#toolbar' ).remove();
 	// Add toolbar module
