@@ -117,8 +117,7 @@
 							actions = $characters.data( 'actions' );
 							for ( i = 0; i < data.characters.length; i++ ) {
 								// Character
-								$characters
-								.append(
+								$characters.append(
 									$( $.wikiEditor.modules.toolbar.fn.buildCharacter( data.characters[ i ], actions ) )
 										.mousedown( function ( e ) {
 											// No dragging!
@@ -285,7 +284,7 @@
 				if ( label ) {
 					$label = $( '<span>' )
 						.addClass( 'label' )
-						.html( label );
+						.text( label );
 					$group.append( $label );
 				}
 				empty = true;
@@ -306,7 +305,7 @@
 				return $group;
 			},
 			buildTool: function ( context, id, tool ) {
-				var i, label, $button, offsetOrIcon, $select, $options,
+				var i, label, $button, config, icon, $select, $options, oouiButton,
 					option, optionLabel;
 				if ( 'filters' in tool ) {
 					for ( i = 0; i < tool.filters.length; i++ ) {
@@ -315,31 +314,51 @@
 						}
 					}
 				}
-				label = $.wikiEditor.autoMsg( tool, 'label' );
+				label = $.wikiEditor.autoSafeMsg( tool, 'label' );
 				switch ( tool.type ) {
 					case 'button':
-						offsetOrIcon = $.wikiEditor.autoIconOrOffset(
-							tool.icon,
-							tool.offset,
-							$.wikiEditor.imgPath + 'toolbar/'
-						);
-						$button = $( '<a>' )
-							.attr( {
-								href: '#',
-								title: label,
-								rel: id,
-								role: 'button',
-								'class': 'tool tool-button'
-							} )
-							.text( label );
-						if ( typeof offsetOrIcon === 'object' ) {
-							$button
-							.addClass( 'wikiEditor-toolbar-spritedButton' )
-							.css( 'backgroundPosition', offsetOrIcon[ 0 ] + 'px ' + offsetOrIcon[ 1 ] + 'px' );
-						} else if ( offsetOrIcon !== undefined ) { // Bug T172500
-							$button
-							.css( 'background-image', 'url(' + offsetOrIcon + ')' );
+					case 'toggle':
+						if ( tool.oouiIcon ) {
+							config = {
+								framed: false,
+								classes: [ 'tool' ],
+								icon: tool.oouiIcon,
+								title: label
+							};
+							if ( tool.type === 'button' ) {
+								oouiButton = new OO.ui.ButtonWidget( config );
+							} else if ( tool.type === 'toggle' ) {
+								oouiButton = new OO.ui.ToggleButtonWidget( config );
+							}
+							$button = oouiButton.$element;
+							$button.attr( 'rel', id );
+							$button.data( 'ooui', oouiButton );
+						} else {
+							$button = $( '<a>' )
+								.attr( {
+									href: '#',
+									title: label,
+									rel: id,
+									role: 'button',
+									'class': 'tool tool-button'
+								} )
+								.text( label );
+							if ( tool.icon ) {
+								icon = $.wikiEditor.autoIcon(
+									tool.icon,
+									$.wikiEditor.imgPath + 'toolbar/'
+								);
+								$button.css( 'background-image', 'url(' + icon + ')' );
+							}
 						}
+						$button.data( 'setActive', function ( active ) {
+							$button.toggleClass( 'tool-active', active );
+
+							// OOUI button
+							if ( $button.data( 'ooui' ) && tool.type === 'toggle' ) {
+								$button.data( 'ooui' ).setValue( active );
+							}
+						} );
 						if ( 'action' in tool ) {
 							$button
 								.data( 'action', tool.action )
@@ -348,14 +367,22 @@
 									// No dragging!
 									e.preventDefault();
 									return false;
-								} )
-								.click( function ( e ) {
+								} );
+							if ( $button.data( 'ooui' ) ) {
+								$button.data( 'ooui' ).on( 'click', function () {
 									$.wikiEditor.modules.toolbar.fn.doAction(
-										$( this ).data( 'context' ), $( this ).data( 'action' ), $( this )
+										context, tool.action
+									);
+								} );
+							} else {
+								$button.on( 'click', function ( e ) {
+									$.wikiEditor.modules.toolbar.fn.doAction(
+										context, tool.action
 									);
 									e.preventDefault();
 									return false;
 								} );
+							}
 						}
 						return $button;
 					case 'select':
@@ -395,20 +422,20 @@
 						}
 						$select.append( $( '<div>' ).addClass( 'menu' ).append( $options ) );
 						$select.append( $( '<a>' )
-								.addClass( 'label' )
-								.text( label )
-								.data( 'options', $options )
-								.attr( 'href', '#' )
-								.mousedown( function ( e ) {
-									// No dragging!
-									e.preventDefault();
-									return false;
-								} )
-								.click( function ( e ) {
-									$( this ).data( 'options' ).animate( { opacity: 'toggle' }, 'fast' );
-									e.preventDefault();
-									return false;
-								} )
+							.addClass( 'label' )
+							.text( label )
+							.data( 'options', $options )
+							.attr( 'href', '#' )
+							.mousedown( function ( e ) {
+								// No dragging!
+								e.preventDefault();
+								return false;
+							} )
+							.click( function ( e ) {
+								$( this ).data( 'options' ).animate( { opacity: 'toggle' }, 'fast' );
+								e.preventDefault();
+								return false;
+							} )
 						);
 						return $select;
 					default:
@@ -519,7 +546,7 @@
 			buildHeading: function ( context, headings ) {
 				var i, html = '<tr>';
 				for ( i = 0; i < headings.length; i++ ) {
-					html += '<th>' + $.wikiEditor.autoMsg( headings[ i ], [ 'html', 'text' ] ) + '</th>';
+					html += '<th>' + $.wikiEditor.autoSafeMsg( headings[ i ], [ 'html', 'text' ] ) + '</th>';
 				}
 				return html + '</tr>';
 			},
@@ -527,6 +554,8 @@
 				var cell,
 					html = '<tr>';
 				for ( cell in row ) {
+					// FIXME: This currently needs to use the "unsafe" .text() message because it embeds raw HTML
+					// in the messages (as used exclusively by the 'help' toolbar panel).
 					html += '<td class="cell cell-' + cell + '"><span>' +
 						$.wikiEditor.autoMsg( row[ cell ], [ 'html', 'text' ] ) + '</span></td>';
 				}
