@@ -208,13 +208,14 @@ class Block {
 	public static function selectFields() {
 		global $wgActorTableSchemaMigrationStage;
 
-		if ( $wgActorTableSchemaMigrationStage > MIGRATION_WRITE_BOTH ) {
+		if ( $wgActorTableSchemaMigrationStage & SCHEMA_COMPAT_READ_NEW ) {
 			// If code is using this instead of self::getQueryInfo(), there's a
 			// decent chance it's going to try to directly access
 			// $row->ipb_by or $row->ipb_by_text and we can't give it
-			// useful values here once those aren't being written anymore.
+			// useful values here once those aren't being used anymore.
 			throw new BadMethodCallException(
-				'Cannot use ' . __METHOD__ . ' when $wgActorTableSchemaMigrationStage > MIGRATION_WRITE_BOTH'
+				'Cannot use ' . __METHOD__
+					. ' when $wgActorTableSchemaMigrationStage has SCHEMA_COMPAT_READ_NEW'
 			);
 		}
 
@@ -224,7 +225,7 @@ class Block {
 			'ipb_address',
 			'ipb_by',
 			'ipb_by_text',
-			'ipb_by_actor' => $wgActorTableSchemaMigrationStage > MIGRATION_OLD ? 'ipb_by_actor' : null,
+			'ipb_by_actor' => 'NULL',
 			'ipb_timestamp',
 			'ipb_auto',
 			'ipb_anon_only',
@@ -299,7 +300,7 @@ class Block {
 	 *     1) A block directly on the given user or IP
 	 *     2) A rangeblock encompassing the given IP (smallest first)
 	 *     3) An autoblock on the given IP
-	 * @param User|string $vagueTarget Also search for blocks affecting this target.  Doesn't
+	 * @param User|string|null $vagueTarget Also search for blocks affecting this target.  Doesn't
 	 *     make any sense to use TYPE_AUTO / TYPE_ID here. Leave blank to skip IP lookups.
 	 * @throws MWException
 	 * @return bool Whether a relevant block was found
@@ -405,7 +406,7 @@ class Block {
 	/**
 	 * Get a set of SQL conditions which will select rangeblocks encompassing a given range
 	 * @param string $start Hexadecimal IP representation
-	 * @param string $end Hexadecimal IP representation, or null to use $start = $end
+	 * @param string|null $end Hexadecimal IP representation, or null to use $start = $end
 	 * @return string
 	 */
 	public static function getRangeCond( $start, $end = null ) {
@@ -458,7 +459,7 @@ class Block {
 	protected function initFromRow( $row ) {
 		$this->setTarget( $row->ipb_address );
 		$this->setBlocker( User::newFromAnyId(
-			$row->ipb_by, $row->ipb_by_text, isset( $row->ipb_by_actor ) ? $row->ipb_by_actor : null
+			$row->ipb_by, $row->ipb_by_text, $row->ipb_by_actor ?? null
 		) );
 
 		$this->mTimestamp = wfTimestamp( TS_MW, $row->ipb_timestamp );
@@ -519,9 +520,9 @@ class Block {
 	 * Insert a block into the block table. Will fail if there is a conflicting
 	 * block (same name and options) already in the database.
 	 *
-	 * @param IDatabase $dbw If you have one available
+	 * @param IDatabase|null $dbw If you have one available
 	 * @return bool|array False on failure, assoc array on success:
-	 *	('id' => block ID, 'autoIds' => array of autoblock IDs)
+	 * 	('id' => block ID, 'autoIds' => array of autoblock IDs)
 	 */
 	public function insert( $dbw = null ) {
 		global $wgBlockDisablesLogin;
@@ -1162,7 +1163,7 @@ class Block {
 	 *     Calling this with a user, IP address or range will not select autoblocks, and will
 	 *     only select a block where the targets match exactly (so looking for blocks on
 	 *     1.2.3.4 will not select 1.2.0.0/16 or even 1.2.3.4/32)
-	 * @param string|User|int $vagueTarget As above, but we will search for *any* block which
+	 * @param string|User|int|null $vagueTarget As above, but we will search for *any* block which
 	 *     affects that target (so for an IP address, get ranges containing that IP; and also
 	 *     get any relevant autoblocks). Leave empty or blank to skip IP-based lookups.
 	 * @param bool $fromMaster Whether to use the DB_MASTER database
@@ -1420,7 +1421,7 @@ class Block {
 		}
 
 		# Consider the possibility that this is not a username at all
-		# but actually an old subpage (bug #29797)
+		# but actually an old subpage (T31797)
 		if ( strpos( $target, '/' ) !== false ) {
 			# An old subpage, drill down to the user behind it
 			$target = explode( '/', $target )[0];
@@ -1641,7 +1642,7 @@ class Block {
 			$reason,
 			$context->getRequest()->getIP(),
 			$this->getByName(),
-			$systemBlockType !== null ? $systemBlockType : $this->getId(),
+			$systemBlockType ?? $this->getId(),
 			$lang->formatExpiry( $this->mExpiry ),
 			(string)$intended,
 			$lang->userTimeAndDate( $this->mTimestamp, $context->getUser() ),
