@@ -38,6 +38,15 @@ class MWNamespace {
 	 */
 	private static $alwaysCapitalizedNamespaces = [ NS_SPECIAL, NS_USER, NS_MEDIAWIKI ];
 
+	/** @var string[]|null Canonical namespaces cache */
+	private static $canonicalNamespaces = null;
+
+	/** @var array|false Canonical namespaces index cache */
+	private static $namespaceIndexes = false;
+
+	/** @var int[]|null Valid namespaces cache */
+	private static $validNamespaces = null;
+
 	/**
 	 * Throw an exception when trying to get the subject or talk page
 	 * for a given namespace where it does not make sense.
@@ -57,6 +66,19 @@ class MWNamespace {
 		return true;
 	}
 
+	/**
+	 * Clear internal caches
+	 *
+	 * For use in unit testing when namespace configuration is changed.
+	 *
+	 * @since 1.31
+	 */
+	public static function clearCaches() {
+		self::$canonicalNamespaces = null;
+		self::$namespaceIndexes = false;
+		self::$validNamespaces = null;
+	}
+
 	/**
 	 * Can pages in the given namespace be moved?
 	 *
@@ -200,23 +222,28 @@ class MWNamespace {
 	 * (English) names.
 	 *
 	 * @param bool $rebuild Rebuild namespace list (default = false). Used for testing.
+	 *  Deprecated since 1.31, use self::clearCaches() instead.
 	 *
 	 * @return array
 	 * @since 1.17
 	 */
 	public static function getCanonicalNamespaces( $rebuild = false ) {
-		static $namespaces = null;
-		if ( $namespaces === null || $rebuild ) {
-			global $wgExtraNamespaces, $wgCanonicalNamespaceNames;
-			$namespaces = [ NS_MAIN => '' ] + $wgCanonicalNamespaceNames;
-			// Add extension namespaces
-			$namespaces += ExtensionRegistry::getInstance()->getAttribute( 'ExtensionNamespaces' );
-			if ( is_array( $wgExtraNamespaces ) ) {
-				$namespaces += $wgExtraNamespaces;
-			}
-			Hooks::run( 'CanonicalNamespaces', [ &$namespaces ] );
+		if ( $rebuild ) {
+			self::clearCaches();
 		}
-		return $namespaces;
+
+		if ( self::$canonicalNamespaces === null ) {
+			global $wgExtraNamespaces, $wgCanonicalNamespaceNames;
+			self::$canonicalNamespaces = [ NS_MAIN => '' ] + $wgCanonicalNamespaceNames;
+			// Add extension namespaces
+			self::$canonicalNamespaces +=
+				ExtensionRegistry::getInstance()->getAttribute( 'ExtensionNamespaces' );
+			if ( is_array( $wgExtraNamespaces ) ) {
+				self::$canonicalNamespaces += $wgExtraNamespaces;
+			}
+			Hooks::run( 'CanonicalNamespaces', [ &self::$canonicalNamespaces ] );
+		}
+		return self::$canonicalNamespaces;
 	}
 
 	/**
@@ -227,11 +254,7 @@ class MWNamespace {
 	 */
 	public static function getCanonicalName( $index ) {
 		$nslist = self::getCanonicalNamespaces();
-		if ( isset( $nslist[$index] ) ) {
-			return $nslist[$index];
-		} else {
-			return false;
-		}
+		return $nslist[$index] ?? false;
 	}
 
 	/**
@@ -242,15 +265,14 @@ class MWNamespace {
 	 * @return int
 	 */
 	public static function getCanonicalIndex( $name ) {
-		static $xNamespaces = false;
-		if ( $xNamespaces === false ) {
-			$xNamespaces = [];
+		if ( self::$namespaceIndexes === false ) {
+			self::$namespaceIndexes = [];
 			foreach ( self::getCanonicalNamespaces() as $i => $text ) {
-				$xNamespaces[strtolower( $text )] = $i;
+				self::$namespaceIndexes[strtolower( $text )] = $i;
 			}
 		}
-		if ( array_key_exists( $name, $xNamespaces ) ) {
-			return $xNamespaces[$name];
+		if ( array_key_exists( $name, self::$namespaceIndexes ) ) {
+			return self::$namespaceIndexes[$name];
 		} else {
 			return null;
 		}
@@ -262,19 +284,17 @@ class MWNamespace {
 	 * @return array
 	 */
 	public static function getValidNamespaces() {
-		static $mValidNamespaces = null;
-
-		if ( is_null( $mValidNamespaces ) ) {
+		if ( is_null( self::$validNamespaces ) ) {
 			foreach ( array_keys( self::getCanonicalNamespaces() ) as $ns ) {
 				if ( $ns >= 0 ) {
-					$mValidNamespaces[] = $ns;
+					self::$validNamespaces[] = $ns;
 				}
 			}
 			// T109137: sort numerically
-			sort( $mValidNamespaces, SORT_NUMERIC );
+			sort( self::$validNamespaces, SORT_NUMERIC );
 		}
 
-		return $mValidNamespaces;
+		return self::$validNamespaces;
 	}
 
 	/**
@@ -448,9 +468,7 @@ class MWNamespace {
 	 */
 	public static function getNamespaceContentModel( $index ) {
 		global $wgNamespaceContentModels;
-		return isset( $wgNamespaceContentModels[$index] )
-			? $wgNamespaceContentModels[$index]
-			: null;
+		return $wgNamespaceContentModels[$index] ?? null;
 	}
 
 	/**
@@ -459,7 +477,7 @@ class MWNamespace {
 	 *
 	 * @since 1.23
 	 * @param int $index Index to check
-	 * @param User $user User to check
+	 * @param User|null $user User to check
 	 * @return array
 	 */
 	public static function getRestrictionLevels( $index, User $user = null ) {
@@ -522,4 +540,26 @@ class MWNamespace {
 
 		return $usableLevels;
 	}
+
+	/**
+	 * Returns the link type to be used for categories.
+	 *
+	 * This determines which section of a category page titles
+	 * in the namespace will appear within.
+	 *
+	 * @since 1.32
+	 * @param int $index Namespace index
+	 * @return string One of 'subcat', 'file', 'page'
+	 */
+	public static function getCategoryLinkType( $index ) {
+		self::isMethodValidFor( $index, __METHOD__ );
+
+		if ( $index == NS_CATEGORY ) {
+			return 'subcat';
+		} elseif ( $index == NS_FILE ) {
+			return 'file';
+		} else {
+			return 'page';
+		}
+	}
 }
