@@ -37,34 +37,10 @@
  * developer of the calling code is reminded that the function can fail, and
  * so that a lack of error-handling will be explicit.
  */
-class Status {
-	/** @var StatusValue */
-	protected $sv;
-
-	/** @var mixed */
-	public $value;
-	/** @var array Map of (key => bool) to indicate success of each part of batch operations */
-	public $success = array();
-	/** @var int Counter for batch operations */
-	public $successCount = 0;
-	/** @var int Counter for batch operations */
-	public $failCount = 0;
-
+class Status extends StatusValue {
 	/** @var callable */
 	public $cleanCallback = false;
 
-	/**
-	 * @param StatusValue $sv [optional]
-	 */
-	public function __construct( StatusValue $sv = null ) {
-		$this->sv = ( $sv === null ) ? new StatusValue() : $sv;
-		// B/C field aliases
-		$this->value =& $this->sv->value;
-		$this->successCount =& $this->sv->successCount;
-		$this->failCount =& $this->sv->failCount;
-		$this->success =& $this->sv->success;
-	}
-
 	/**
 	 * Succinct helper method to wrap a StatusValue
 	 *
@@ -77,90 +53,83 @@ class Status {
 	 * @return Status
 	 */
 	public static function wrap( $sv ) {
-		return $sv instanceof Status ? $sv : new self( $sv );
+		if ( $sv instanceof static ) {
+			return $sv;
+		}
+
+		$result = new static();
+		$result->ok =& $sv->ok;
+		$result->errors =& $sv->errors;
+		$result->value =& $sv->value;
+		$result->successCount =& $sv->successCount;
+		$result->failCount =& $sv->failCount;
+		$result->success =& $sv->success;
+
+		return $result;
 	}
 
 	/**
-	 * Factory function for fatal errors
+	 * Backwards compatibility logic
 	 *
-	 * @param string|Message $message Message name or object
-	 * @return Status
+	 * @param string $name
+	 * @return mixed
+	 * @throws RuntimeException
 	 */
-	public static function newFatal( $message /*, parameters...*/ ) {
-		return new self( call_user_func_array(
-			array( 'StatusValue', 'newFatal' ), func_get_args()
-		) );
-	}
+	function __get( $name ) {
+		if ( $name === 'ok' ) {
+			return $this->isOK();
+		} elseif ( $name === 'errors' ) {
+			return $this->getErrors();
+		}
 
-	/**
-	 * Factory function for good results
-	 *
-	 * @param mixed $value
-	 * @return Status
-	 */
-	public static function newGood( $value = null ) {
-		$sv = new StatusValue();
-		$sv->value = $value;
-
-		return new self( $sv );
+		throw new RuntimeException( "Cannot get '$name' property." );
 	}
 
 	/**
 	 * Change operation result
+	 * Backwards compatibility logic
 	 *
-	 * @param bool $ok Whether the operation completed
+	 * @param string $name
 	 * @param mixed $value
+	 * @throws RuntimeException
 	 */
-	public function setResult( $ok, $value = null ) {
-		$this->sv->setResult( $ok, $value );
+	function __set( $name, $value ) {
+		if ( $name === 'ok' ) {
+			$this->setOK( $value );
+		} elseif ( !property_exists( $this, $name ) ) {
+			// Caller is using undeclared ad-hoc properties
+			$this->$name = $value;
+		} else {
+			throw new RuntimeException( "Cannot set '$name' property." );
+		}
 	}
 
 	/**
-	 * Returns whether the operation completed and didn't have any error or
-	 * warnings
+	 * Splits this Status object into two new Status objects, one which contains only
+	 * the error messages, and one that contains the warnings, only. The returned array is
+	 * defined as:
+	 * [
+	 *     0 => object(Status) # the Status with error messages, only
+	 *     1 => object(Status) # The Status with warning messages, only
+	 * ]
 	 *
-	 * @return bool
+	 * @return Status[]
 	 */
-	public function isGood() {
-		return $this->sv->isGood();
+	public function splitByErrorType() {
+		list( $errorsOnlyStatus, $warningsOnlyStatus ) = parent::splitByErrorType();
+		$errorsOnlyStatus->cleanCallback =
+			$warningsOnlyStatus->cleanCallback = $this->cleanCallback;
+
+		return [ $errorsOnlyStatus, $warningsOnlyStatus ];
 	}
 
 	/**
-	 * Returns whether the operation completed
-	 *
-	 * @return bool
+	 * Returns the wrapped StatusValue object
+	 * @return StatusValue
+	 * @since 1.27
 	 */
-	public function isOK() {
-		return $this->sv->isOK();
-	}
-
-	/**
-	 * Add a new warning
-	 *
-	 * @param string|Message $message Message name or object
-	 */
-	public function warning( $message /*, parameters... */ ) {
-		call_user_func_array( array( $this->sv, 'warning' ), func_get_args() );
-	}
-
-	/**
-	 * Add an error, do not set fatal flag
-	 * This can be used for non-fatal errors
-	 *
-	 * @param string|Message $message Message name or object
-	 */
-	public function error( $message /*, parameters... */ ) {
-		call_user_func_array( array( $this->sv, 'error' ), func_get_args() );
-	}
-
-	/**
-	 * Add an error and set OK to false, indicating that the operation
-	 * as a whole was fatal
-	 *
-	 * @param string|Message $message Message name or object
-	 */
-	public function fatal( $message /*, parameters... */ ) {
-		call_user_func_array( array( $this->sv, 'fatal' ), func_get_args() );
+	public function getStatusValue() {
+		return $this;
 	}
 
 	/**
@@ -171,103 +140,132 @@ class Status {
 		if ( !$this->cleanCallback ) {
 			return $params;
 		}
-		$cleanParams = array();
+		$cleanParams = [];
 		foreach ( $params as $i => $param ) {
 			$cleanParams[$i] = call_user_func( $this->cleanCallback, $param );
 		}
 		return $cleanParams;
 	}
 
+	/**
+	 * @param string|Language|null $lang Language to use for processing
+	 *  messages, or null to default to the user language.
+	 * @return Language
+	 */
+	protected function languageFromParam( $lang ) {
+		global $wgLang;
+
+		if ( $lang === null ) {
+			// @todo: Use RequestContext::getMain()->getLanguage() instead
+			return $wgLang;
+		} elseif ( $lang instanceof Language || $lang instanceof StubUserLang ) {
+			return $lang;
+		} else {
+			return Language::factory( $lang );
+		}
+	}
+
 	/**
 	 * Get the error list as a wikitext formatted list
 	 *
 	 * @param string|bool $shortContext A short enclosing context message name, to
 	 *        be used when there is a single error
 	 * @param string|bool $longContext A long enclosing context message name, for a list
+	 * @param string|Language|null $lang Language to use for processing messages
 	 * @return string
 	 */
-	public function getWikiText( $shortContext = false, $longContext = false ) {
-		$rawErrors = $this->sv->getErrors();
+	public function getWikiText( $shortContext = false, $longContext = false, $lang = null ) {
+		$lang = $this->languageFromParam( $lang );
+
+		$rawErrors = $this->getErrors();
 		if ( count( $rawErrors ) == 0 ) {
-			if ( $this->sv->isOK() ) {
-				$this->sv->fatal( 'internalerror_info',
+			if ( $this->isOK() ) {
+				$this->fatal( 'internalerror_info',
 					__METHOD__ . " called for a good result, this is incorrect\n" );
 			} else {
-				$this->sv->fatal( 'internalerror_info',
+				$this->fatal( 'internalerror_info',
 					__METHOD__ . ": Invalid result object: no error text but not OK\n" );
 			}
-			$rawErrors = $this->sv->getErrors(); // just added a fatal
+			$rawErrors = $this->getErrors(); // just added a fatal
 		}
 		if ( count( $rawErrors ) == 1 ) {
-			$s = $this->getErrorMessage( $rawErrors[0] )->plain();
+			$s = $this->getErrorMessage( $rawErrors[0], $lang )->plain();
 			if ( $shortContext ) {
-				$s = wfMessage( $shortContext, $s )->plain();
+				$s = wfMessage( $shortContext, $s )->inLanguage( $lang )->plain();
 			} elseif ( $longContext ) {
-				$s = wfMessage( $longContext, "* $s\n" )->plain();
+				$s = wfMessage( $longContext, "* $s\n" )->inLanguage( $lang )->plain();
 			}
 		} else {
-			$errors = $this->getErrorMessageArray( $rawErrors );
+			$errors = $this->getErrorMessageArray( $rawErrors, $lang );
 			foreach ( $errors as &$error ) {
 				$error = $error->plain();
 			}
 			$s = '* ' . implode( "\n* ", $errors ) . "\n";
 			if ( $longContext ) {
-				$s = wfMessage( $longContext, $s )->plain();
+				$s = wfMessage( $longContext, $s )->inLanguage( $lang )->plain();
 			} elseif ( $shortContext ) {
-				$s = wfMessage( $shortContext, "\n$s\n" )->plain();
+				$s = wfMessage( $shortContext, "\n$s\n" )->inLanguage( $lang )->plain();
 			}
 		}
 		return $s;
 	}
 
 	/**
-	 * Get the error list as a Message object
+	 * Get a bullet list of the errors as a Message object.
 	 *
-	 * @param string|string[] $shortContext A short enclosing context message name (or an array of
-	 * message names), to be used when there is a single error.
-	 * @param string|string[] $longContext A long enclosing context message name (or an array of
-	 * message names), for a list.
+	 * $shortContext and $longContext can be used to wrap the error list in some text.
+	 * $shortContext will be preferred when there is a single error; $longContext will be
+	 * preferred when there are multiple ones. In either case, $1 will be replaced with
+	 * the list of errors.
 	 *
+	 * $shortContext is assumed to use $1 as an inline parameter: if there is a single item,
+	 * it will not be made into a list; if there are multiple items, newlines will be inserted
+	 * around the list.
+	 * $longContext is assumed to use $1 as a standalone parameter; it will always receive a list.
+	 *
+	 * If both parameters are missing, and there is only one error, no bullet will be added.
+	 *
+	 * @param string|string[]|bool $shortContext A message name or an array of message names.
+	 * @param string|string[]|bool $longContext A message name or an array of message names.
+	 * @param string|Language|null $lang Language to use for processing messages
 	 * @return Message
 	 */
-	public function getMessage( $shortContext = false, $longContext = false ) {
-		$rawErrors = $this->sv->getErrors();
+	public function getMessage( $shortContext = false, $longContext = false, $lang = null ) {
+		$lang = $this->languageFromParam( $lang );
+
+		$rawErrors = $this->getErrors();
 		if ( count( $rawErrors ) == 0 ) {
-			if ( $this->sv->isOK() ) {
-				$this->sv->fatal( 'internalerror_info',
+			if ( $this->isOK() ) {
+				$this->fatal( 'internalerror_info',
 					__METHOD__ . " called for a good result, this is incorrect\n" );
 			} else {
-				$this->sv->fatal( 'internalerror_info',
+				$this->fatal( 'internalerror_info',
 					__METHOD__ . ": Invalid result object: no error text but not OK\n" );
 			}
-			$rawErrors = $this->sv->getErrors(); // just added a fatal
+			$rawErrors = $this->getErrors(); // just added a fatal
 		}
 		if ( count( $rawErrors ) == 1 ) {
-			$s = $this->getErrorMessage( $rawErrors[0] );
+			$s = $this->getErrorMessage( $rawErrors[0], $lang );
 			if ( $shortContext ) {
-				$s = wfMessage( $shortContext, $s );
+				$s = wfMessage( $shortContext, $s )->inLanguage( $lang );
 			} elseif ( $longContext ) {
 				$wrapper = new RawMessage( "* \$1\n" );
 				$wrapper->params( $s )->parse();
-				$s = wfMessage( $longContext, $wrapper );
+				$s = wfMessage( $longContext, $wrapper )->inLanguage( $lang );
 			}
 		} else {
-			$msgs = $this->getErrorMessageArray( $rawErrors );
+			$msgs = $this->getErrorMessageArray( $rawErrors, $lang );
 			$msgCount = count( $msgs );
 
-			if ( $shortContext ) {
-				$msgCount++;
-			}
-
 			$s = new RawMessage( '* $' . implode( "\n* \$", range( 1, $msgCount ) ) );
 			$s->params( $msgs )->parse();
 
 			if ( $longContext ) {
-				$s = wfMessage( $longContext, $s );
+				$s = wfMessage( $longContext, $s )->inLanguage( $lang );
 			} elseif ( $shortContext ) {
-				$wrapper = new RawMessage( "\n\$1\n", $s );
+				$wrapper = new RawMessage( "\n\$1\n", [ $s ] );
 				$wrapper->parse();
-				$s = wfMessage( $shortContext, $wrapper );
+				$s = wfMessage( $shortContext, $wrapper )->inLanguage( $lang );
 			}
 		}
 
@@ -275,15 +273,16 @@ class Status {
 	}
 
 	/**
-	 * Return the message for a single error.
-	 * @param mixed $error With an array & two values keyed by
-	 * 'message' and 'params', use those keys-value pairs.
-	 * Otherwise, if its an array, just use the first value as the
-	 * message and the remaining items as the params.
+	 * Return the message for a single error
 	 *
+	 * The code string can be used a message key with per-language versions.
+	 * If $error is an array, the "params" field is a list of parameters for the message.
+	 *
+	 * @param array|string $error Code string or (key: code string, params: string[]) map
+	 * @param string|Language|null $lang Language to use for processing messages
 	 * @return Message
 	 */
-	protected function getErrorMessage( $error ) {
+	protected function getErrorMessage( $error, $lang = null ) {
 		if ( is_array( $error ) ) {
 			if ( isset( $error['message'] ) && $error['message'] instanceof Message ) {
 				$msg = $error['message'];
@@ -295,43 +294,44 @@ class Status {
 				$msg = wfMessage( $msgName,
 					array_map( 'wfEscapeWikiText', $this->cleanParams( $error ) ) );
 			}
-		} else {
+		} elseif ( is_string( $error ) ) {
 			$msg = wfMessage( $error );
+		} else {
+			throw new UnexpectedValueException( "Got " . get_class( $error ) . " for key." );
 		}
+
+		$msg->inLanguage( $this->languageFromParam( $lang ) );
 		return $msg;
 	}
 
 	/**
-	 * Get the error message as HTML. This is done by parsing the wikitext error
-	 * message.
-	 * @param string $shortContext A short enclosing context message name, to
+	 * Get the error message as HTML. This is done by parsing the wikitext error message
+	 * @param string|bool $shortContext A short enclosing context message name, to
 	 *        be used when there is a single error
-	 * @param string $longContext A long enclosing context message name, for a list
+	 * @param string|bool $longContext A long enclosing context message name, for a list
+	 * @param string|Language|null $lang Language to use for processing messages
 	 * @return string
 	 */
-	public function getHTML( $shortContext = false, $longContext = false ) {
-		$text = $this->getWikiText( $shortContext, $longContext );
-		$out = MessageCache::singleton()->parse( $text, null, true, true );
-		return $out instanceof ParserOutput ? $out->getText() : $out;
+	public function getHTML( $shortContext = false, $longContext = false, $lang = null ) {
+		$lang = $this->languageFromParam( $lang );
+		$text = $this->getWikiText( $shortContext, $longContext, $lang );
+		$out = MessageCache::singleton()->parse( $text, null, true, true, $lang );
+		return $out instanceof ParserOutput
+			? $out->getText( [ 'enableSectionEditLinks' => false ] )
+			: $out;
 	}
 
 	/**
 	 * Return an array with a Message object for each error.
 	 * @param array $errors
+	 * @param string|Language|null $lang Language to use for processing messages
 	 * @return Message[]
 	 */
-	protected function getErrorMessageArray( $errors ) {
-		return array_map( array( $this, 'getErrorMessage' ), $errors );
-	}
-
-	/**
-	 * Merge another status object into this one
-	 *
-	 * @param Status $other Other Status object
-	 * @param bool $overwriteValue Whether to override the "value" member
-	 */
-	public function merge( $other, $overwriteValue = false ) {
-		$this->sv->merge( $other->sv, $overwriteValue );
+	protected function getErrorMessageArray( $errors, $lang = null ) {
+		$lang = $this->languageFromParam( $lang );
+		return array_map( function ( $e ) use ( $lang ) {
+			return $this->getErrorMessage( $e, $lang );
+		}, $errors );
 	}
 
 	/**
@@ -339,7 +339,7 @@ class Status {
 	 *
 	 * @return array A list in which each entry is an array with a message key as its first element.
 	 *         The remaining array elements are the message parameters.
-	 * @deprecated 1.25
+	 * @deprecated since 1.25
 	 */
 	public function getErrorsArray() {
 		return $this->getStatusArray( 'error' );
@@ -350,7 +350,7 @@ class Status {
 	 *
 	 * @return array A list in which each entry is an array with a message key as its first element.
 	 *         The remaining array elements are the message parameters.
-	 * @deprecated 1.25
+	 * @deprecated since 1.25
 	 */
 	public function getWarningsArray() {
 		return $this->getStatusArray( 'warning' );
@@ -359,25 +359,25 @@ class Status {
 	/**
 	 * Returns a list of status messages of the given type (or all if false)
 	 *
-	 * @note: this handles RawMessage poorly
+	 * @note this handles RawMessage poorly
 	 *
-	 * @param string $type
+	 * @param string|bool $type
 	 * @return array
 	 */
 	protected function getStatusArray( $type = false ) {
-		$result = array();
+		$result = [];
 
-		foreach ( $this->sv->getErrors() as $error ) {
+		foreach ( $this->getErrors() as $error ) {
 			if ( $type === false || $error['type'] === $type ) {
 				if ( $error['message'] instanceof MessageSpecifier ) {
 					$result[] = array_merge(
-						array( $error['message']->getKey() ),
+						[ $error['message']->getKey() ],
 						$error['message']->getParams()
 					);
 				} elseif ( $error['params'] ) {
-					$result[] = array_merge( array( $error['message'] ), $error['params'] );
+					$result[] = array_merge( [ $error['message'] ], $error['params'] );
 				} else {
-					$result[] = array( $error['message'] );
+					$result[] = [ $error['message'] ];
 				}
 			}
 		}
@@ -385,96 +385,13 @@ class Status {
 		return $result;
 	}
 
-	/**
-	 * Returns a list of status messages of the given type, with message and
-	 * params left untouched, like a sane version of getStatusArray
-	 *
-	 * @param string $type
-	 *
-	 * @return array
-	 */
-	public function getErrorsByType( $type ) {
-		return $this->sv->getErrorsByType( $type );
-	}
-
-	/**
-	 * Returns true if the specified message is present as a warning or error
-	 *
-	 * @param string|Message $message Message key or object to search for
-	 *
-	 * @return bool
-	 */
-	public function hasMessage( $message ) {
-		return $this->sv->hasMessage( $message );
-	}
-
-	/**
-	 * If the specified source message exists, replace it with the specified
-	 * destination message, but keep the same parameters as in the original error.
-	 *
-	 * Note, due to the lack of tools for comparing Message objects, this
-	 * function will not work when using a Message object as the search parameter.
-	 *
-	 * @param Message|string $source Message key or object to search for
-	 * @param Message|string $dest Replacement message key or object
-	 * @return bool Return true if the replacement was done, false otherwise.
-	 */
-	public function replaceMessage( $source, $dest ) {
-		return $this->sv->replaceMessage( $source, $dest );
-	}
-
-	/**
-	 * @return mixed
-	 */
-	public function getValue() {
-		return $this->sv->getValue();
-	}
-
-	/**
-	 * Backwards compatibility logic
-	 *
-	 * @param string $name
-	 */
-	function __get( $name ) {
-		if ( $name === 'ok' ) {
-			return $this->sv->isOK();
-		} elseif ( $name === 'errors' ) {
-			return $this->sv->getErrors();
-		}
-		throw new Exception( "Cannot get '$name' property." );
-	}
-
-	/**
-	 * Backwards compatibility logic
-	 *
-	 * @param string $name
-	 * @param mixed $value
-	 */
-	function __set( $name, $value ) {
-		if ( $name === 'ok' ) {
-			$this->sv->setOK( $value );
-		} elseif ( !property_exists( $this, $name ) ) {
-			// Caller is using undeclared ad-hoc properties
-			$this->$name = $value;
-		} else {
-			throw new Exception( "Cannot set '$name' property." );
-		}
-	}
-
-	/**
-	 * @return string
-	 */
-	public function __toString() {
-		return $this->sv->__toString();
-	}
-
 	/**
 	 * Don't save the callback when serializing, because Closures can't be
 	 * serialized and we're going to clear it in __wakeup anyway.
 	 */
 	function __sleep() {
 		$keys = array_keys( get_object_vars( $this ) );
-		return array_diff( $keys, array( 'cleanCallback' ) );
+		return array_diff( $keys, [ 'cleanCallback' ] );
 	}
 
 	/**
