@@ -41,6 +41,7 @@ class RollbackAction extends FormlessAction {
 	 * - confirm-rollback-button
 	 * - rollbackfailed
 	 * - rollback-missingparam
+	 * - rollback-success-notify
 	 */
 
 	/**
@@ -123,8 +124,13 @@ class RollbackAction extends FormlessAction {
 
 		$old = Linker::revUserTools( $current );
 		$new = Linker::revUserTools( $target );
-		$this->getOutput()->addHTML( $this->msg( 'rollback-success' )->rawParams( $old, $new )
-			->parseAsBlock() );
+		$this->getOutput()->addHTML(
+			$this->msg( 'rollback-success' )
+				->rawParams( $old, $new )
+				->params( $current->getUserText( Revision::FOR_THIS_USER, $user ) )
+				->params( $target->getUserText( Revision::FOR_THIS_USER, $user ) )
+				->parseAsBlock()
+		);
 
 		if ( $user->getBoolOption( 'watchrollback' ) ) {
 			$user->addWatch( $this->page->getTitle(), User::IGNORE_USER_RIGHTS );
@@ -145,7 +151,6 @@ class RollbackAction extends FormlessAction {
 			);
 			$de->showDiff( '', '' );
 		}
-		return;
 	}
 
 	protected function getDescription() {
