@@ -1,9 +1,5 @@
 <?php
 /**
- *
- *
- * Created on Oct 13, 2006
- *
  * Copyright © 2006 Yuri Astrakhan "<Firstname><Lastname>@gmail.com"
  *
  * This program is free software; you can redistribute it and/or modify
@@ -148,7 +144,7 @@ class ApiFeedWatchlist extends ApiBase {
 				' [' . $this->getConfig()->get( 'LanguageCode' ) . ']';
 			$feedUrl = SpecialPage::getTitleFor( 'Watchlist' )->getFullURL();
 
-			$feedFormat = isset( $params['feedformat'] ) ? $params['feedformat'] : 'rss';
+			$feedFormat = $params['feedformat'] ?? 'rss';
 			$msg = wfMessage( 'watchlist' )->inContentLanguage()->escaped();
 			$feed = new $feedClasses[$feedFormat] ( $feedTitle, $msg, $feedUrl );
 
@@ -161,12 +157,8 @@ class ApiFeedWatchlist extends ApiBase {
 					$feedItems[] = new FeedItem( $errorTitle, $errorText, '', '', '' );
 				}
 			} else {
-				if ( $e instanceof UsageException ) {
-					$errorCode = $e->getCodeString();
-				} else {
-					// Something is seriously wrong
-					$errorCode = 'internal_api_error';
-				}
+				// Something is seriously wrong
+				$errorCode = 'internal_api_error';
 				$errorTitle = $this->msg( 'api-feed-error-title', $errorCode );
 				$errorText = $e->getMessage();
 				$feedItems[] = new FeedItem( $errorTitle, $errorText, '', '', '' );
@@ -205,7 +197,7 @@ class ApiFeedWatchlist extends ApiBase {
 		} else {
 			$titleUrl = $title->getFullURL( $curidParam );
 		}
-		$comment = isset( $info['comment'] ) ? $info['comment'] : null;
+		$comment = $info['comment'] ?? null;
 
 		// Create an anchor to section.
 		// The anchor won't work for sections that have dupes on page
@@ -215,10 +207,7 @@ class ApiFeedWatchlist extends ApiBase {
 			preg_match( '!(.*)/\*\s*(.*?)\s*\*/(.*)!', $comment, $matches )
 		) {
 			global $wgParser;
-
-			$sectionTitle = $wgParser->stripSectionName( $matches[2] );
-			$sectionTitle = Sanitizer::normalizeSectionNameWhitespace( $sectionTitle );
-			$titleUrl .= Title::newFromText( '#' . $sectionTitle )->getFragmentForURL();
+			$titleUrl .= $wgParser->guessSectionNameFromWikiText( $matches[ 2 ] );
 		}
 
 		$timestamp = $info['timestamp'];
