@@ -1,9 +1,5 @@
 <?php
 /**
- *
- *
- * Created on Sep 7, 2006
- *
  * Copyright © 2006 Yuri Astrakhan "<Firstname><Lastname>@gmail.com"
  *
  * This program is free software; you can redistribute it and/or modify
@@ -44,26 +40,26 @@ class ApiQuery extends ApiBase {
 	 * @var array
 	 */
 	private static $QueryPropModules = [
-		'categories' => 'ApiQueryCategories',
-		'categoryinfo' => 'ApiQueryCategoryInfo',
-		'contributors' => 'ApiQueryContributors',
-		'deletedrevisions' => 'ApiQueryDeletedRevisions',
-		'duplicatefiles' => 'ApiQueryDuplicateFiles',
-		'extlinks' => 'ApiQueryExternalLinks',
-		'fileusage' => 'ApiQueryBacklinksprop',
-		'images' => 'ApiQueryImages',
-		'imageinfo' => 'ApiQueryImageInfo',
-		'info' => 'ApiQueryInfo',
-		'links' => 'ApiQueryLinks',
-		'linkshere' => 'ApiQueryBacklinksprop',
-		'iwlinks' => 'ApiQueryIWLinks',
-		'langlinks' => 'ApiQueryLangLinks',
-		'pageprops' => 'ApiQueryPageProps',
-		'redirects' => 'ApiQueryBacklinksprop',
-		'revisions' => 'ApiQueryRevisions',
-		'stashimageinfo' => 'ApiQueryStashImageInfo',
-		'templates' => 'ApiQueryLinks',
-		'transcludedin' => 'ApiQueryBacklinksprop',
+		'categories' => ApiQueryCategories::class,
+		'categoryinfo' => ApiQueryCategoryInfo::class,
+		'contributors' => ApiQueryContributors::class,
+		'deletedrevisions' => ApiQueryDeletedRevisions::class,
+		'duplicatefiles' => ApiQueryDuplicateFiles::class,
+		'extlinks' => ApiQueryExternalLinks::class,
+		'fileusage' => ApiQueryBacklinksprop::class,
+		'images' => ApiQueryImages::class,
+		'imageinfo' => ApiQueryImageInfo::class,
+		'info' => ApiQueryInfo::class,
+		'links' => ApiQueryLinks::class,
+		'linkshere' => ApiQueryBacklinksprop::class,
+		'iwlinks' => ApiQueryIWLinks::class,
+		'langlinks' => ApiQueryLangLinks::class,
+		'pageprops' => ApiQueryPageProps::class,
+		'redirects' => ApiQueryBacklinksprop::class,
+		'revisions' => ApiQueryRevisions::class,
+		'stashimageinfo' => ApiQueryStashImageInfo::class,
+		'templates' => ApiQueryLinks::class,
+		'transcludedin' => ApiQueryBacklinksprop::class,
 	];
 
 	/**
@@ -71,41 +67,41 @@ class ApiQuery extends ApiBase {
 	 * @var array
 	 */
 	private static $QueryListModules = [
-		'allcategories' => 'ApiQueryAllCategories',
-		'alldeletedrevisions' => 'ApiQueryAllDeletedRevisions',
-		'allfileusages' => 'ApiQueryAllLinks',
-		'allimages' => 'ApiQueryAllImages',
-		'alllinks' => 'ApiQueryAllLinks',
-		'allpages' => 'ApiQueryAllPages',
-		'allredirects' => 'ApiQueryAllLinks',
-		'allrevisions' => 'ApiQueryAllRevisions',
-		'mystashedfiles' => 'ApiQueryMyStashedFiles',
-		'alltransclusions' => 'ApiQueryAllLinks',
-		'allusers' => 'ApiQueryAllUsers',
-		'backlinks' => 'ApiQueryBacklinks',
-		'blocks' => 'ApiQueryBlocks',
-		'categorymembers' => 'ApiQueryCategoryMembers',
-		'deletedrevs' => 'ApiQueryDeletedrevs',
-		'embeddedin' => 'ApiQueryBacklinks',
-		'exturlusage' => 'ApiQueryExtLinksUsage',
-		'filearchive' => 'ApiQueryFilearchive',
-		'imageusage' => 'ApiQueryBacklinks',
-		'iwbacklinks' => 'ApiQueryIWBacklinks',
-		'langbacklinks' => 'ApiQueryLangBacklinks',
-		'logevents' => 'ApiQueryLogEvents',
-		'pageswithprop' => 'ApiQueryPagesWithProp',
-		'pagepropnames' => 'ApiQueryPagePropNames',
-		'prefixsearch' => 'ApiQueryPrefixSearch',
-		'protectedtitles' => 'ApiQueryProtectedTitles',
-		'querypage' => 'ApiQueryQueryPage',
-		'random' => 'ApiQueryRandom',
-		'recentchanges' => 'ApiQueryRecentChanges',
-		'search' => 'ApiQuerySearch',
-		'tags' => 'ApiQueryTags',
-		'usercontribs' => 'ApiQueryContributions',
-		'users' => 'ApiQueryUsers',
-		'watchlist' => 'ApiQueryWatchlist',
-		'watchlistraw' => 'ApiQueryWatchlistRaw',
+		'allcategories' => ApiQueryAllCategories::class,
+		'alldeletedrevisions' => ApiQueryAllDeletedRevisions::class,
+		'allfileusages' => ApiQueryAllLinks::class,
+		'allimages' => ApiQueryAllImages::class,
+		'alllinks' => ApiQueryAllLinks::class,
+		'allpages' => ApiQueryAllPages::class,
+		'allredirects' => ApiQueryAllLinks::class,
+		'allrevisions' => ApiQueryAllRevisions::class,
+		'mystashedfiles' => ApiQueryMyStashedFiles::class,
+		'alltransclusions' => ApiQueryAllLinks::class,
+		'allusers' => ApiQueryAllUsers::class,
+		'backlinks' => ApiQueryBacklinks::class,
+		'blocks' => ApiQueryBlocks::class,
+		'categorymembers' => ApiQueryCategoryMembers::class,
+		'deletedrevs' => ApiQueryDeletedrevs::class,
+		'embeddedin' => ApiQueryBacklinks::class,
+		'exturlusage' => ApiQueryExtLinksUsage::class,
+		'filearchive' => ApiQueryFilearchive::class,
+		'imageusage' => ApiQueryBacklinks::class,
+		'iwbacklinks' => ApiQueryIWBacklinks::class,
+		'langbacklinks' => ApiQueryLangBacklinks::class,
+		'logevents' => ApiQueryLogEvents::class,
+		'pageswithprop' => ApiQueryPagesWithProp::class,
+		'pagepropnames' => ApiQueryPagePropNames::class,
+		'prefixsearch' => ApiQueryPrefixSearch::class,
+		'protectedtitles' => ApiQueryProtectedTitles::class,
+		'querypage' => ApiQueryQueryPage::class,
+		'random' => ApiQueryRandom::class,
+		'recentchanges' => ApiQueryRecentChanges::class,
+		'search' => ApiQuerySearch::class,
+		'tags' => ApiQueryTags::class,
+		'usercontribs' => ApiQueryUserContribs::class,
+		'users' => ApiQueryUsers::class,
+		'watchlist' => ApiQueryWatchlist::class,
+		'watchlistraw' => ApiQueryWatchlistRaw::class,
 	];
 
 	/**
@@ -113,12 +109,12 @@ class ApiQuery extends ApiBase {
 	 * @var array
 	 */
 	private static $QueryMetaModules = [
-		'allmessages' => 'ApiQueryAllMessages',
-		'authmanagerinfo' => 'ApiQueryAuthManagerInfo',
-		'siteinfo' => 'ApiQuerySiteinfo',
-		'userinfo' => 'ApiQueryUserInfo',
-		'filerepoinfo' => 'ApiQueryFileRepoInfo',
-		'tokens' => 'ApiQueryTokens',
+		'allmessages' => ApiQueryAllMessages::class,
+		'authmanagerinfo' => ApiQueryAuthManagerInfo::class,
+		'siteinfo' => ApiQuerySiteinfo::class,
+		'userinfo' => ApiQueryUserInfo::class,
+		'filerepoinfo' => ApiQueryFileRepoInfo::class,
+		'tokens' => ApiQueryTokens::class,
 	];
 
 	/**
