@@ -32,6 +32,11 @@
  */
 class JavaScriptContent extends TextContent {
 
+	/**
+	 * @var bool|Title|null
+	 */
+	private $redirectTarget = false;
+
 	/**
 	 * @param string $text JavaScript code.
 	 * @param string $modelId the content model name
@@ -53,7 +58,7 @@ class JavaScriptContent extends TextContent {
 	public function preSaveTransform( Title $title, User $user, ParserOptions $popts ) {
 		global $wgParser;
 		// @todo Make pre-save transformation optional for script pages
-		// See bug #32858
+		// See T34858
 
 		$text = $this->getNativeData();
 		$pst = $wgParser->preSaveTransform( $text, $title, $user, $popts );
@@ -73,4 +78,46 @@ class JavaScriptContent extends TextContent {
 		return $html;
 	}
 
+	/**
+	 * If this page is a redirect, return the content
+	 * if it should redirect to $target instead
+	 *
+	 * @param Title $target
+	 * @return JavaScriptContent
+	 */
+	public function updateRedirect( Title $target ) {
+		if ( !$this->isRedirect() ) {
+			return $this;
+		}
+
+		return $this->getContentHandler()->makeRedirectContent( $target );
+	}
+
+	/**
+	 * @return Title|null
+	 */
+	public function getRedirectTarget() {
+		if ( $this->redirectTarget !== false ) {
+			return $this->redirectTarget;
+		}
+		$this->redirectTarget = null;
+		$text = $this->getNativeData();
+		if ( strpos( $text, '/* #REDIRECT */' ) === 0 ) {
+			// Extract the title from the url
+			preg_match( '/title=(.*?)\\\\u0026action=raw/', $text, $matches );
+			if ( isset( $matches[1] ) ) {
+				$title = Title::newFromText( urldecode( $matches[1] ) );
+				if ( $title ) {
+					// Have a title, check that the current content equals what
+					// the redirect content should be
+					if ( $this->equals( $this->getContentHandler()->makeRedirectContent( $title ) ) ) {
+						$this->redirectTarget = $title;
+					}
+				}
+			}
+		}
+
+		return $this->redirectTarget;
+	}
+
 }
