@@ -1,6 +1,7 @@
 <?php
 
-use Wikimedia\Rdbms\Database;
+use MediaWiki\MediaWikiServices;
+use Wikimedia\Rdbms\IDatabase;
 use Wikimedia\Rdbms\LoadBalancer;
 
 /**
@@ -27,7 +28,7 @@ use Wikimedia\Rdbms\LoadBalancer;
  * @file
  * @ingroup Database
  *
- * @licence GNU GPL v2+
+ * @license GPL-2.0-or-later
  * @author Daniel Kinzler
  */
 abstract class DBAccessBase implements IDBAccessObject {
@@ -56,10 +57,10 @@ abstract class DBAccessBase implements IDBAccessObject {
 	 * @param int $id Which connection to use
 	 * @param array $groups Query groups
 	 *
-	 * @return Database
+	 * @return IDatabase
 	 */
 	protected function getConnection( $id, $groups = [] ) {
-		$loadBalancer = wfGetLB( $this->wiki );
+		$loadBalancer = $this->getLoadBalancer();
 
 		return $loadBalancer->getConnection( $id, $groups, $this->wiki );
 	}
@@ -71,9 +72,9 @@ abstract class DBAccessBase implements IDBAccessObject {
 	 *
 	 * @since 1.21
 	 *
-	 * @param Database $db The database connection to release.
+	 * @param IDatabase $db The database connection to release.
 	 */
-	protected function releaseConnection( Database $db ) {
+	protected function releaseConnection( IDatabase $db ) {
 		if ( $this->wiki !== false ) {
 			$loadBalancer = $this->getLoadBalancer();
 			$loadBalancer->reuseConnection( $db );
@@ -83,13 +84,14 @@ abstract class DBAccessBase implements IDBAccessObject {
 	/**
 	 * Get the database type used for read operations.
 	 *
-	 * @see wfGetLB
+	 * @see MediaWikiServices::getDBLoadBalancer
 	 *
 	 * @since 1.21
 	 *
 	 * @return LoadBalancer The database load balancer object
 	 */
 	public function getLoadBalancer() {
-		return wfGetLB( $this->wiki );
+		$lbFactory = MediaWikiServices::getInstance()->getDBLoadBalancerFactory();
+		return $lbFactory->getMainLB( $this->wiki );
 	}
 }
