@@ -75,6 +75,23 @@ class HTMLInfoField extends HTMLFormField {
 		return parent::getRaw( $value );
 	}
 
+	/**
+	 * @param mixed $value
+	 * @return OOUI\FieldLayout
+	 * @since 1.32
+	 */
+	public function getOOUI( $value ) {
+		if ( !empty( $this->mParams['rawrow'] ) ) {
+			if ( !( $value instanceof OOUI\FieldLayout ) ) {
+				wfDeprecated( "'default' parameter as a string when using 'rawrow' " .
+					"(must be a FieldLayout or subclass)", '1.32' );
+			}
+			return $value;
+		}
+
+		return parent::getOOUI( $value );
+	}
+
 	protected function needsLabel() {
 		return false;
 	}
