@@ -23,8 +23,7 @@
 
 /**
  * Class to both describe a background job and handle jobs.
- * The queue aspects of this class are now deprecated.
- * Using the class to push jobs onto queues is deprecated (use JobSpecification).
+ * To push jobs onto queues, use JobQueueGroup::singleton()->push();
  *
  * @ingroup JobQueue
  */
@@ -50,6 +49,12 @@ abstract class Job implements IJobSpecification {
 	/** @var callable[] */
 	protected $teardownCallbacks = [];
 
+	/** @var int Bitfield of JOB_* class constants */
+	protected $executionFlags = 0;
+
+	/** @var int Job must not be wrapped in the usual explicit LBFactory transaction round */
+	const JOB_NO_EXPLICIT_TRX_ROUND = 1;
+
 	/**
 	 * Run the job
 	 * @return bool Success
@@ -109,20 +114,12 @@ abstract class Job implements IJobSpecification {
 	}
 
 	/**
-	 * Batch-insert a group of jobs into the queue.
-	 * This will be wrapped in a transaction with a forced commit.
-	 *
-	 * This may add duplicate at insert time, but they will be
-	 * removed later on, when the first one is popped.
-	 *
-	 * @param Job[] $jobs Array of Job objects
+	 * @param int $flag JOB_* class constant
 	 * @return bool
-	 * @deprecated since 1.21
+	 * @since 1.31
 	 */
-	public static function batchInsert( $jobs ) {
-		wfDeprecated( __METHOD__, '1.21' );
-		JobQueueGroup::singleton()->push( $jobs );
-		return true;
+	public function hasExecutionFlag( $flag ) {
+		return ( $this->executionFlags && $flag ) === $flag;
 	}
 
 	/**
@@ -173,9 +170,7 @@ abstract class Job implements IJobSpecification {
 	 * @since 1.27
 	 */
 	public function getRequestId() {
-		return isset( $this->params['requestId'] )
-			? $this->params['requestId']
-			: null;
+		return $this->params['requestId'] ?? null;
 	}
 
 	/**
@@ -283,12 +278,8 @@ abstract class Job implements IJobSpecification {
 	 */
 	public function getRootJobParams() {
 		return [
-			'rootJobSignature' => isset( $this->params['rootJobSignature'] )
-				? $this->params['rootJobSignature']
-				: null,
-			'rootJobTimestamp' => isset( $this->params['rootJobTimestamp'] )
-				? $this->params['rootJobTimestamp']
-				: null
+			'rootJobSignature' => $this->params['rootJobSignature'] ?? null,
+			'rootJobTimestamp' => $this->params['rootJobTimestamp'] ?? null
 		];
 	}
 
@@ -331,16 +322,6 @@ abstract class Job implements IJobSpecification {
 		}
 	}
 
-	/**
-	 * Insert a single job into the queue.
-	 * @return bool True on success
-	 * @deprecated since 1.21
-	 */
-	public function insert() {
-		JobQueueGroup::singleton()->push( $this );
-		return true;
-	}
-
 	/**
 	 * @return string
 	 */
