@@ -27,7 +27,7 @@
 
 namespace Wikimedia\Rdbms;
 
-use MediaWiki;
+use Wikimedia;
 use Exception;
 use stdClass;
 
@@ -35,19 +35,28 @@ use stdClass;
  * @ingroup Database
  */
 class DatabaseMssql extends Database {
-	protected $mPort;
-	protected $mUseWindowsAuth = false;
-
-	protected $mInsertId = null;
-	protected $mLastResult = null;
-	protected $mAffectedRows = null;
-	protected $mSubqueryId = 0;
-	protected $mScrollableCursor = true;
-	protected $mPrepareStatements = true;
-	protected $mBinaryColumnCache = null;
-	protected $mBitColumnCache = null;
-	protected $mIgnoreDupKeyErrors = false;
-	protected $mIgnoreErrors = [];
+	/** @var int */
+	protected $serverPort;
+	/** @var bool */
+	protected $useWindowsAuth = false;
+	/** @var int|null */
+	protected $lastInsertId = null;
+	/** @var int|null */
+	protected $lastAffectedRowCount = null;
+	/** @var int */
+	protected $subqueryId = 0;
+	/** @var bool */
+	protected $scrollableCursor = true;
+	/** @var bool */
+	protected $prepareStatements = true;
+	/** @var stdClass[][]|null */
+	protected $binaryColumnCache = null;
+	/** @var stdClass[][]|null */
+	protected $bitColumnCache = null;
+	/** @var bool */
+	protected $ignoreDupKeyErrors = false;
+	/** @var string[] */
+	protected $ignoreErrors = [];
 
 	public function implicitGroupby() {
 		return false;
@@ -62,22 +71,13 @@ class DatabaseMssql extends Database {
 	}
 
 	public function __construct( array $params ) {
-		$this->mPort = $params['port'];
-		$this->mUseWindowsAuth = $params['UseWindowsAuth'];
+		$this->serverPort = $params['port'];
+		$this->useWindowsAuth = $params['UseWindowsAuth'];
 
 		parent::__construct( $params );
 	}
 
-	/**
-	 * Usually aborts on failure
-	 * @param string $server
-	 * @param string $user
-	 * @param string $password
-	 * @param string $dbName
-	 * @throws DBConnectionError
-	 * @return bool|resource|null
-	 */
-	public function open( $server, $user, $password, $dbName ) {
+	protected function open( $server, $user, $password, $dbName, $schema, $tablePrefix ) {
 		# Test for driver support, to avoid suppressed fatal error
 		if ( !function_exists( 'sqlsrv_connect' ) ) {
 			throw new DBConnectionError(
@@ -93,35 +93,39 @@ class DatabaseMssql extends Database {
 		}
 
 		$this->close();
-		$this->mServer = $server;
-		$this->mUser = $user;
-		$this->mPassword = $password;
-		$this->mDBname = $dbName;
+		$this->server = $server;
+		$this->user = $user;
+		$this->password = $password;
 
 		$connectionInfo = [];
 
-		if ( $dbName ) {
+		if ( $dbName != '' ) {
 			$connectionInfo['Database'] = $dbName;
 		}
 
 		// Decide which auth scenerio to use
 		// if we are using Windows auth, then don't add credentials to $connectionInfo
-		if ( !$this->mUseWindowsAuth ) {
+		if ( !$this->useWindowsAuth ) {
 			$connectionInfo['UID'] = $user;
 			$connectionInfo['PWD'] = $password;
 		}
 
-		MediaWiki\suppressWarnings();
-		$this->mConn = sqlsrv_connect( $server, $connectionInfo );
-		MediaWiki\restoreWarnings();
+		Wikimedia\suppressWarnings();
+		$this->conn = sqlsrv_connect( $server, $connectionInfo );
+		Wikimedia\restoreWarnings();
 
-		if ( $this->mConn === false ) {
+		if ( $this->conn === false ) {
 			throw new DBConnectionError( $this, $this->lastError() );
 		}
 
-		$this->mOpened = true;
+		$this->opened = true;
+		$this->currentDomain = new DatabaseDomain(
+			( $dbName != '' ) ? $dbName : null,
+			null,
+			$tablePrefix
+		);
 
-		return $this->mConn;
+		return (bool)$this->conn;
 	}
 
 	/**
@@ -130,7 +134,7 @@ class DatabaseMssql extends Database {
 	 * @return bool
 	 */
 	protected function closeConnection() {
-		return sqlsrv_close( $this->mConn );
+		return sqlsrv_close( $this->conn );
 	}
 
 	/**
@@ -178,25 +182,25 @@ class DatabaseMssql extends Database {
 		// needed if we want to be able to seek around the result set), however CLIENT_BUFFERED
 		// has a bug in the sqlsrv driver where wchar_t types (such as nvarchar) that are empty
 		// strings make php throw a fatal error "Severe error translating Unicode"
-		if ( $this->mScrollableCursor ) {
+		if ( $this->scrollableCursor ) {
 			$scrollArr = [ 'Scrollable' => SQLSRV_CURSOR_STATIC ];
 		} else {
 			$scrollArr = [];
 		}
 
-		if ( $this->mPrepareStatements ) {
+		if ( $this->prepareStatements ) {
 			// we do prepare + execute so we can get its field metadata for later usage if desired
-			$stmt = sqlsrv_prepare( $this->mConn, $sql, [], $scrollArr );
+			$stmt = sqlsrv_prepare( $this->conn, $sql, [], $scrollArr );
 			$success = sqlsrv_execute( $stmt );
 		} else {
-			$stmt = sqlsrv_query( $this->mConn, $sql, [], $scrollArr );
+			$stmt = sqlsrv_query( $this->conn, $sql, [], $scrollArr );
 			$success = (bool)$stmt;
 		}
 
 		// Make a copy to ensure what we add below does not get reflected in future queries
-		$ignoreErrors = $this->mIgnoreErrors;
+		$ignoreErrors = $this->ignoreErrors;
 
-		if ( $this->mIgnoreDupKeyErrors ) {
+		if ( $this->ignoreDupKeyErrors ) {
 			// ignore duplicate key errors
 			// this emulates INSERT IGNORE in MySQL
 			$ignoreErrors[] = '2601'; // duplicate key error caused by unique index
@@ -220,7 +224,7 @@ class DatabaseMssql extends Database {
 			}
 		}
 		// remember number of rows affected
-		$this->mAffectedRows = sqlsrv_rows_affected( $stmt );
+		$this->lastAffectedRowCount = sqlsrv_rows_affected( $stmt );
 
 		return $stmt;
 	}
@@ -234,7 +238,7 @@ class DatabaseMssql extends Database {
 	}
 
 	/**
-	 * @param MssqlResultWrapper $res
+	 * @param IResultWrapper $res
 	 * @return stdClass
 	 */
 	public function fetchObject( $res ) {
@@ -243,7 +247,7 @@ class DatabaseMssql extends Database {
 	}
 
 	/**
-	 * @param MssqlResultWrapper $res
+	 * @param IResultWrapper $res
 	 * @return array
 	 */
 	public function fetchRow( $res ) {
@@ -300,7 +304,7 @@ class DatabaseMssql extends Database {
 	 * @return int|null
 	 */
 	public function insertId() {
-		return $this->mInsertId;
+		return $this->lastInsertId;
 	}
 
 	/**
@@ -350,11 +354,33 @@ class DatabaseMssql extends Database {
 		}
 	}
 
+	protected function wasKnownStatementRollbackError() {
+		$errors = sqlsrv_errors( SQLSRV_ERR_ALL );
+		if ( !$errors ) {
+			return false;
+		}
+		// The transaction vs statement rollback behavior depends on XACT_ABORT, so make sure
+		// that the "statement has been terminated" error (3621) is specifically present.
+		// https://docs.microsoft.com/en-us/sql/t-sql/statements/set-xact-abort-transact-sql
+		$statementOnly = false;
+		$codeWhitelist = [ '2601', '2627', '547' ];
+		foreach ( $errors as $error ) {
+			if ( $error['code'] == '3621' ) {
+				$statementOnly = true;
+			} elseif ( !in_array( $error['code'], $codeWhitelist ) ) {
+				$statementOnly = false;
+				break;
+			}
+		}
+
+		return $statementOnly;
+	}
+
 	/**
 	 * @return int
 	 */
-	public function affectedRows() {
-		return $this->mAffectedRows;
+	protected function fetchAffectedRowCount() {
+		return $this->lastAffectedRowCount;
 	}
 
 	/**
@@ -381,8 +407,8 @@ class DatabaseMssql extends Database {
 		$sql = $this->selectSQLText( $table, $vars, $conds, $fname, $options, $join_conds );
 		if ( isset( $options['EXPLAIN'] ) ) {
 			try {
-				$this->mScrollableCursor = false;
-				$this->mPrepareStatements = false;
+				$this->scrollableCursor = false;
+				$this->prepareStatements = false;
 				$this->query( "SET SHOWPLAN_ALL ON" );
 				$ret = $this->query( $sql, $fname );
 				$this->query( "SET SHOWPLAN_ALL OFF" );
@@ -402,13 +428,13 @@ class DatabaseMssql extends Database {
 				} else {
 					// someone actually wanted the query plan instead of an est row count
 					// let them know of the error
-					$this->mScrollableCursor = true;
-					$this->mPrepareStatements = true;
+					$this->scrollableCursor = true;
+					$this->prepareStatements = true;
 					throw $dqe;
 				}
 			}
-			$this->mScrollableCursor = true;
-			$this->mPrepareStatements = true;
+			$this->scrollableCursor = true;
+			$this->prepareStatements = true;
 			return $ret;
 		}
 		return $this->query( $sql, $fname );
@@ -440,8 +466,14 @@ class DatabaseMssql extends Database {
 		if ( strpos( $sql, 'MAX(' ) !== false || strpos( $sql, 'MIN(' ) !== false ) {
 			$bitColumns = [];
 			if ( is_array( $table ) ) {
-				foreach ( $table as $t ) {
-					$bitColumns += $this->getBitColumns( $this->tableName( $t ) );
+				$tables = $table;
+				while ( $tables ) {
+					$t = array_pop( $tables );
+					if ( is_array( $t ) ) {
+						$tables = array_merge( $tables, $t );
+					} else {
+						$bitColumns += $this->getBitColumns( $this->tableName( $t ) );
+					}
 				}
 			} else {
 				$bitColumns = $this->getBitColumns( $this->tableName( $table ) );
@@ -462,25 +494,25 @@ class DatabaseMssql extends Database {
 	public function deleteJoin( $delTable, $joinTable, $delVar, $joinVar, $conds,
 		$fname = __METHOD__
 	) {
-		$this->mScrollableCursor = false;
+		$this->scrollableCursor = false;
 		try {
 			parent::deleteJoin( $delTable, $joinTable, $delVar, $joinVar, $conds, $fname );
 		} catch ( Exception $e ) {
-			$this->mScrollableCursor = true;
+			$this->scrollableCursor = true;
 			throw $e;
 		}
-		$this->mScrollableCursor = true;
+		$this->scrollableCursor = true;
 	}
 
 	public function delete( $table, $conds, $fname = __METHOD__ ) {
-		$this->mScrollableCursor = false;
+		$this->scrollableCursor = false;
 		try {
 			parent::delete( $table, $conds, $fname );
 		} catch ( Exception $e ) {
-			$this->mScrollableCursor = true;
+			$this->scrollableCursor = true;
 			throw $e;
 		}
-		$this->mScrollableCursor = true;
+		$this->scrollableCursor = true;
 	}
 
 	/**
@@ -490,19 +522,26 @@ class DatabaseMssql extends Database {
 	 * Returns -1 if count cannot be found
 	 * Takes same arguments as Database::select()
 	 * @param string $table
-	 * @param string $vars
+	 * @param string $var
 	 * @param string $conds
 	 * @param string $fname
 	 * @param array $options
+	 * @param array $join_conds
 	 * @return int
 	 */
-	public function estimateRowCount( $table, $vars = '*', $conds = '',
-		$fname = __METHOD__, $options = []
+	public function estimateRowCount( $table, $var = '*', $conds = '',
+		$fname = __METHOD__, $options = [], $join_conds = []
 	) {
+		$conds = $this->normalizeConditions( $conds, $fname );
+		$column = $this->extractSingleFieldFromList( $var );
+		if ( is_string( $column ) && !in_array( $column, [ '*', '1' ] ) ) {
+			$conds[] = "$column IS NOT NULL";
+		}
+
 		// http://msdn2.microsoft.com/en-us/library/aa259203.aspx
 		$options['EXPLAIN'] = true;
 		$options['FOR COUNT'] = true;
-		$res = $this->select( $table, $vars, $conds, $fname, $options );
+		$res = $this->select( $table, $var, $conds, $fname, $options, $join_conds );
 
 		$rows = -1;
 		if ( $res ) {
@@ -554,7 +593,7 @@ class DatabaseMssql extends Database {
 			}
 		}
 
-		return empty( $result ) ? false : $result;
+		return $result ?: false;
 	}
 
 	/**
@@ -611,7 +650,7 @@ class DatabaseMssql extends Database {
 		// remove IGNORE from options list and set ignore flag to true
 		if ( in_array( 'IGNORE', $options ) ) {
 			$options = array_diff( $options, [ 'IGNORE' ] );
-			$this->mIgnoreDupKeyErrors = true;
+			$this->ignoreDupKeyErrors = true;
 		}
 
 		$ret = null;
@@ -675,32 +714,32 @@ class DatabaseMssql extends Database {
 			$sql .= ')' . $sqlPost;
 
 			// Run the query
-			$this->mScrollableCursor = false;
+			$this->scrollableCursor = false;
 			try {
 				$ret = $this->query( $sql );
 			} catch ( Exception $e ) {
-				$this->mScrollableCursor = true;
-				$this->mIgnoreDupKeyErrors = false;
+				$this->scrollableCursor = true;
+				$this->ignoreDupKeyErrors = false;
 				throw $e;
 			}
-			$this->mScrollableCursor = true;
+			$this->scrollableCursor = true;
 
 			if ( $ret instanceof ResultWrapper && !is_null( $identity ) ) {
 				// Then we want to get the identity column value we were assigned and save it off
 				$row = $ret->fetchObject();
 				if ( is_object( $row ) ) {
-					$this->mInsertId = $row->$identity;
+					$this->lastInsertId = $row->$identity;
 					// It seems that mAffectedRows is -1 sometimes when OUTPUT INSERTED.identity is
 					// used if we got an identity back, we know for sure a row was affected, so
 					// adjust that here
-					if ( $this->mAffectedRows == -1 ) {
-						$this->mAffectedRows = 1;
+					if ( $this->lastAffectedRowCount == -1 ) {
+						$this->lastAffectedRowCount = 1;
 					}
 				}
 			}
 		}
 
-		$this->mIgnoreDupKeyErrors = false;
+		$this->ignoreDupKeyErrors = false;
 
 		return $ret;
 	}
@@ -718,13 +757,13 @@ class DatabaseMssql extends Database {
 	 * @param array $insertOptions
 	 * @param array $selectOptions
 	 * @param array $selectJoinConds
-	 * @return null|ResultWrapper
+	 * @return bool
 	 * @throws Exception
 	 */
 	public function nativeInsertSelect( $destTable, $srcTable, $varMap, $conds, $fname = __METHOD__,
 		$insertOptions = [], $selectOptions = [], $selectJoinConds = []
 	) {
-		$this->mScrollableCursor = false;
+		$this->scrollableCursor = false;
 		try {
 			$ret = parent::nativeInsertSelect(
 				$destTable,
@@ -737,10 +776,10 @@ class DatabaseMssql extends Database {
 				$selectJoinConds
 			);
 		} catch ( Exception $e ) {
-			$this->mScrollableCursor = true;
+			$this->scrollableCursor = true;
 			throw $e;
 		}
-		$this->mScrollableCursor = true;
+		$this->scrollableCursor = true;
 
 		return $ret;
 	}
@@ -781,14 +820,14 @@ class DatabaseMssql extends Database {
 			$sql .= " WHERE " . $this->makeList( $conds, LIST_AND, $binaryColumns );
 		}
 
-		$this->mScrollableCursor = false;
+		$this->scrollableCursor = false;
 		try {
 			$this->query( $sql );
 		} catch ( Exception $e ) {
-			$this->mScrollableCursor = true;
+			$this->scrollableCursor = true;
 			throw $e;
 		}
-		$this->mScrollableCursor = true;
+		$this->scrollableCursor = true;
 		return true;
 	}
 
@@ -881,9 +920,9 @@ class DatabaseMssql extends Database {
 			$postOrder = '';
 			$first = $offset + 1;
 			$last = $offset + $limit;
-			$sub1 = 'sub_' . $this->mSubqueryId;
-			$sub2 = 'sub_' . ( $this->mSubqueryId + 1 );
-			$this->mSubqueryId += 2;
+			$sub1 = 'sub_' . $this->subqueryId;
+			$sub2 = 'sub_' . ( $this->subqueryId + 1 );
+			$this->subqueryId += 2;
 			if ( !$s1 ) {
 				// wat
 				throw new DBUnexpectedError( $this, "Attempting to LIMIT a non-SELECT query\n" );
@@ -947,7 +986,7 @@ class DatabaseMssql extends Database {
 	 * @return string Version information from the database
 	 */
 	public function getServerVersion() {
-		$server_info = sqlsrv_server_info( $this->mConn );
+		$server_info = sqlsrv_server_info( $this->conn );
 		$version = 'Error';
 		if ( isset( $server_info['SQLServerVersion'] ) ) {
 			$version = $server_info['SQLServerVersion'];
@@ -971,7 +1010,7 @@ class DatabaseMssql extends Database {
 		}
 
 		if ( $schema === false ) {
-			$schema = $this->mSchema;
+			$schema = $this->dbSchema();
 		}
 
 		$res = $this->query( "SELECT 1 FROM INFORMATION_SCHEMA.TABLES
@@ -1031,13 +1070,26 @@ class DatabaseMssql extends Database {
 		return false;
 	}
 
+	protected function doSavepoint( $identifier, $fname ) {
+		$this->query( 'SAVE TRANSACTION ' . $this->addIdentifierQuotes( $identifier ), $fname );
+	}
+
+	protected function doReleaseSavepoint( $identifier, $fname ) {
+		// Not supported. Also not really needed, a new doSavepoint() for the
+		// same identifier will overwrite the old.
+	}
+
+	protected function doRollbackToSavepoint( $identifier, $fname ) {
+		$this->query( 'ROLLBACK TRANSACTION ' . $this->addIdentifierQuotes( $identifier ), $fname );
+	}
+
 	/**
 	 * Begin a transaction, committing any previously open transaction
 	 * @param string $fname
 	 */
 	protected function doBegin( $fname = __METHOD__ ) {
-		sqlsrv_begin_transaction( $this->mConn );
-		$this->mTrxLevel = 1;
+		sqlsrv_begin_transaction( $this->conn );
+		$this->trxLevel = 1;
 	}
 
 	/**
@@ -1045,8 +1097,8 @@ class DatabaseMssql extends Database {
 	 * @param string $fname
 	 */
 	protected function doCommit( $fname = __METHOD__ ) {
-		sqlsrv_commit( $this->mConn );
-		$this->mTrxLevel = 0;
+		sqlsrv_commit( $this->conn );
+		$this->trxLevel = 0;
 	}
 
 	/**
@@ -1055,8 +1107,8 @@ class DatabaseMssql extends Database {
 	 * @param string $fname
 	 */
 	protected function doRollback( $fname = __METHOD__ ) {
-		sqlsrv_rollback( $this->mConn );
-		$this->mTrxLevel = 0;
+		sqlsrv_rollback( $this->conn );
+		$this->trxLevel = 0;
 	}
 
 	/**
@@ -1119,18 +1171,13 @@ class DatabaseMssql extends Database {
 			$s );
 	}
 
-	/**
-	 * @param string $db
-	 * @return bool
-	 */
-	public function selectDB( $db ) {
-		try {
-			$this->mDBname = $db;
-			$this->query( "USE $db" );
-			return true;
-		} catch ( Exception $e ) {
-			return false;
-		}
+	protected function doSelectDomain( DatabaseDomain $domain ) {
+		$encDatabase = $this->addIdentifierQuotes( $domain->getDatabase() );
+		$this->query( "USE $encDatabase" );
+		// Update that domain fields on success (no exception thrown)
+		$this->currentDomain = $domain;
+
+		return true;
 	}
 
 	/**
@@ -1198,8 +1245,8 @@ class DatabaseMssql extends Database {
 	public function buildGroupConcatField( $delim, $table, $field, $conds = '',
 		$join_conds = []
 	) {
-		$gcsq = 'gcsq_' . $this->mSubqueryId;
-		$this->mSubqueryId++;
+		$gcsq = 'gcsq_' . $this->subqueryId;
+		$this->subqueryId++;
 
 		$delimLen = strlen( $delim );
 		$fld = "{$field} + {$this->addQuotes( $delim )}";
@@ -1210,6 +1257,19 @@ class DatabaseMssql extends Database {
 		return $sql;
 	}
 
+	public function buildSubstring( $input, $startPosition, $length = null ) {
+		$this->assertBuildSubstringParams( $startPosition, $length );
+		if ( $length === null ) {
+			/**
+			 * MSSQL doesn't allow an empty length parameter, so when we don't want to limit the
+			 * length returned use the default maximum size of text.
+			 * @see https://docs.microsoft.com/en-us/sql/t-sql/statements/set-textsize-transact-sql
+			 */
+			$length = 2147483647;
+		}
+		return 'SUBSTRING(' . implode( ',', [ $input, $startPosition, $length ] ) . ')';
+	}
+
 	/**
 	 * Returns an associative array for fields that are of type varbinary, binary, or image
 	 * $table can be either a raw table name or passed through tableName() first
@@ -1220,13 +1280,11 @@ class DatabaseMssql extends Database {
 		$tableRawArr = explode( '.', preg_replace( '#\[([^\]]*)\]#', '$1', $table ) );
 		$tableRaw = array_pop( $tableRawArr );
 
-		if ( $this->mBinaryColumnCache === null ) {
+		if ( $this->binaryColumnCache === null ) {
 			$this->populateColumnCaches();
 		}
 
-		return isset( $this->mBinaryColumnCache[$tableRaw] )
-			? $this->mBinaryColumnCache[$tableRaw]
-			: [];
+		return $this->binaryColumnCache[$tableRaw] ?? [];
 	}
 
 	/**
@@ -1237,30 +1295,28 @@ class DatabaseMssql extends Database {
 		$tableRawArr = explode( '.', preg_replace( '#\[([^\]]*)\]#', '$1', $table ) );
 		$tableRaw = array_pop( $tableRawArr );
 
-		if ( $this->mBitColumnCache === null ) {
+		if ( $this->bitColumnCache === null ) {
 			$this->populateColumnCaches();
 		}
 
-		return isset( $this->mBitColumnCache[$tableRaw] )
-			? $this->mBitColumnCache[$tableRaw]
-			: [];
+		return $this->bitColumnCache[$tableRaw] ?? [];
 	}
 
 	private function populateColumnCaches() {
 		$res = $this->select( 'INFORMATION_SCHEMA.COLUMNS', '*',
 			[
-				'TABLE_CATALOG' => $this->mDBname,
-				'TABLE_SCHEMA' => $this->mSchema,
+				'TABLE_CATALOG' => $this->getDBname(),
+				'TABLE_SCHEMA' => $this->dbSchema(),
 				'DATA_TYPE' => [ 'varbinary', 'binary', 'image', 'bit' ]
 			] );
 
-		$this->mBinaryColumnCache = [];
-		$this->mBitColumnCache = [];
+		$this->binaryColumnCache = [];
+		$this->bitColumnCache = [];
 		foreach ( $res as $row ) {
 			if ( $row->DATA_TYPE == 'bit' ) {
-				$this->mBitColumnCache[$row->TABLE_NAME][$row->COLUMN_NAME] = $row;
+				$this->bitColumnCache[$row->TABLE_NAME][$row->COLUMN_NAME] = $row;
 			} else {
-				$this->mBinaryColumnCache[$row->TABLE_NAME][$row->COLUMN_NAME] = $row;
+				$this->binaryColumnCache[$row->TABLE_NAME][$row->COLUMN_NAME] = $row;
 			}
 		}
 	}
@@ -1324,9 +1380,9 @@ class DatabaseMssql extends Database {
 	 * @return bool|null
 	 */
 	public function prepareStatements( $value = null ) {
-		$old = $this->mPrepareStatements;
+		$old = $this->prepareStatements;
 		if ( $value !== null ) {
-			$this->mPrepareStatements = $value;
+			$this->prepareStatements = $value;
 		}
 
 		return $old;
@@ -1339,13 +1395,16 @@ class DatabaseMssql extends Database {
 	 * @return bool|null
 	 */
 	public function scrollableCursor( $value = null ) {
-		$old = $this->mScrollableCursor;
+		$old = $this->scrollableCursor;
 		if ( $value !== null ) {
-			$this->mScrollableCursor = $value;
+			$this->scrollableCursor = $value;
 		}
 
 		return $old;
 	}
 }
 
+/**
+ * @deprecated since 1.29
+ */
 class_alias( DatabaseMssql::class, 'DatabaseMssql' );
