@@ -56,7 +56,11 @@ class LBFactorySingle extends LBFactory {
 	 * @since 1.28
 	 */
 	public static function newFromConnection( IDatabase $db, array $params = [] ) {
-		return new static( [ 'connection' => $db ] + $params );
+		return new static( array_merge(
+			[ 'localDomain' => $db->getDomainID() ],
+			$params,
+			[ 'connection' => $db ]
+		) );
 	}
 
 	/**
@@ -102,6 +106,8 @@ class LBFactorySingle extends LBFactory {
 	 * @param array $params
 	 */
 	public function forEachLB( $callback, array $params = [] ) {
-		call_user_func_array( $callback, array_merge( [ $this->lb ], $params ) );
+		if ( isset( $this->lb ) ) { // may not be set during _destruct()
+			$callback( $this->lb, ...$params );
+		}
 	}
 }
