@@ -22,6 +22,7 @@
  */
 
 use MediaWiki\Logger\LoggerFactory;
+use Wikimedia\XMPReader\Reader as XMPReader;
 
 /**
  * Class to deal with reconciling and extracting metadata from bitmap images.
@@ -122,7 +123,7 @@ class BitmapMetadataHandler {
 	 */
 	function getMetadataArray() {
 		// this seems a bit ugly... This is all so its merged in right order
-		// based on the MWG recomendation.
+		// based on the MWG recommendation.
 		$temp = [];
 		krsort( $this->metaPriority );
 		foreach ( $this->metaPriority as $pri ) {
@@ -170,7 +171,7 @@ class BitmapMetadataHandler {
 			}
 		}
 		if ( isset( $seg['XMP'] ) && $showXMP ) {
-			$xmp = new XMPReader( LoggerFactory::getInstance( 'XMP' ) );
+			$xmp = new XMPReader( LoggerFactory::getInstance( 'XMP' ), $filename );
 			$xmp->parse( $seg['XMP'] );
 			foreach ( $seg['XMP_ext'] as $xmpExt ) {
 				/* Support for extended xmp in jpeg files
@@ -184,7 +185,7 @@ class BitmapMetadataHandler {
 			}
 		}
 
-		$meta->getExif( $filename, isset( $seg['byteOrder'] ) ? $seg['byteOrder'] : 'BE' );
+		$meta->getExif( $filename, $seg['byteOrder'] ?? 'BE' );
 
 		return $meta->getMetadataArray();
 	}
@@ -205,7 +206,7 @@ class BitmapMetadataHandler {
 		if ( isset( $array['text']['xmp']['x-default'] )
 			&& $array['text']['xmp']['x-default'] !== '' && $showXMP
 		) {
-			$xmp = new XMPReader( LoggerFactory::getInstance( 'XMP' ) );
+			$xmp = new XMPReader( LoggerFactory::getInstance( 'XMP' ), $filename );
 			$xmp->parse( $array['text']['xmp']['x-default'] );
 			$xmpRes = $xmp->getResults();
 			foreach ( $xmpRes as $type => $xmpSection ) {
@@ -238,7 +239,7 @@ class BitmapMetadataHandler {
 		}
 
 		if ( $baseArray['xmp'] !== '' && XMPReader::isSupported() ) {
-			$xmp = new XMPReader( LoggerFactory::getInstance( 'XMP' ) );
+			$xmp = new XMPReader( LoggerFactory::getInstance( 'XMP' ), $filename );
 			$xmp->parse( $baseArray['xmp'] );
 			$xmpRes = $xmp->getResults();
 			foreach ( $xmpRes as $type => $xmpSection ) {
@@ -292,7 +293,7 @@ class BitmapMetadataHandler {
 	 * Read the first 2 bytes of a tiff file to figure out
 	 * Little Endian or Big Endian. Needed for exif stuff.
 	 *
-	 * @param string $filename The filename
+	 * @param string $filename
 	 * @return string 'BE' or 'LE' or false
 	 */
 	static function getTiffByteOrder( $filename ) {
