@@ -127,7 +127,7 @@ abstract class TransformationalImageHandler extends ImageHandler {
 			'mimeType' => $image->getMimeType(),
 			'dstPath' => $dstPath,
 			'dstUrl' => $dstUrl,
-			'interlace' => isset( $params['interlace'] ) ? $params['interlace'] : false,
+			'interlace' => $params['interlace'] ?? false,
 		];
 
 		if ( isset( $params['quality'] ) && $params['quality'] === 'low' ) {
@@ -512,7 +512,7 @@ abstract class TransformationalImageHandler extends ImageHandler {
 		$cache = MediaWikiServices::getInstance()->getLocalServerObjectCache();
 		$method = __METHOD__;
 		return $cache->getWithSetCallback(
-			'imagemagick-version',
+			$cache->makeGlobalKey( 'imagemagick-version' ),
 			$cache::TTL_HOUR,
 			function () use ( $method ) {
 				global $wgImageMagickConvertCommand;
