@@ -20,6 +20,8 @@
  * @file
  * @ingroup Parser
  */
+
+use MediaWiki\MediaWikiServices;
 use Wikimedia\ScopedCallback;
 
 /**
@@ -39,6 +41,13 @@ use Wikimedia\ScopedCallback;
  */
 class ParserOptions {
 
+	/**
+	 * Flag indicating that newCanonical() accepts an IContextSource or the string 'canonical', for
+	 * back-compat checks from extensions.
+	 * @since 1.32
+	 */
+	const HAS_NEWCANONICAL_FROM_CONTEXT = 1;
+
 	/**
 	 * Default values for all options that are relevant for caching.
 	 * @see self::getDefaults()
@@ -52,6 +61,7 @@ class ParserOptions {
 	 */
 	private static $lazyOptions = [
 		'dateformat' => [ __CLASS__, 'initDateFormat' ],
+		'speculativeRevId' => [ __CLASS__, 'initSpeculativeRevId' ],
 	];
 
 	/**
@@ -657,7 +667,7 @@ class ParserOptions {
 	/**
 	 * Get the user language used by the parser for this page and split the parser cache.
 	 *
-	 * @warning: Calling this causes the parser cache to be fragmented by user language!
+	 * @warning Calling this causes the parser cache to be fragmented by user language!
 	 * To avoid cache fragmentation, output should not depend on the user language.
 	 * Use Parser::getFunctionLang() or Parser::getTargetLanguage() instead!
 	 *
@@ -678,7 +688,7 @@ class ParserOptions {
 	/**
 	 * Same as getUserLangObj() but returns a string instead.
 	 *
-	 * @warning: Calling this causes the parser cache to be fragmented by user language!
+	 * @warning Calling this causes the parser cache to be fragmented by user language!
 	 * To avoid cache fragmentation, output should not depend on the user language.
 	 * Use Parser::getFunctionLang() or Parser::getTargetLanguage() instead!
 	 *
@@ -822,9 +832,38 @@ class ParserOptions {
 		return $this->setOptionLegacy( 'templateCallback', $x );
 	}
 
+	/**
+	 * A guess for {{REVISIONID}}, calculated using the callback provided via
+	 * setSpeculativeRevIdCallback(). For consistency, the value will be calculated upon the
+	 * first call of this method, and re-used for subsequent calls.
+	 *
+	 * If no callback was defined via setSpeculativeRevIdCallback(), this method will return false.
+	 *
+	 * @since 1.32
+	 * @return int|false
+	 */
+	public function getSpeculativeRevId() {
+		return $this->getOption( 'speculativeRevId' );
+	}
+
+	/**
+	 * Callback registered with ParserOptions::$lazyOptions, triggered by getSpeculativeRevId().
+	 *
+	 * @param ParserOptions $popt
+	 * @return bool|false
+	 */
+	private static function initSpeculativeRevId( ParserOptions $popt ) {
+		$cb = $popt->getOption( 'speculativeRevIdCallback' );
+		$id = $cb ? $cb() : null;
+
+		// returning null would result in this being re-called every access
+		return $id ?? false;
+	}
+
 	/**
 	 * Callback to generate a guess for {{REVISIONID}}
 	 * @since 1.28
+	 * @deprecated since 1.32, use getSpeculativeRevId() instead!
 	 * @return callable|null
 	 */
 	public function getSpeculativeRevIdCallback() {
@@ -838,6 +877,7 @@ class ParserOptions {
 	 * @return callable|null Old value
 	 */
 	public function setSpeculativeRevIdCallback( $x ) {
+		$this->setOption( 'speculativeRevId', null ); // reset
 		return $this->setOptionLegacy( 'speculativeRevIdCallback', $x );
 	}
 
@@ -928,10 +968,9 @@ class ParserOptions {
 
 	/**
 	 * @warning For interaction with the parser cache, use
-	 *  WikiPage::makeParserOptions(), ContentHandler::makeParserOptions(), or
-	 *  ParserOptions::newCanonical() instead.
-	 * @param User $user
-	 * @param Language $lang
+	 *  WikiPage::makeParserOptions() or ParserOptions::newCanonical() instead.
+	 * @param User|null $user
+	 * @param Language|null $lang
 	 */
 	public function __construct( $user = null, $lang = null ) {
 		if ( $user === null ) {
@@ -955,14 +994,13 @@ class ParserOptions {
 	/**
 	 * Get a ParserOptions object for an anonymous user
 	 * @warning For interaction with the parser cache, use
-	 *  WikiPage::makeParserOptions(), ContentHandler::makeParserOptions(), or
-	 *  ParserOptions::newCanonical() instead.
+	 *  WikiPage::makeParserOptions() or ParserOptions::newCanonical() instead.
 	 * @since 1.27
 	 * @return ParserOptions
 	 */
 	public static function newFromAnon() {
-		global $wgContLang;
-		return new ParserOptions( new User, $wgContLang );
+		return new ParserOptions( new User,
+			MediaWikiServices::getInstance()->getContentLanguage() );
 	}
 
 	/**
@@ -970,8 +1008,7 @@ class ParserOptions {
 	 * Language will be taken from $wgLang.
 	 *
 	 * @warning For interaction with the parser cache, use
-	 *  WikiPage::makeParserOptions(), ContentHandler::makeParserOptions(), or
-	 *  ParserOptions::newCanonical() instead.
+	 *  WikiPage::makeParserOptions() or ParserOptions::newCanonical() instead.
 	 * @param User $user
 	 * @return ParserOptions
 	 */
@@ -983,8 +1020,7 @@ class ParserOptions {
 	 * Get a ParserOptions object from a given user and language
 	 *
 	 * @warning For interaction with the parser cache, use
-	 *  WikiPage::makeParserOptions(), ContentHandler::makeParserOptions(), or
-	 *  ParserOptions::newCanonical() instead.
+	 *  WikiPage::makeParserOptions() or ParserOptions::newCanonical() instead.
 	 * @param User $user
 	 * @param Language $lang
 	 * @return ParserOptions
@@ -997,8 +1033,7 @@ class ParserOptions {
 	 * Get a ParserOptions object from a IContextSource object
 	 *
 	 * @warning For interaction with the parser cache, use
-	 *  WikiPage::makeParserOptions(), ContentHandler::makeParserOptions(), or
-	 *  ParserOptions::newCanonical() instead.
+	 *  WikiPage::makeParserOptions() or ParserOptions::newCanonical() instead.
 	 * @param IContextSource $context
 	 * @return ParserOptions
 	 */
@@ -1013,12 +1048,29 @@ class ParserOptions {
 	 * different from the canonical values used for caching.
 	 *
 	 * @since 1.30
-	 * @param User|null $user
-	 * @param Language|StubObject|null $lang
+	 * @since 1.32 Added string and IContextSource as options for the first parameter
+	 * @param IContextSource|string|User|null $context
+	 *  - If an IContextSource, the options are initialized based on the source's User and Language.
+	 *  - If the string 'canonical', the options are initialized with an anonymous user and
+	 *    the content language.
+	 *  - If a User or null, the options are initialized for that User (or $wgUser if null).
+	 *    'userlang' is taken from the $userLang parameter, defaulting to $wgLang if that is null.
+	 * @param Language|StubObject|null $userLang (see above)
 	 * @return ParserOptions
 	 */
-	public static function newCanonical( User $user = null, $lang = null ) {
-		$ret = new ParserOptions( $user, $lang );
+	public static function newCanonical( $context = null, $userLang = null ) {
+		if ( $context instanceof IContextSource ) {
+			$ret = self::newFromContext( $context );
+		} elseif ( $context === 'canonical' ) {
+			$ret = self::newFromAnon();
+		} elseif ( $context instanceof User || $context === null ) {
+			$ret = new self( $context, $userLang );
+		} else {
+			throw new InvalidArgumentException(
+				'$context must be an IContextSource, the string "canonical", a User, or null'
+			);
+		}
+
 		foreach ( self::getCanonicalOverrides() as $k => $v ) {
 			$ret->setOption( $k, $v );
 		}
@@ -1040,7 +1092,7 @@ class ParserOptions {
 			$wgMaxArticleSize, $wgMaxPPNodeCount, $wgMaxTemplateDepth, $wgMaxPPExpandDepth,
 			$wgCleanSignatures, $wgExternalLinkTarget, $wgExpensiveParserFunctionLimit,
 			$wgMaxGeneratedPPNodeCount, $wgDisableLangConversion, $wgDisableTitleConversion,
-			$wgEnableMagicLinks, $wgContLang;
+			$wgEnableMagicLinks;
 
 		if ( self::$defaults === null ) {
 			// *UPDATE* ParserOptions::matches() if any of this changes as needed
@@ -1060,6 +1112,7 @@ class ParserOptions {
 				'currentRevisionCallback' => [ Parser::class, 'statelessFetchRevision' ],
 				'templateCallback' => [ Parser::class, 'statelessFetchTemplate' ],
 				'speculativeRevIdCallback' => null,
+				'speculativeRevId' => null,
 			];
 
 			Hooks::run( 'ParserOptionsRegister', [
@@ -1094,7 +1147,7 @@ class ParserOptions {
 			'numberheadings' => User::getDefaultOption( 'numberheadings' ),
 			'thumbsize' => User::getDefaultOption( 'thumbsize' ),
 			'stubthreshold' => 0,
-			'userlang' => $wgContLang,
+			'userlang' => MediaWikiServices::getInstance()->getContentLanguage(),
 		];
 	}
 
@@ -1211,7 +1264,7 @@ class ParserOptions {
 	 * in 1.16.
 	 * Used to get the old parser cache entries when available.
 	 * @deprecated since 1.30. You probably want self::allCacheVaryingOptions() instead.
-	 * @return array
+	 * @return string[]
 	 */
 	public static function legacyOptions() {
 		wfDeprecated( __METHOD__, '1.30' );
@@ -1268,16 +1321,24 @@ class ParserOptions {
 	 * the same cached data safely.
 	 *
 	 * @since 1.17
-	 * @param array $forOptions
-	 * @param Title $title Used to get the content language of the page (since r97636)
+	 * @param string[] $forOptions
+	 * @param Title|null $title Used to get the content language of the page (since r97636)
 	 * @return string Page rendering hash
 	 */
 	public function optionsHash( $forOptions, $title = null ) {
 		global $wgRenderHashAppend;
 
+		$inCacheKey = self::allCacheVaryingOptions();
+
+		// Resolve any lazy options
+		foreach ( array_intersect( $forOptions, $inCacheKey, array_keys( self::$lazyOptions ) ) as $k ) {
+			if ( $this->options[$k] === null ) {
+				$this->options[$k] = call_user_func( self::$lazyOptions[$k], $this, $k );
+			}
+		}
+
 		$options = $this->options;
 		$defaults = self::getCanonicalOverrides() + self::getDefaults();
-		$inCacheKey = self::$inCacheKey;
 
 		// We only include used options with non-canonical values in the key
 		// so adding a new option doesn't invalidate the entire parser cache.
@@ -1285,13 +1346,11 @@ class ParserOptions {
 		// requires manual invalidation of existing cache entries, as mentioned
 		// in the docs on the relevant methods and hooks.
 		$values = [];
-		foreach ( $inCacheKey as $option => $include ) {
-			if ( $include && in_array( $option, $forOptions, true ) ) {
-				$v = $this->optionToString( $options[$option] );
-				$d = $this->optionToString( $defaults[$option] );
-				if ( $v !== $d ) {
-					$values[] = "$option=$v";
-				}
+		foreach ( array_intersect( $inCacheKey, $forOptions ) as $option ) {
+			$v = $this->optionToString( $options[$option] );
+			$d = $this->optionToString( $defaults[$option] );
+			if ( $v !== $d ) {
+				$values[] = "$option=$v";
 			}
 		}
 
@@ -1302,8 +1361,8 @@ class ParserOptions {
 		if ( !is_null( $title ) ) {
 			$confstr .= $title->getPageLanguage()->getExtraHashOptions();
 		} else {
-			global $wgContLang;
-			$confstr .= $wgContLang->getExtraHashOptions();
+			$confstr .=
+				MediaWikiServices::getInstance()->getContentLanguage()->getExtraHashOptions();
 		}
 
 		$confstr .= $wgRenderHashAppend;
@@ -1380,11 +1439,12 @@ class ParserOptions {
 			};
 		end( $wgHooks['TitleExists'] );
 		$key = key( $wgHooks['TitleExists'] );
-		LinkCache::singleton()->clearBadLink( $title->getPrefixedDBkey() );
-		return new ScopedCallback( function () use ( $title, $key ) {
+		$linkCache = MediaWikiServices::getInstance()->getLinkCache();
+		$linkCache->clearBadLink( $title->getPrefixedDBkey() );
+		return new ScopedCallback( function () use ( $title, $key, $linkCache ) {
 			global $wgHooks;
 			unset( $wgHooks['TitleExists'][$key] );
-			LinkCache::singleton()->clearLink( $title );
+			$linkCache->clearLink( $title );
 		} );
 	}
 }
