@@ -33,9 +33,7 @@ class ProfilerOutputText extends ProfilerOutput {
 
 	function __construct( Profiler $collector, array $params ) {
 		parent::__construct( $collector, $params );
-		$this->thresholdMs = isset( $params['thresholdMs'] )
-			? $params['thresholdMs']
-			: 1.0;
+		$this->thresholdMs = $params['thresholdMs'] ?? 1.0;
 	}
 	public function log( array $stats ) {
 		if ( $this->collector->getTemplated() ) {
@@ -48,7 +46,7 @@ class ProfilerOutputText extends ProfilerOutput {
 			} );
 			// Sort descending by time elapsed
 			usort( $stats, function ( $a, $b ) {
-				return $a['real'] < $b['real'];
+				return $b['real'] <=> $a['real'];
 			} );
 
 			array_walk( $stats,
@@ -59,11 +57,10 @@ class ProfilerOutputText extends ProfilerOutput {
 			);
 
 			$contentType = $this->collector->getContentType();
-			if ( PHP_SAPI === 'cli' ) {
+			if ( wfIsCLI() ) {
 				print "<!--\n{$out}\n-->\n";
 			} elseif ( $contentType === 'text/html' ) {
-				$visible = isset( $this->params['visible'] ) ?
-					$this->params['visible'] : false;
+				$visible = $this->params['visible'] ?? false;
 				if ( $visible ) {
 					print "<pre>{$out}</pre>";
 				} else {
