@@ -1,6 +1,6 @@
 <?php
 
-use MediaWiki\MediaWikiServices;
+use Composer\Semver\Semver;
 
 /**
  * ExtensionRegistry class
@@ -20,9 +20,16 @@ class ExtensionRegistry {
 
 	/**
 	 * Version of the highest supported manifest version
+	 * Note: Update MANIFEST_VERSION_MW_VERSION when changing this
 	 */
 	const MANIFEST_VERSION = 2;
 
+	/**
+	 * MediaWiki version constraint representing what the current
+	 * highest MANIFEST_VERSION is supported in
+	 */
+	const MANIFEST_VERSION_MW_VERSION = '>= 1.29.0';
+
 	/**
 	 * Version of the oldest supported manifest version
 	 */
@@ -31,7 +38,7 @@ class ExtensionRegistry {
 	/**
 	 * Bump whenever the registration cache needs resetting
 	 */
-	const CACHE_VERSION = 6;
+	const CACHE_VERSION = 7;
 
 	/**
 	 * Special key that defines the merge strategy
@@ -75,6 +82,7 @@ class ExtensionRegistry {
 	private static $instance;
 
 	/**
+	 * @codeCoverageIgnore
 	 * @return ExtensionRegistry
 	 */
 	public static function getInstance() {
@@ -98,9 +106,11 @@ class ExtensionRegistry {
 			} else {
 				throw new Exception( "$path does not exist!" );
 			}
-			if ( !$mtime ) {
+			// @codeCoverageIgnoreStart
+			if ( $mtime === false ) {
 				$err = error_get_last();
 				throw new Exception( "Couldn't stat $path: {$err['message']}" );
+				// @codeCoverageIgnoreEnd
 			}
 		}
 		$this->queued[$path] = $mtime;
@@ -132,12 +142,15 @@ class ExtensionRegistry {
 		// We use a try/catch because we don't want to fail here
 		// if $wgObjectCaches is not configured properly for APC setup
 		try {
-			$cache = MediaWikiServices::getInstance()->getLocalServerObjectCache();
-		} catch ( MWException $e ) {
+			// Don't use MediaWikiServices here to prevent instantiating it before extensions have
+			// been loaded
+			$cacheId = ObjectCache::detectLocalServerCache();
+			$cache = ObjectCache::newFromId( $cacheId );
+		} catch ( InvalidArgumentException $e ) {
 			$cache = new EmptyBagOStuff();
 		}
 		// See if this queue is in APC
-		$key = wfMemcKey(
+		$key = $cache->makeKey(
 			'registration',
 			md5( json_encode( $this->queued + $versions ) )
 		);
@@ -192,13 +205,19 @@ class ExtensionRegistry {
 	 * @param array $queue keys are filenames, values are ignored
 	 * @return array extracted info
 	 * @throws Exception
+	 * @throws ExtensionDependencyError
 	 */
 	public function readFromQueue( array $queue ) {
 		global $wgVersion;
 		$autoloadClasses = [];
+		$autoloadNamespaces = [];
 		$autoloaderPaths = [];
 		$processor = new ExtensionProcessor();
-		$versionChecker = new VersionChecker( $wgVersion );
+		$versionChecker = new VersionChecker(
+			$wgVersion,
+			PHP_MAJOR_VERSION . '.' . PHP_MINOR_VERSION . '.' . PHP_RELEASE_VERSION,
+			get_loaded_extensions()
+		);
 		$extDependencies = [];
 		$incompatible = [];
 		$warnings = false;
@@ -226,10 +245,16 @@ class ExtensionRegistry {
 				$incompatible[] = "$path: unsupported manifest_version: {$version}";
 			}
 
-			$autoload = $this->processAutoLoader( dirname( $path ), $info );
-			// Set up the autoloader now so custom processors will work
-			$GLOBALS['wgAutoloadClasses'] += $autoload;
-			$autoloadClasses += $autoload;
+			$dir = dirname( $path );
+			if ( isset( $info['AutoloadClasses'] ) ) {
+				$autoload = $this->processAutoLoader( $dir, $info['AutoloadClasses'] );
+				$GLOBALS['wgAutoloadClasses'] += $autoload;
+				$autoloadClasses += $autoload;
+			}
+			if ( isset( $info['AutoloadNamespaces'] ) ) {
+				$autoloadNamespaces += $this->processAutoLoader( $dir, $info['AutoloadNamespaces'] );
+				AutoLoader::$psr4Namespaces += $autoloadNamespaces;
+			}
 
 			// get all requirements/dependencies for this extension
 			$requires = $processor->getRequirements( $info );
@@ -241,7 +266,7 @@ class ExtensionRegistry {
 
 			// Get extra paths for later inclusion
 			$autoloaderPaths = array_merge( $autoloaderPaths,
-				$processor->getExtraAutoloaderPaths( dirname( $path ), $info ) );
+				$processor->getExtraAutoloaderPaths( $dir, $info ) );
 			// Compatible, read and extract info
 			$processor->extractInfo( $path, $info, $version );
 		}
@@ -257,17 +282,14 @@ class ExtensionRegistry {
 		);
 
 		if ( $incompatible ) {
-			if ( count( $incompatible ) === 1 ) {
-				throw new Exception( $incompatible[0] );
-			} else {
-				throw new Exception( implode( "\n", $incompatible ) );
-			}
+			throw new ExtensionDependencyError( $incompatible );
 		}
 
 		// Need to set this so we can += to it later
 		$data['globals']['wgAutoloadClasses'] = [];
 		$data['autoload'] = $autoloadClasses;
 		$data['autoloaderPaths'] = $autoloaderPaths;
+		$data['autoloaderNS'] = $autoloadNamespaces;
 		return $data;
 	}
 
@@ -284,7 +306,7 @@ class ExtensionRegistry {
 
 			// Optimistic: If the global is not set, or is an empty array, replace it entirely.
 			// Will be O(1) performance.
-			if ( !isset( $GLOBALS[$key] ) || ( is_array( $GLOBALS[$key] ) && !$GLOBALS[$key] ) ) {
+			if ( !array_key_exists( $key, $GLOBALS ) || ( is_array( $GLOBALS[$key] ) && !$GLOBALS[$key] ) ) {
 				$GLOBALS[$key] = $val;
 				continue;
 			}
@@ -315,11 +337,17 @@ class ExtensionRegistry {
 			}
 		}
 
+		if ( isset( $info['autoloaderNS'] ) ) {
+			AutoLoader::$psr4Namespaces += $info['autoloaderNS'];
+		}
+
 		foreach ( $info['defines'] as $name => $val ) {
 			define( $name, $val );
 		}
 		foreach ( $info['autoloaderPaths'] as $path ) {
-			require_once $path;
+			if ( file_exists( $path ) ) {
+				require_once $path;
+			}
 		}
 
 		$this->loaded += $info['credits'];
@@ -332,6 +360,12 @@ class ExtensionRegistry {
 		}
 
 		foreach ( $info['callbacks'] as $name => $cb ) {
+			if ( !is_callable( $cb ) ) {
+				if ( is_array( $cb ) ) {
+					$cb = '[ ' . implode( ', ', $cb ) . ' ]';
+				}
+				throw new UnexpectedValueException( "callback '$cb' is not callable" );
+			}
 			call_user_func( $cb, $info['credits'][$name] );
 		}
 	}
@@ -353,10 +387,24 @@ class ExtensionRegistry {
 	/**
 	 * Whether a thing has been loaded
 	 * @param string $name
+	 * @param string $constraint The required version constraint for this dependency
+	 * @throws LogicException if a specific contraint is asked for,
+	 *                        but the extension isn't versioned
 	 * @return bool
 	 */
-	public function isLoaded( $name ) {
-		return isset( $this->loaded[$name] );
+	public function isLoaded( $name, $constraint = '*' ) {
+		$isLoaded = isset( $this->loaded[$name] );
+		if ( $constraint === '*' || !$isLoaded ) {
+			return $isLoaded;
+		}
+		// if a specific constraint is requested, but no version is set, throw an exception
+		if ( !isset( $this->loaded[$name]['version'] ) ) {
+			$msg = "{$name} does not expose its version, but an extension or a skin"
+					. " requires: {$constraint}.";
+			throw new LogicException( $msg );
+		}
+
+		return SemVer::satisfies( $this->loaded[$name]['version'], $constraint );
 	}
 
 	/**
@@ -364,11 +412,7 @@ class ExtensionRegistry {
 	 * @return array
 	 */
 	public function getAttribute( $name ) {
-		if ( isset( $this->attributes[$name] ) ) {
-			return $this->attributes[$name];
-		} else {
-			return [];
-		}
+		return $this->attributes[$name] ?? [];
 	}
 
 	/**
@@ -381,30 +425,17 @@ class ExtensionRegistry {
 	}
 
 	/**
-	 * Mark a thing as loaded
-	 *
-	 * @param string $name
-	 * @param array $credits
-	 */
-	protected function markLoaded( $name, array $credits ) {
-		$this->loaded[$name] = $credits;
-	}
-
-	/**
-	 * Register classes with the autoloader
+	 * Fully expand autoloader paths
 	 *
 	 * @param string $dir
-	 * @param array $info
+	 * @param array $files
 	 * @return array
 	 */
-	protected function processAutoLoader( $dir, array $info ) {
-		if ( isset( $info['AutoloadClasses'] ) ) {
-			// Make paths absolute, relative to the JSON file
-			return array_map( function( $file ) use ( $dir ) {
-				return "$dir/$file";
-			}, $info['AutoloadClasses'] );
-		} else {
-			return [];
+	protected function processAutoLoader( $dir, array $files ) {
+		// Make paths absolute, relative to the JSON file
+		foreach ( $files as &$file ) {
+			$file = "$dir/$file";
 		}
+		return $files;
 	}
 }
