@@ -18,7 +18,8 @@
  * @file
  */
 
-use WrappedString\WrappedStringList;
+use Wikimedia\WrappedString;
+use Wikimedia\WrappedStringList;
 
 /**
  * Bootstrap a ResourceLoader client on an HTML page.
@@ -33,8 +34,8 @@ class ResourceLoaderClientHtml {
 	/** @var ResourceLoader */
 	private $resourceLoader;
 
-	/** @var string|null */
-	private $target;
+	/** @var array */
+	private $options;
 
 	/** @var array */
 	private $config = [];
@@ -56,12 +57,19 @@ class ResourceLoaderClientHtml {
 
 	/**
 	 * @param ResourceLoaderContext $context
-	 * @param string|null $target [optional] Custom 'target' parameter for the startup module
+	 * @param array $options [optional] Array of options
+	 *  - 'target': Parameter for modules=startup request, see ResourceLoaderStartUpModule.
+	 *  - 'safemode': Parameter for modules=startup request, see ResourceLoaderStartUpModule.
+	 *  - 'nonce': From OutputPage::getCSPNonce().
 	 */
-	public function __construct( ResourceLoaderContext $context, $target = null ) {
+	public function __construct( ResourceLoaderContext $context, array $options = [] ) {
 		$this->context = $context;
 		$this->resourceLoader = $context->getResourceLoader();
-		$this->target = $target;
+		$this->options = $options + [
+			'target' => null,
+			'safemode' => null,
+			'nonce' => null,
+		];
 	}
 
 	/**
@@ -87,7 +95,6 @@ class ResourceLoaderClientHtml {
 	/**
 	 * Ensure the styles of one or more modules are loaded.
 	 *
-	 * @deprecated since 1.28
 	 * @param array $modules Array of module names
 	 */
 	public function setModuleStyles( array $modules ) {
@@ -131,16 +138,15 @@ class ResourceLoaderClientHtml {
 				// moduleName => state
 			],
 			'general' => [],
-			'styles' => [
-				// moduleName
-			],
+			'styles' => [],
 			'scripts' => [],
 			// Embedding for private modules
 			'embed' => [
 				'styles' => [],
 				'general' => [],
 			],
-
+			// Deprecations for style-only modules
+			'styleDeprecations' => [],
 		];
 
 		foreach ( $this->modules as $name ) {
@@ -149,8 +155,22 @@ class ResourceLoaderClientHtml {
 				continue;
 			}
 
-			if ( $module->shouldEmbedModule( $this->context ) ) {
-				// Embed via mw.loader.implement per T36907.
+			$group = $module->getGroup();
+			$context = $this->getContext( $group, ResourceLoaderModule::TYPE_COMBINED );
+			if ( $module->isKnownEmpty( $context ) ) {
+				// Avoid needless request or embed for empty module
+				$data['states'][$name] = 'ready';
+				continue;
+			}
+
+			if ( $group === 'user' || $module->shouldEmbedModule( $this->context ) ) {
+				// Call makeLoad() to decide how to load these, instead of
+				// loading via mw.loader.load().
+				// - For group=user: We need to provide a pre-generated load.php
+				//   url to the client that has the 'user' and 'version' parameters
+				//   filled in. Without this, the client would wrongly use the static
+				//   version hash, per T64602.
+				// - For shouldEmbed=true:  Embed via mw.loader.implement, per T36907.
 				$data['embed']['general'][] = $name;
 				// Avoid duplicate request from mw.loader
 				$data['states'][$name] = 'loading';
@@ -175,25 +195,26 @@ class ResourceLoaderClientHtml {
 			}
 
 			// Stylesheet doesn't trigger mw.loader callback.
-			// Set "ready" state to allow dependencies and avoid duplicate requests. (T87871)
+			// Set "ready" state to allow script modules to depend on this module  (T87871).
+			// And to avoid duplicate requests at run-time from mw.loader.
 			$data['states'][$name] = 'ready';
 
 			$group = $module->getGroup();
 			$context = $this->getContext( $group, ResourceLoaderModule::TYPE_STYLES );
-			if ( $module->isKnownEmpty( $context ) ) {
-				// Avoid needless request for empty module
-				$data['states'][$name] = 'ready';
-			} else {
+			// Avoid needless request for empty module
+			if ( !$module->isKnownEmpty( $context ) ) {
 				if ( $module->shouldEmbedModule( $this->context ) ) {
 					// Embed via style element
 					$data['embed']['styles'][] = $name;
-					// Avoid duplicate request from mw.loader
-					$data['states'][$name] = 'ready';
 				} else {
 					// Load from load.php?only=styles via <link rel=stylesheet>
 					$data['styles'][] = $name;
 				}
 			}
+			$deprecation = $module->getDeprecationInformation();
+			if ( $deprecation ) {
+				$data['styleDeprecations'][] = $deprecation;
+			}
 		}
 
 		foreach ( $this->moduleScripts as $name ) {
@@ -241,6 +262,7 @@ class ResourceLoaderClientHtml {
 	 * @return string|WrappedStringList HTML
 	 */
 	public function getHeadHtml() {
+		$nonce = $this->options['nonce'];
 		$data = $this->getData();
 		$chunks = [];
 
@@ -249,13 +271,15 @@ class ResourceLoaderClientHtml {
 		// See also #getDocumentAttributes() and /resources/src/startup.js.
 		$chunks[] = Html::inlineScript(
 			'document.documentElement.className = document.documentElement.className'
-			. '.replace( /(^|\s)client-nojs(\s|$)/, "$1client-js$2" );'
+			. '.replace( /(^|\s)client-nojs(\s|$)/, "$1client-js$2" );',
+			$nonce
 		);
 
 		// Inline RLQ: Set page variables
 		if ( $this->config ) {
 			$chunks[] = ResourceLoader::makeInlineScript(
-				ResourceLoader::makeConfigSetScript( $this->config )
+				ResourceLoader::makeConfigSetScript( $this->config ),
+				$nonce
 			);
 		}
 
@@ -263,7 +287,8 @@ class ResourceLoaderClientHtml {
 		$states = array_merge( $this->exemptStates, $data['states'] );
 		if ( $states ) {
 			$chunks[] = ResourceLoader::makeInlineScript(
-				ResourceLoader::makeLoaderStateScript( $states )
+				ResourceLoader::makeLoaderStateScript( $states ),
+				$nonce
 			);
 		}
 
@@ -271,14 +296,19 @@ class ResourceLoaderClientHtml {
 		if ( $data['embed']['general'] ) {
 			$chunks[] = $this->getLoad(
 				$data['embed']['general'],
-				ResourceLoaderModule::TYPE_COMBINED
+				ResourceLoaderModule::TYPE_COMBINED,
+				$nonce
 			);
 		}
 
 		// Inline RLQ: Load general modules
 		if ( $data['general'] ) {
 			$chunks[] = ResourceLoader::makeInlineScript(
-				Xml::encodeJsCall( 'mw.loader.load', [ $data['general'] ] )
+				'RLPAGEMODULES='
+					. ResourceLoader::encodeJsonForScript( $data['general'] )
+					. ';'
+					. 'mw.loader.load(RLPAGEMODULES);',
+				$nonce
 			);
 		}
 
@@ -286,15 +316,17 @@ class ResourceLoaderClientHtml {
 		if ( $data['scripts'] ) {
 			$chunks[] = $this->getLoad(
 				$data['scripts'],
-				ResourceLoaderModule::TYPE_SCRIPTS
+				ResourceLoaderModule::TYPE_SCRIPTS,
+				$nonce
 			);
 		}
 
-		// External stylesheets
+		// External stylesheets (only=styles)
 		if ( $data['styles'] ) {
 			$chunks[] = $this->getLoad(
 				$data['styles'],
-				ResourceLoaderModule::TYPE_STYLES
+				ResourceLoaderModule::TYPE_STYLES,
+				$nonce
 			);
 		}
 
@@ -302,35 +334,53 @@ class ResourceLoaderClientHtml {
 		if ( $data['embed']['styles'] ) {
 			$chunks[] = $this->getLoad(
 				$data['embed']['styles'],
-				ResourceLoaderModule::TYPE_STYLES
+				ResourceLoaderModule::TYPE_STYLES,
+				$nonce
 			);
 		}
 
 		// Async scripts. Once the startup is loaded, inline RLQ scripts will run.
-		// Pass-through a custom target from OutputPage (T143066).
-		$startupQuery = $this->target ? [ 'target' => $this->target ] : [];
+		// Pass-through a custom 'target' from OutputPage (T143066).
+		$startupQuery = [];
+		foreach ( [ 'target', 'safemode' ] as $param ) {
+			if ( $this->options[$param] !== null ) {
+				$startupQuery[$param] = (string)$this->options[$param];
+			}
+		}
 		$chunks[] = $this->getLoad(
 			'startup',
 			ResourceLoaderModule::TYPE_SCRIPTS,
+			$nonce,
 			$startupQuery
 		);
 
-		return WrappedStringList::join( "\n", $chunks );
+		return WrappedString::join( "\n", $chunks );
 	}
 
 	/**
 	 * @return string|WrappedStringList HTML
 	 */
 	public function getBodyHtml() {
-		return '';
+		$data = $this->getData();
+		$chunks = [];
+
+		// Deprecations for only=styles modules
+		if ( $data['styleDeprecations'] ) {
+			$chunks[] = ResourceLoader::makeInlineScript(
+				implode( '', $data['styleDeprecations'] ),
+				$this->options['nonce']
+			);
+		}
+
+		return WrappedString::join( "\n", $chunks );
 	}
 
 	private function getContext( $group, $type ) {
 		return self::makeContext( $this->context, $group, $type );
 	}
 
-	private function getLoad( $modules, $only, array $extraQuery = [] ) {
-		return self::makeLoad( $this->context, (array)$modules, $only, $extraQuery );
+	private function getLoad( $modules, $only, $nonce, array $extraQuery = [] ) {
+		return self::makeLoad( $this->context, (array)$modules, $only, $extraQuery, $nonce );
 	}
 
 	private static function makeContext( ResourceLoaderContext $mainContext, $group, $type,
@@ -346,7 +396,9 @@ class ResourceLoaderClientHtml {
 		}
 		$context = new ResourceLoaderContext( $mainContext->getResourceLoader(), $req );
 		// Allow caller to setVersion() and setModules()
-		return new DerivativeResourceLoaderContext( $context );
+		$ret = new DerivativeResourceLoaderContext( $context );
+		$ret->setContentOverrideCallback( $mainContext->getContentOverrideCallback() );
+		return $ret;
 	}
 
 	/**
@@ -356,10 +408,12 @@ class ResourceLoaderClientHtml {
 	 * @param array $modules One or more module names
 	 * @param string $only ResourceLoaderModule TYPE_ class constant
 	 * @param array $extraQuery [optional] Array with extra query parameters for the request
+	 * @param string|null $nonce [optional] Content-Security-Policy nonce
+	 *  (from OutputPage::getCSPNonce)
 	 * @return string|WrappedStringList HTML
 	 */
 	public static function makeLoad( ResourceLoaderContext $mainContext, array $modules, $only,
-		array $extraQuery = []
+		array $extraQuery = [], $nonce = null
 	) {
 		$rl = $mainContext->getResourceLoader();
 		$chunks = [];
@@ -371,7 +425,7 @@ class ResourceLoaderClientHtml {
 			$chunks = [];
 			// Recursively call us for every item
 			foreach ( $modules as $name ) {
-				$chunks[] = self::makeLoad( $mainContext, [ $name ], $only, $extraQuery );
+				$chunks[] = self::makeLoad( $mainContext, [ $name ], $only, $extraQuery, $nonce );
 			}
 			return new WrappedStringList( "\n", $chunks );
 		}
@@ -413,7 +467,8 @@ class ResourceLoaderClientHtml {
 							);
 						} else {
 							$chunks[] = ResourceLoader::makeInlineScript(
-								$rl->makeModuleResponse( $context, $moduleSet )
+								$rl->makeModuleResponse( $context, $moduleSet ),
+								$nonce
 							);
 						}
 					} else {
@@ -447,7 +502,8 @@ class ResourceLoaderClientHtml {
 								] );
 							} else {
 								$chunk = ResourceLoader::makeInlineScript(
-									Xml::encodeJsCall( 'mw.loader.load', [ $url ] )
+									Xml::encodeJsCall( 'mw.loader.load', [ $url ] ),
+									$nonce
 								);
 							}
 						}
