@@ -35,7 +35,7 @@ class ResourceLoaderOOUIImageModule extends ResourceLoaderImageModule {
 		$themes = self::getSkinThemeMap();
 
 		// For backwards-compatibility, allow missing 'themeImages'
-		$module = isset( $this->definition['themeImages'] ) ? $this->definition['themeImages'] : '';
+		$module = $this->definition['themeImages'] ?? '';
 
 		$definition = [];
 		foreach ( $themes as $skin => $theme ) {
@@ -98,11 +98,8 @@ class ResourceLoaderOOUIImageModule extends ResourceLoaderImageModule {
 		// Extra selectors to allow using the same icons for old-style MediaWiki UI code
 		if ( substr( $module, 0, 5 ) === 'icons' ) {
 			$definition['selectorWithoutVariant'] = '.oo-ui-icon-{name}, .mw-ui-icon-{name}:before';
-			$definition['selectorWithVariant'] = '
-				.oo-ui-image-{variant}.oo-ui-icon-{name}, .mw-ui-icon-{name}-{variant}:before,
-				/* Hack for Flow, see T110051 */
-				.mw-ui-hovericon:hover .mw-ui-icon-{name}-{variant}-hover:before,
-				.mw-ui-hovericon.mw-ui-icon-{name}-{variant}-hover:hover:before';
+			$definition['selectorWithVariant'] = '.oo-ui-image-{variant}.oo-ui-icon-{name}, ' .
+				'.mw-ui-icon-{name}-{variant}:before';
 		}
 
 		// Fields from module definition silently override keys from JSON files
