@@ -31,20 +31,14 @@ abstract class QuickTemplate {
 	 */
 	public $data;
 
-	/**
-	 * @var MediaWikiI18N
-	 */
-	public $translator;
-
 	/** @var Config $config */
 	protected $config;
 
 	/**
-	 * @param Config $config
+	 * @param Config|null $config
 	 */
 	function __construct( Config $config = null ) {
 		$this->data = [];
-		$this->translator = new MediaWikiI18N();
 		if ( $config === null ) {
 			wfDebug( __METHOD__ . ' was called with no Config instance passed to it' );
 			$config = MediaWikiServices::getInstance()->getMainConfig();
@@ -79,32 +73,26 @@ abstract class QuickTemplate {
 	 * Gets the template data requested
 	 * @since 1.22
 	 * @param string $name Key for the data
-	 * @param mixed $default Optional default (or null)
+	 * @param mixed|null $default Optional default (or null)
 	 * @return mixed The value of the data requested or the deafult
+	 * @return-taint onlysafefor_htmlnoent
 	 */
 	public function get( $name, $default = null ) {
-		if ( isset( $this->data[$name] ) ) {
-			return $this->data[$name];
-		} else {
-			return $default;
-		}
+		return $this->data[$name] ?? $default;
 	}
 
 	/**
+	 * @deprecated since 1.31 This function is a now-redundant optimisation intended
+	 *  for very old versions of PHP. The use of references here makes the code
+	 *  more fragile and is incompatible with plans like T140664. Use set() instead.
 	 * @param string $name
 	 * @param mixed &$value
 	 */
 	public function setRef( $name, &$value ) {
+		wfDeprecated( __METHOD__, '1.31' );
 		$this->data[$name] =& $value;
 	}
 
-	/**
-	 * @param MediaWikiI18N &$t
-	 */
-	public function setTranslator( &$t ) {
-		$this->translator = &$t;
-	}
-
 	/**
 	 * Main function, used by classes that subclass QuickTemplate
 	 * to show the actual HTML output
@@ -114,6 +102,7 @@ abstract class QuickTemplate {
 	/**
 	 * @private
 	 * @param string $str
+	 * @suppress SecurityCheck-DoubleEscaped $this->data can be either
 	 */
 	function text( $str ) {
 		echo htmlspecialchars( $this->data[$str] );
@@ -122,6 +111,7 @@ abstract class QuickTemplate {
 	/**
 	 * @private
 	 * @param string $str
+	 * @suppress SecurityCheck-XSS phan-taint-check cannot tell if $str is pre-escaped
 	 */
 	function html( $str ) {
 		echo $this->data[$str];
@@ -129,29 +119,34 @@ abstract class QuickTemplate {
 
 	/**
 	 * @private
-	 * @param string $str
+	 * @param string $msgKey
 	 */
-	function msg( $str ) {
-		echo htmlspecialchars( $this->translator->translate( $str ) );
+	function msg( $msgKey ) {
+		echo htmlspecialchars( wfMessage( $msgKey )->text() );
 	}
 
 	/**
 	 * @private
-	 * @param string $str
+	 * @param string $msgKey
+	 * @warning You should never use this method. I18n messages should be escaped
+	 * @deprecated 1.32 Use ->msg() or ->msgWiki() instead.
+	 * @suppress SecurityCheck-XSS
+	 * @return-taint exec_html
 	 */
-	function msgHtml( $str ) {
-		echo $this->translator->translate( $str );
+	function msgHtml( $msgKey ) {
+		wfDeprecated( __METHOD__, '1.32' );
+		echo wfMessage( $msgKey )->text();
 	}
 
 	/**
 	 * An ugly, ugly hack.
 	 * @private
-	 * @param string $str
+	 * @param string $msgKey
 	 */
-	function msgWiki( $str ) {
+	function msgWiki( $msgKey ) {
 		global $wgOut;
 
-		$text = $this->translator->translate( $str );
+		$text = wfMessage( $msgKey )->text();
 		echo $wgOut->parse( $text );
 	}
 
@@ -167,12 +162,12 @@ abstract class QuickTemplate {
 	/**
 	 * @private
 	 *
-	 * @param string $str
+	 * @param string $msgKey
 	 * @return bool
 	 */
-	function haveMsg( $str ) {
-		$msg = $this->translator->translate( $str );
-		return ( $msg != '-' ) && ( $msg != '' ); # ????
+	function haveMsg( $msgKey ) {
+		$msg = wfMessage( $msgKey );
+		return $msg->exists() && !$msg->isDisabled();
 	}
 
 	/**
