@@ -50,11 +50,7 @@ class SpecialContributions extends IncludableSpecialPage {
 		$this->opts = [];
 		$request = $this->getRequest();
 
-		if ( $par !== null ) {
-			$target = $par;
-		} else {
-			$target = $request->getVal( 'target' );
-		}
+		$target = $par ?? $request->getVal( 'target' );
 
 		if ( $request->getVal( 'contribs' ) == 'newbie' || $par === 'newbies' ) {
 			$target = 'newbies';
@@ -556,7 +552,7 @@ class SpecialContributions extends IncludableSpecialPage {
 			$filterSelection = Html::rawElement(
 				'div',
 				[],
-				implode( '&#160;', $tagFilter )
+				implode( "\u{00A0}", $tagFilter )
 			);
 		} else {
 			$filterSelection = Html::rawElement( 'div', [], '' );
@@ -609,7 +605,7 @@ class SpecialContributions extends IncludableSpecialPage {
 				$this->msg( 'namespace' )->text(),
 				'namespace',
 				''
-			) . '&#160;' .
+			) . "\u{00A0}" .
 			Html::namespaceSelector(
 				[ 'selected' => $this->opts['namespace'], 'all' => '' ],
 				[
@@ -617,7 +613,7 @@ class SpecialContributions extends IncludableSpecialPage {
 					'id' => 'namespace',
 					'class' => 'namespaceselector',
 				]
-			) . '&#160;' .
+			) . "\u{00A0}" .
 				Html::rawElement(
 					'span',
 					[ 'class' => 'mw-input-with-label' ],
@@ -630,7 +626,7 @@ class SpecialContributions extends IncludableSpecialPage {
 							'title' => $this->msg( 'tooltip-invert' )->text(),
 							'class' => 'mw-input'
 						]
-					) . '&#160;'
+					) . "\u{00A0}"
 				) .
 				Html::rawElement( 'span', [ 'class' => 'mw-input-with-label' ],
 					Xml::checkLabel(
@@ -642,7 +638,7 @@ class SpecialContributions extends IncludableSpecialPage {
 							'title' => $this->msg( 'tooltip-namespace_association' )->text(),
 							'class' => 'mw-input'
 						]
-					) . '&#160;'
+					) . "\u{00A0}"
 				)
 		);
 
@@ -710,7 +706,7 @@ class SpecialContributions extends IncludableSpecialPage {
 		$dateRangeSelection = Html::rawElement(
 			'div',
 			[],
-			Xml::label( wfMessage( 'date-range-from' )->text(), 'mw-date-start' ) . ' ' .
+			Xml::label( $this->msg( 'date-range-from' )->text(), 'mw-date-start' ) . ' ' .
 			new DateInputWidget( [
 				'infusable' => true,
 				'id' => 'mw-date-start',
@@ -718,7 +714,7 @@ class SpecialContributions extends IncludableSpecialPage {
 				'value' => $this->opts['start'],
 				'longDisplayFormat' => true,
 			] ) . '<br>' .
-			Xml::label( wfMessage( 'date-range-to' )->text(), 'mw-date-end' ) . ' ' .
+			Xml::label( $this->msg( 'date-range-to' )->text(), 'mw-date-end' ) . ' ' .
 			new DateInputWidget( [
 				'infusable' => true,
 				'id' => 'mw-date-end',
