@@ -21,6 +21,7 @@
  * @ingroup SpecialPage
  */
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Preferences\MultiUsernameFilter;
 
 /**
  * A special page that allows users to send e-mails to other users
@@ -204,7 +205,7 @@ class SpecialEmailUser extends UnlistedSpecialPage {
 		$nu = User::newFromName( $target );
 		$error = self::validateTarget( $nu, $sender );
 
-		return $error ? $error : $nu;
+		return $error ?: $nu;
 	}
 
 	/**
@@ -224,17 +225,32 @@ class SpecialEmailUser extends UnlistedSpecialPage {
 			wfDebug( "Target is invalid user.\n" );
 
 			return 'notarget';
-		} elseif ( !$target->isEmailConfirmed() ) {
+		}
+
+		if ( !$target->isEmailConfirmed() ) {
 			wfDebug( "User has no valid email.\n" );
 
 			return 'noemail';
-		} elseif ( !$target->canReceiveEmail() ) {
+		}
+
+		if ( !$target->canReceiveEmail() ) {
 			wfDebug( "User does not allow user emails.\n" );
 
 			return 'nowikiemail';
-		} elseif ( $sender !== null ) {
-			$blacklist = $target->getOption( 'email-blacklist', [] );
+		}
+
+		if ( $sender !== null && !$target->getOption( 'email-allow-new-users' ) &&
+			$sender->isNewbie()
+		) {
+			wfDebug( "User does not allow user emails from new users.\n" );
+
+			return 'nowikiemail';
+		}
+
+		if ( $sender !== null ) {
+			$blacklist = $target->getOption( 'email-blacklist', '' );
 			if ( $blacklist ) {
+				$blacklist = MultiUsernameFilter::splitIds( $blacklist );
 				$lookup = CentralIdLookup::factory();
 				$senderId = $lookup->centralIdFromLocalUser( $sender );
 				if ( $senderId !== 0 && in_array( $senderId, $blacklist ) ) {
@@ -253,7 +269,7 @@ class SpecialEmailUser extends UnlistedSpecialPage {
 	 *
 	 * @param User $user
 	 * @param string $editToken Edit token
-	 * @param Config $config optional for backwards compatibility
+	 * @param Config|null $config optional for backwards compatibility
 	 * @return string|null Null on success or string on error
 	 */
 	public static function getPermissionsError( $user, $editToken, Config $config = null ) {
@@ -280,7 +296,9 @@ class SpecialEmailUser extends UnlistedSpecialPage {
 			return "blockedemailuser";
 		}
 
-		if ( $user->pingLimiter( 'emailuser' ) ) {
+		// Check the ping limiter without incrementing it - we'll check it
+		// again later and increment it on a successful send
+		if ( $user->pingLimiter( 'emailuser', 0 ) ) {
 			wfDebug( "Ping limiter triggered.\n" );
 
 			return 'actionthrottledtext';
@@ -316,7 +334,7 @@ class SpecialEmailUser extends UnlistedSpecialPage {
 			Html::label(
 				$this->msg( 'emailusername' )->text(),
 				'emailusertarget'
-			) . '&#160;' .
+			) . "\u{00A0}" .
 			Html::input(
 				'target',
 				$name,
@@ -376,6 +394,11 @@ class SpecialEmailUser extends UnlistedSpecialPage {
 		$text .= $context->msg( 'emailuserfooter',
 			$from->name, $to->name )->inContentLanguage()->text();
 
+		// Check and increment the rate limits
+		if ( $context->getUser()->pingLimiter( 'emailuser' ) ) {
+			throw new ThrottledError();
+		}
+
 		$error = false;
 		if ( !Hooks::run( 'EmailUser', [ &$to, &$from, &$subject, &$text, &$error ] ) ) {
 			if ( $error instanceof Status ) {
@@ -415,7 +438,7 @@ class SpecialEmailUser extends UnlistedSpecialPage {
 			 * SPF and bounce problems with some mailers (see below).
 			 */
 			$mailFrom = new MailAddress( $config->get( 'PasswordSender' ),
-				wfMessage( 'emailsender' )->inContentLanguage()->text() );
+				$context->msg( 'emailsender' )->inContentLanguage()->text() );
 			$replyTo = $from;
 		} else {
 			/**
@@ -459,7 +482,7 @@ class SpecialEmailUser extends UnlistedSpecialPage {
 				if ( $config->get( 'UserEmailUseReplyTo' ) ) {
 					$mailFrom = new MailAddress(
 						$config->get( 'PasswordSender' ),
-						wfMessage( 'emailsender' )->inContentLanguage()->text()
+						$context->msg( 'emailsender' )->inContentLanguage()->text()
 					);
 					$replyTo = $ccFrom;
 				} else {
