@@ -21,6 +21,8 @@
  * @ingroup SpecialPage
  */
 
+use Wikimedia\Timestamp\TimestampException;
+
 /**
  * A special page that lists log entries
  *
@@ -46,11 +48,13 @@ class SpecialLog extends SpecialPage {
 		$opts->add( 'pattern', false );
 		$opts->add( 'year', null, FormOptions::INTNULL );
 		$opts->add( 'month', null, FormOptions::INTNULL );
+		$opts->add( 'day', null, FormOptions::INTNULL );
 		$opts->add( 'tagfilter', '' );
 		$opts->add( 'offset', '' );
 		$opts->add( 'dir', '' );
 		$opts->add( 'offender', '' );
 		$opts->add( 'subtype', '' );
+		$opts->add( 'logid', '' );
 
 		// Set values
 		$opts->fetchValuesFromRequest( $this->getRequest() );
@@ -58,6 +62,23 @@ class SpecialLog extends SpecialPage {
 			$this->parseParams( $opts, (string)$par );
 		}
 
+		// Set date values
+		$dateString = $this->getRequest()->getVal( 'wpdate' );
+		if ( !empty( $dateString ) ) {
+			try {
+				$dateStamp = MWTimestamp::getInstance( $dateString . ' 00:00:00' );
+			} catch ( TimestampException $e ) {
+				// If users provide an invalid date, silently ignore it
+				// instead of letting an exception bubble up (T201411)
+				$dateStamp = false;
+			}
+			if ( $dateStamp ) {
+				$opts->setValue( 'year', (int)$dateStamp->format( 'Y' ) );
+				$opts->setValue( 'month', (int)$dateStamp->format( 'm' ) );
+				$opts->setValue( 'day', (int)$dateStamp->format( 'd' ) );
+			}
+		}
+
 		# Don't let the user get stuck with a certain date
 		if ( $opts->getValue( 'offset' ) || $opts->getValue( 'dir' ) == 'prev' ) {
 			$opts->setValue( 'year', '' );
@@ -83,30 +104,12 @@ class SpecialLog extends SpecialPage {
 			$offenderName = $opts->getValue( 'offender' );
 			$offender = empty( $offenderName ) ? null : User::newFromName( $offenderName, false );
 			if ( $offender ) {
-				if ( $wgActorTableSchemaMigrationStage === MIGRATION_NEW ) {
+				if ( $wgActorTableSchemaMigrationStage & SCHEMA_COMPAT_READ_NEW ) {
 					$qc = [ 'ls_field' => 'target_author_actor', 'ls_value' => $offender->getActorId() ];
+				} elseif ( $offender->getId() > 0 ) {
+					$qc = [ 'ls_field' => 'target_author_id', 'ls_value' => $offender->getId() ];
 				} else {
-					if ( $offender->getId() > 0 ) {
-						$field = 'target_author_id';
-						$value = $offender->getId();
-					} else {
-						$field = 'target_author_ip';
-						$value = $offender->getName();
-					}
-					if ( !$offender->getActorId() ) {
-						$qc = [ 'ls_field' => $field, 'ls_value' => $value ];
-					} else {
-						$db = wfGetDB( DB_REPLICA );
-						$qc = [
-							'ls_field' => [ 'target_author_actor', $field ], // So LogPager::getQueryInfo() works right
-							$db->makeList( [
-								$db->makeList(
-									[ 'ls_field' => 'target_author_actor', 'ls_value' => $offender->getActorId() ], LIST_AND
-								),
-								$db->makeList( [ 'ls_field' => $field, 'ls_value' => $value ], LIST_AND ),
-							], LIST_OR ),
-						];
-					}
+					$qc = [ 'ls_field' => 'target_author_ip', 'ls_value' => $offender->getName() ];
 				}
 			}
 		} else {
@@ -163,19 +166,29 @@ class SpecialLog extends SpecialPage {
 	 * @return string[] subpages
 	 */
 	public function getSubpagesForPrefixSearch() {
-		$subpages = $this->getConfig()->get( 'LogTypes' );
+		$subpages = LogPage::validTypes();
 		$subpages[] = 'all';
 		sort( $subpages );
 		return $subpages;
 	}
 
+	/**
+	 * Set options based on the subpage title parts:
+	 * - One part that is a valid log type: Special:Log/logtype
+	 * - Two parts: Special:Log/logtype/username
+	 * - Otherwise, assume the whole subpage is a username.
+	 *
+	 * @param FormOptions $opts
+	 * @param $par
+	 * @throws ConfigException
+	 */
 	private function parseParams( FormOptions $opts, $par ) {
 		# Get parameters
 		$par = $par !== null ? $par : '';
 		$parms = explode( '/', $par );
 		$symsForAll = [ '*', 'all' ];
 		if ( $parms[0] != '' &&
-			( in_array( $par, $this->getConfig()->get( 'LogTypes' ) ) || in_array( $par, $symsForAll ) )
+			( in_array( $par, LogPage::validTypes() ) || in_array( $par, $symsForAll ) )
 		) {
 			$opts->setValue( 'type', $par );
 		} elseif ( count( $parms ) == 2 ) {
@@ -203,8 +216,10 @@ class SpecialLog extends SpecialPage {
 			$extraConds,
 			$opts->getValue( 'year' ),
 			$opts->getValue( 'month' ),
+			$opts->getValue( 'day' ),
 			$opts->getValue( 'tagfilter' ),
-			$opts->getValue( 'subtype' )
+			$opts->getValue( 'subtype' ),
+			$opts->getValue( 'logid' )
 		);
 
 		$this->addHeader( $opts->getValue( 'type' ) );
@@ -223,6 +238,7 @@ class SpecialLog extends SpecialPage {
 			$pager->getPattern(),
 			$pager->getYear(),
 			$pager->getMonth(),
+			$pager->getDay(),
 			$pager->getFilterParams(),
 			$pager->getTagFilter(),
 			$pager->getAction()
