@@ -246,7 +246,6 @@ class SpecialTags extends SpecialPage {
 		}
 
 		if ( $showManageActions ) { // we've already checked that the user had the requisite userright
-
 			// activate
 			if ( ChangeTags::canActivateTag( $tag )->isOK() ) {
 				$actionLinks[] = $linkRenderer->makeKnownLink(
@@ -264,7 +263,6 @@ class SpecialTags extends SpecialPage {
 					[],
 					[ 'tag' => $tag ] );
 			}
-
 		}
 
 		if ( $showDeleteActions || $showManageActions ) {
@@ -443,7 +441,7 @@ class SpecialTags extends SpecialPage {
 		$out = $context->getOutput();
 
 		$tag = $data['HiddenTag'];
-		$status = call_user_func( [ 'ChangeTags', "{$form->tagAction}TagWithChecks" ],
+		$status = call_user_func( [ ChangeTags::class, "{$form->tagAction}TagWithChecks" ],
 			$tag, $data['Reason'], $context->getUser(), true );
 
 		if ( $status->isGood() ) {
