@@ -85,13 +85,12 @@ class MergeHistoryPager extends ReverseChronologicalPager {
 		$conds['rev_page'] = $this->articleID;
 		$conds[] = "rev_timestamp < " . $this->mDb->addQuotes( $this->maxTimestamp );
 
+		$revQuery = Revision::getQueryInfo( [ 'page', 'user' ] );
 		return [
-			'tables' => [ 'revision', 'page', 'user' ],
-			'fields' => array_merge( Revision::selectFields(), Revision::selectUserFields() ),
+			'tables' => $revQuery['tables'],
+			'fields' => $revQuery['fields'],
 			'conds' => $conds,
-			'join_conds' => [
-				'page' => Revision::pageJoinCond(),
-				'user' => Revision::userJoinCond() ]
+			'join_conds' => $revQuery['joins']
 		];
 	}
 
