@@ -29,8 +29,6 @@ use Wikimedia\Rdbms\IDatabase;
 class UserRightsProxy {
 
 	/**
-	 * Constructor.
-	 *
 	 * @see newFromId()
 	 * @see newFromName()
 	 * @param IDatabase $db Db connection
@@ -279,11 +277,11 @@ class UserRightsProxy {
 			__METHOD__
 		);
 
-		$wikiId = $this->db->getWikiID();
+		$domainId = $this->db->getDomainID();
 		$userId = $this->id;
 		$this->db->onTransactionPreCommitOrIdle(
-			function () use ( $wikiId, $userId ) {
-				User::purge( $wikiId, $userId );
+			function () use ( $domainId, $userId ) {
+				User::purge( $domainId, $userId );
 			},
 			__METHOD__
 		);
