@@ -1,16 +1,14 @@
 <?php
-/**
- * MediaWiki Widgets – UsersMultiselectWidget class.
- *
- * @copyright 2017 MediaWiki Widgets Team and others; see AUTHORS.txt
- * @license The MIT License (MIT); see LICENSE.txt
- */
+
 namespace MediaWiki\Widget;
 
-use \OOUI\TextInputWidget;
+use OOUI\MultilineTextInputWidget;
 
 /**
  * Widget to select multiple users.
+ *
+ * @copyright 2017 MediaWiki Widgets Team and others; see AUTHORS.txt
+ * @license MIT
  */
 class UsersMultiselectWidget extends \OOUI\Widget {
 
@@ -20,9 +18,9 @@ class UsersMultiselectWidget extends \OOUI\Widget {
 
 	/**
 	 * @param array $config Configuration options
-	 * @param array $config['users'] Array of usernames to use as preset data
-	 * @param array $config['placeholder'] Placeholder message for input
-	 * @param array $config['name'] Name attribute (used in forms)
+	 *   - array $config['users'] Array of usernames to use as preset data
+	 *   - array $config['placeholder'] Placeholder message for input
+	 *   - array $config['name'] Name attribute (used in forms)
 	 */
 	public function __construct( array $config = [] ) {
 		parent::__construct( $config );
@@ -38,11 +36,10 @@ class UsersMultiselectWidget extends \OOUI\Widget {
 			$this->inputPlaceholder = $config['placeholder'];
 		}
 
-		$textarea = new TextInputWidget( [
+		$textarea = new MultilineTextInputWidget( [
 			'name' => $this->inputName,
-			'multiline' => true,
 			'value' => implode( "\n", $this->usersArray ),
-			'rows' => 25,
+			'rows' => 10,
 		] );
 		$this->prependContent( $textarea );
 	}
@@ -62,6 +59,7 @@ class UsersMultiselectWidget extends \OOUI\Widget {
 			$config['placeholder'] = $this->inputPlaceholder;
 		}
 
+		$config['$overlay'] = true;
 		return parent::getConfig( $config );
 	}
 
