@@ -57,16 +57,16 @@ class KuConverter extends LanguageConverter {
 		'؟' => '?',
 
 		# digits
-		'٠' => '0', # &#x0660;
-		'١' => '1', # &#x0661;
-		'٢' => '2', # &#x0662;
-		'٣' => '3', # &#x0663;
-		'٤' => '4', # &#x0664;
-		'٥' => '5', # &#x0665;
-		'٦' => '6', # &#x0666;
-		'٧' => '7', # &#x0667;
-		'٨' => '8', # &#x0668;
-		'٩' => '9', # &#x0669;
+		'٠' => '0', # U+0660
+		'١' => '1', # U+0661
+		'٢' => '2', # U+0662
+		'٣' => '3', # U+0663
+		'٤' => '4', # U+0664
+		'٥' => '5', # U+0665
+		'٦' => '6', # U+0666
+		'٧' => '7', # U+0667
+		'٨' => '8', # U+0668
+		'٩' => '9', # U+0669
 	];
 
 	public $mLatinToArabic = [
@@ -130,16 +130,16 @@ class KuConverter extends LanguageConverter {
 
 /*		# deactivated for now, breaks links i.e. in header of Special:Recentchanges :-(
 		# digits
-		'0' => '٠', # &#x0660;
-		'1' => '١', # &#x0661;
-		'2' => '٢', # &#x0662;
-		'3' => '٣', # &#x0663;
-		'4' => '٤', # &#x0664;
-		'5' => '٥', # &#x0665;
-		'6' => '٦', # &#x0666;
-		'7' => '٧', # &#x0667;
-		'8' => '٨', # &#x0668;
-		'9' => '٩', # &#x0669;
+		'0' => '٠', # U+0660
+		'1' => '١', # U+0661
+		'2' => '٢', # U+0662
+		'3' => '٣', # U+0663
+		'4' => '٤', # U+0664
+		'5' => '٥', # U+0665
+		'6' => '٦', # U+0666
+		'7' => '٧', # U+0667
+		'8' => '٨', # U+0668
+		'9' => '٩', # U+0669
 */
 		];
 
@@ -192,7 +192,8 @@ class KuConverter extends LanguageConverter {
 		/* From Kazakh interface, maybe we need it later
 		$breaks = '[^\w\x80-\xff]';
 		// regexp for roman numbers
-		$roman = 'M{0,4}(CM|CD|D?C{0,3})(XC|XL|L?X{0,3})(IX|IV|V?I{0,3})';
+		// Lookahead assertion ensures $roman doesn't match the empty string
+		$roman = '(?=[MDCLXVI])M{0,4}(C[DM]|D?C{0,3})(X[LC]|L?X{0,3})(I[VX]|V?I{0,3})';
 		$roman = '';
 
 		$reg = '/^'.$roman.'$|^'.$roman.$breaks.'|'.$breaks.$roman.'$|'.$breaks.$roman.$breaks.'/';
@@ -227,7 +228,7 @@ class KuConverter extends LanguageConverter {
  *
  * @ingroup Language
  */
-class LanguageKu extends LanguageKu_ku {
+class LanguageKu extends Language {
 
 	function __construct() {
 		parent::__construct();
