@@ -160,6 +160,7 @@ $specialPageAliases = [
 	'Withoutinterwiki'          => [ 'Bez_interwiki', 'Stránky_bez_interwiki_odkazů' ],
 ];
 
+// TODO: unify "Strana" with "Stránka"
 $magicWords = [
 	'redirect'                  => [ '0', '#PŘESMĚRUJ', '#REDIRECT' ],
 	'notoc'                     => [ '0', '__BEZOBSAHU__', '__NOTOC__' ],
@@ -199,6 +200,7 @@ $magicWords = [
 	'pagenamee'                 => [ '1', 'NÁZEVSTRANYE', 'PAGENAMEE' ],
 	'namespace'                 => [ '1', 'JMENNÝPROSTOR', 'NAMESPACE' ],
 	'namespacee'                => [ '1', 'JMENNÝPROSTORE', 'NAMESPACEE' ],
+	'namespacenumber'           => [ '1', 'ČÍSLOJMENNÉHOPROSTORU', 'NAMESPACENUMBER' ],
 	'talkspace'                 => [ '1', 'DISKUSNÍPROSTOR', 'TALKSPACE' ],
 	'talkspacee'                => [ '1', 'DISKUSNÍPROSTORE', 'TALKSPACEE' ],
 	'subjectspace'              => [ '1', 'ČLÁNEKPROSTOR', 'SUBJECTSPACE', 'ARTICLESPACE' ],
@@ -207,6 +209,8 @@ $magicWords = [
 	'fullpagenamee'             => [ '1', 'PLNÝNÁZEVSTRANYE', 'FULLPAGENAMEE' ],
 	'subpagename'               => [ '1', 'NÁZEVPODSTRANY', 'SUBPAGENAME' ],
 	'subpagenamee'              => [ '1', 'NÁZEVPODSTRANYE', 'SUBPAGENAMEE' ],
+	'rootpagename'              => [ '1', 'NÁZEVKOŘENOVÉSTRANY', 'ROOTPAGENAME' ],
+	'rootpagenamee'             => [ '1', 'NÁZEVKOŘENOVÉSTRANYE', 'ROOTPAGENAMEE' ],
 	'basepagename'              => [ '1', 'NÁZEVNADSTRANY', 'BASEPAGENAME' ],
 	'basepagenamee'             => [ '1', 'NÁZEVNADSTRANYE', 'BASEPAGENAMEE' ],
 	'talkpagename'              => [ '1', 'NÁZEVDISKUSE', 'TALKPAGENAME' ],
@@ -227,12 +231,22 @@ $magicWords = [
 	'img_lang'                  => [ '1', 'jazyk=$1', 'lang=$1' ],
 	'img_page'                  => [ '1', 'strana=$1', 'strana_$1', 'page=$1', 'page $1' ],
 	'img_border'                => [ '1', 'okraj', 'border' ],
+	'img_link'                  => [ '1', 'odkaz=$1', 'link=$1' ],
+	'img_class'                 => [ '1', 'třída=$1', 'class=$1' ],
+	'int'                       => [ '0', 'HLÁŠENÍ:', 'INT:' ],
 	'sitename'                  => [ '1', 'NÁZEVWEBU', 'SITENAME' ],
 	'ns'                        => [ '0', 'JMENNÝPROSTOR:', 'NS:' ],
+	'nse'                       => [ '0', 'JMENNÝPROSTORE:', 'NSE:' ],
 	'localurl'                  => [ '0', 'MÍSTNÍURL:', 'LOCALURL:' ],
 	'localurle'                 => [ '0', 'MÍSTNÍURLE:', 'LOCALURLE:' ],
+	'articlepath'               => [ '0', 'CESTAKČLÁNKU', 'ARTICLEPATH' ],
+	'pageid'                    => [ '0', 'IDSTRÁNKY', 'PAGEID' ],
 	'servername'                => [ '0', 'NÁZEVSERVERU', 'SERVERNAME' ],
+	'scriptpath'                => [ '0', 'CESTAKESKRIPTŮM', 'SCRIPTPATH' ],
+	'stylepath'                 => [ '0', 'CESTAKESTYLŮM', 'STYLEPATH' ],
 	'grammar'                   => [ '0', 'SKLOŇUJ:', 'GRAMMAR:' ],
+	'gender'                    => [ '0', 'POHLAVÍ:', 'GENDER:' ],
+	'bidi'                      => [ '0', 'OBASMĚRY:', 'BIDI:' ],
 	'notitleconvert'            => [ '0', '__BEZKONVERZENADPISU__', '__NOTITLECONVERT__', '__NOTC__' ],
 	'nocontentconvert'          => [ '0', '__BEZKONVERZEOBSAHU__', '__NOCONTENTCONVERT__', '__NOCC__' ],
 	'currentweek'               => [ '1', 'AKTUÁLNÍTÝDEN', 'CURRENTWEEK' ],
@@ -243,24 +257,32 @@ $magicWords = [
 	'revisionday'               => [ '1', 'DENREVIZE', 'REVISIONDAY' ],
 	'revisionday2'              => [ '1', 'DENREVIZE2', 'REVISIONDAY2' ],
 	'revisionmonth'             => [ '1', 'MĚSÍCREVIZE', 'REVISIONMONTH' ],
+	'revisionmonth1'            => [ '1', 'MĚSÍCREVIZE1', 'REVISIONMONTH1' ],
 	'revisionyear'              => [ '1', 'ROKREVIZE', 'REVISIONYEAR' ],
 	'revisiontimestamp'         => [ '1', 'KÓDČASUREVIZE', 'REVISIONTIMESTAMP' ],
+	'revisionuser'              => [ '1', 'AUTORREVIZE', 'REVISIONUSER' ],
+	'revisionsize'              => [ '1', 'VELIKOSTREVIZE', 'REVISIONSIZE' ],
 	'plural'                    => [ '0', 'PLURÁL:', 'PLURAL:' ],
 	'fullurl'                   => [ '0', 'PLNÉURL:', 'FULLURL:' ],
 	'fullurle'                  => [ '0', 'PLNÉURLE:', 'FULLURLE:' ],
+	'canonicalurl'              => [ '0', 'KANONICKÉURL:', 'CANONICALURL:' ],
+	'canonicalurle'             => [ '0', 'KANONICKÉURLE:', 'CANONICALURLE:' ],
 	'lcfirst'                   => [ '0', 'PRVNÍMALÉ:', 'LCFIRST:' ],
 	'ucfirst'                   => [ '0', 'PRVNÍVELKÉ:', 'UCFIRST:' ],
 	'lc'                        => [ '0', 'MALÁ:', 'LC:' ],
 	'uc'                        => [ '0', 'VELKÁ:', 'UC:' ],
 	'displaytitle'              => [ '1', 'ZOBRAZOVANÝNADPIS', 'DISPLAYTITLE' ],
 	'newsectionlink'            => [ '1', '__LINKPŘIDATKOMENTÁŘ__', '__NEWSECTIONLINK__' ],
+	'nonewsectionlink'          => [ '1', '__BEZLINKUPŘIDATKOMENTÁŘ__', '__NONEWSECTIONLINK__' ],
 	'currentversion'            => [ '1', 'VERZESOFTWARE', 'CURRENTVERSION' ],
 	'urlencode'                 => [ '0', 'ENKÓDOVATURL:', 'URLENCODE:' ],
 	'anchorencode'              => [ '0', 'ENKÓDOVATNADPIS', 'ANCHORENCODE' ],
 	'currenttimestamp'          => [ '1', 'AKTUÁLNÍKÓDČASU', 'CURRENTTIMESTAMP' ],
 	'localtimestamp'            => [ '1', 'MÍSTNÍKÓDČASU', 'LOCALTIMESTAMP' ],
+	'directionmark'             => [ '1', 'ZNAKSMĚRU', 'DIRECTIONMARK', 'DIRMARK' ],
 	'language'                  => [ '0', '#JAZYK:', '#LANGUAGE:' ],
 	'contentlanguage'           => [ '1', 'JAZYKOBSAHU', 'CONTENTLANGUAGE', 'CONTENTLANG' ],
+	'pagelanguage'              => [ '1', 'JAZYKSTRÁNKY', 'PAGELANGUAGE' ],
 	'pagesinnamespace'          => [ '1', 'STRÁNEKVEJMENNÉMPROSTORU:', 'PAGESINNAMESPACE:', 'PAGESINNS:' ],
 	'numberofadmins'            => [ '1', 'POČETSPRÁVCŮ', 'NUMBEROFADMINS' ],
 	'formatnum'                 => [ '0', 'FORMÁTUJČÍSLO', 'FORMATNUM' ],
@@ -268,7 +290,7 @@ $magicWords = [
 	'padright'                  => [ '0', 'ZAROVNATVPRAVO', 'PADRIGHT' ],
 	'special'                   => [ '0', 'speciální', 'special' ],
 	'defaultsort'               => [ '1', 'KLÍČŘAZENÍ:', 'DEFAULTSORT:', 'DEFAULTSORTKEY:', 'DEFAULTCATEGORYSORT:' ],
-	'filepath'                  => [ '0', 'CESTAKSOUBORU', 'FILEPATH:' ],
+	'filepath'                  => [ '0', 'CESTAKSOUBORU:', 'FILEPATH:' ],
 	'tag'                       => [ '0', 'značka', 'tag' ],
 	'hiddencat'                 => [ '1', '__SKRÝTKAT__', '__HIDDENCAT__' ],
 	'pagesincategory'           => [ '1', 'STRÁNEKVKATEGORII', 'STRÁNEKVKAT', 'PAGESINCATEGORY', 'PAGESINCAT' ],
@@ -276,7 +298,13 @@ $magicWords = [
 	'index'                     => [ '1', '__INDEXOVAT__', '__INDEX__' ],
 	'noindex'                   => [ '1', '__NEINDEXOVAT__', '__NOINDEX__' ],
 	'staticredirect'            => [ '1', '__STATICKÉPŘESMĚROVÁNÍ__', '__STATICREDIRECT__' ],
+	'numberingroup'             => [ '1', 'POČETVESKUPINĚ', 'NUMBERINGROUP', 'NUMINGROUP' ],
 	'protectionlevel'           => [ '1', 'ÚROVEŇZAMČENÍ', 'PROTECTIONLEVEL' ],
+	'protectionexpiry'          => [ '1', 'VYPRŠENÍZAMČENÍ', 'PROTECTIONEXPIRY' ],
+	'formatdate'                => [ '0', 'formátujdatum', 'formatdate', 'dateformat' ],
+	'pagesincategory_all'       => [ '0', 'vše', 'all' ],
+	'pagesincategory_pages'     => [ '0', 'stránky', 'pages' ],
+	'pagesincategory_subcats'   => [ '0', 'kategorie', 'subcats' ],
 	'pagesincategory_files'     => [ '0', 'soubory', 'files' ],
 ];
 
@@ -355,4 +383,4 @@ $bookstoreList = [
 # Písmena, která se mají objevit jako část odkazu ve formě '[[jazyk]]y' atd:
 $linkTrail = '/^([a-záčďéěíňóřšťúůýž]+)(.*)$/sDu';
 
-$separatorTransformTable = [ ',' => "\xc2\xa0", '.' => ',' ];
+$separatorTransformTable = [ ',' => "\u{00A0}", '.' => ',' ];
