@@ -33,19 +33,19 @@ $namespaceNames = [
 ];
 
 $digitTransformTable = [
-	'0' => '٠', # &#x0660;
-	'1' => '١', # &#x0661;
-	'2' => '٢', # &#x0662;
-	'3' => '٣', # &#x0663;
-	'4' => '٤', # &#x0664;
-	'5' => '٥', # &#x0665;
-	'6' => '٦', # &#x0666;
-	'7' => '٧', # &#x0667;
-	'8' => '٨', # &#x0668;
-	'9' => '٩', # &#x0669;
+	'0' => '٠', # U+0660
+	'1' => '١', # U+0661
+	'2' => '٢', # U+0662
+	'3' => '٣', # U+0663
+	'4' => '٤', # U+0664
+	'5' => '٥', # U+0665
+	'6' => '٦', # U+0666
+	'7' => '٧', # U+0667
+	'8' => '٨', # U+0668
+	'9' => '٩', # U+0669
 ];
 
 $separatorTransformTable = [
-	'.' => '٫', # &#x066b;
-	',' => '٬', # &#x066c;
+	'.' => '٫', # U+066B
+	',' => '٬', # U+066C
 ];
