@@ -28,14 +28,14 @@ $namespaceNames = [
 ];
 
 $digitTransformTable = [
-	'0' => '०', # &#x0966;
-	'1' => '१', # &#x0967;
-	'2' => '२', # &#x0968;
-	'3' => '३', # &#x0969;
-	'4' => '४', # &#x096a;
-	'5' => '५', # &#x096b;
-	'6' => '६', # &#x096c;
-	'7' => '७', # &#x096d;
-	'8' => '८', # &#x096e;
-	'9' => '९', # &#x096f;
+	'0' => '०', # U+0966
+	'1' => '१', # U+0967
+	'2' => '२', # U+0968
+	'3' => '३', # U+0969
+	'4' => '४', # U+096A
+	'5' => '५', # U+096B
+	'6' => '६', # U+096C
+	'7' => '७', # U+096D
+	'8' => '८', # U+096E
+	'9' => '९', # U+096F
 ];
