@@ -123,15 +123,9 @@ TEXT
 	protected function cleanupTable( $tableParams ) {
 		$table = $tableParams[0];
 		$prefix = $tableParams[1];
-		$idField = isset( $tableParams['idField'] ) ?
-			$tableParams['idField'] :
-			"{$prefix}_id";
-		$nsField = isset( $tableParams['nsField'] ) ?
-			$tableParams['nsField'] :
-			"{$prefix}_namespace";
-		$titleField = isset( $tableParams['titleField'] ) ?
-			$tableParams['titleField'] :
-			"{$prefix}_title";
+		$idField = $tableParams['idField'] ?? "{$prefix}_id";
+		$nsField = $tableParams['nsField'] ?? "{$prefix}_namespace";
+		$titleField = $tableParams['titleField'] ?? "{$prefix}_title";
 
 		$this->outputStatus( "Looking for invalid $titleField entries in $table...\n" );
 
@@ -161,7 +155,7 @@ TEXT
 				$titleField . $dbr->buildLike( $percent, '_' ),
 			], LIST_OR ) ],
 			__METHOD__,
-			[ 'LIMIT' => $this->mBatchSize ]
+			[ 'LIMIT' => $this->getBatchSize() ]
 		);
 
 		$this->outputStatus( "Number of invalid rows: " . $res->numRows() . "\n" );
@@ -292,7 +286,6 @@ TEXT
 		}
 
 		$this->outputStatus( "\n" );
-		return;
 	}
 
 	/**
@@ -307,5 +300,5 @@ TEXT
 	}
 }
 
-$maintClass = 'CleanupInvalidDbKeys';
+$maintClass = CleanupInvalidDbKeys::class;
 require_once RUN_MAINTENANCE_IF_MAIN;
