@@ -67,11 +67,11 @@ class GenerateCollationData extends Maintenance {
 
 		// As of January 2013, these links work for all versions of Unicode
 		// between 5.1 and 6.2, inclusive.
-		$allkeysURL = "http://www.unicode.org/Public/UCA/<Unicode version>/allkeys.txt";
-		$ucdallURL = "http://www.unicode.org/Public/<Unicode version>/ucdxml/ucd.all.grouped.zip";
+		$allkeysURL = "https://www.unicode.org/Public/UCA/<Unicode version>/allkeys.txt";
+		$ucdallURL = "https://www.unicode.org/Public/<Unicode version>/ucdxml/ucd.all.grouped.zip";
 
 		if ( !$allkeysPresent || !$ucdallPresent ) {
-			$icuVersion = IcuCollation::getICUVersion();
+			$icuVersion = INTL_ICU_VERSION;
 			$unicodeVersion = IcuCollation::getUnicodeVersionForICU();
 
 			$error = "";
@@ -88,16 +88,7 @@ class GenerateCollationData extends Maintenance {
 			}
 
 			$versionKnown = false;
-			if ( !$icuVersion ) {
-				// Unknown version - either very old intl,
-				// or PHP < 5.3.7 which does not expose this information
-				$error .= "As MediaWiki could not determine the version of ICU library used by your PHP's "
-					. "intl extension it can't suggest which file version to download. "
-					. "This can be caused by running a very old version of intl or PHP < 5.3.7. "
-					. "If you are sure everything is all right, find out the ICU version "
-					. "by running phpinfo(), check what is the Unicode version it is using "
-					. "at http://site.icu-project.org/download, then try finding appropriate data file(s) at:";
-			} elseif ( version_compare( $icuVersion, "4.0", "<" ) ) {
+			if ( version_compare( $icuVersion, "4.0", "<" ) ) {
 				// Extra old version
 				$error .= "You are using outdated version of ICU ($icuVersion), intended for "
 					. ( $unicodeVersion ? "Unicode $unicodeVersion" : "an unknown version of Unicode" )
@@ -131,16 +122,14 @@ class GenerateCollationData extends Maintenance {
 				$error .= "* $ucdallURL\n";
 			}
 
-			$this->error( $error );
-			exit( 1 );
+			$this->fatalError( $error );
 		}
 
 		$debugOutFileName = $this->getOption( 'debug-output' );
 		if ( $debugOutFileName ) {
 			$this->debugOutFile = fopen( $debugOutFileName, 'w' );
 			if ( !$this->debugOutFile ) {
-				$this->error( "Unable to open debug output file for writing" );
-				exit( 1 );
+				$this->fatalError( "Unable to open debug output file for writing" );
 			}
 		}
 		$this->loadUcd();
@@ -205,14 +194,7 @@ class GenerateCollationData extends Maintenance {
 	function generateFirstChars() {
 		$file = fopen( "{$this->dataDir}/allkeys.txt", 'r' );
 		if ( !$file ) {
-			$this->error( "Unable to open allkeys.txt" );
-			exit( 1 );
-		}
-		global $IP;
-		$outFile = fopen( "$IP/serialized/first-letters-root.ser", 'w' );
-		if ( !$outFile ) {
-			$this->error( "Unable to open output file first-letters-root.ser" );
-			exit( 1 );
+			$this->fatalError( "Unable to open allkeys.txt" );
 		}
 
 		$goodTertiaryChars = [];
@@ -281,7 +263,7 @@ class GenerateCollationData extends Maintenance {
 		// character has a longer primary weight sequence with an initial
 		// portion equal to the first character, then remove the second
 		// character. This avoids having characters like U+A732 (double A)
-		// polluting the basic latin sort area.
+		// polluting the basic Latin sort area.
 
 		foreach ( $this->groups as $weight => $group ) {
 			if ( preg_match( '/(\.[0-9A-F]*)\./', $weight, $m ) ) {
@@ -327,11 +309,6 @@ class GenerateCollationData extends Maintenance {
 			$headerChars[] = $char;
 			if ( $primaryCollator->compare( $char, $prevChar ) <= 0 ) {
 				$numOutOfOrder++;
-				/*
-				printf( "Out of order: U+%05X > U+%05X\n",
-					utf8ToCodepoint( $prevChar ),
-					utf8ToCodepoint( $char ) );
-				 */
 			}
 			$prevChar = $char;
 
@@ -343,7 +320,13 @@ class GenerateCollationData extends Maintenance {
 
 		print "Out of order: $numOutOfOrder / " . count( $headerChars ) . "\n";
 
-		fwrite( $outFile, serialize( $headerChars ) );
+		global $IP;
+		$writer = new StaticArrayWriter();
+		file_put_contents(
+			"$IP/includes/collation/data/first-letters-root.php",
+			$writer->create( $headerChars, 'File created by generateCollationData.php' )
+		);
+		echo "first-letters-root: file written.\n";
 	}
 }
 
@@ -468,5 +451,5 @@ class UcdXmlReader {
 	}
 }
 
-$maintClass = 'GenerateCollationData';
+$maintClass = GenerateCollationData::class;
 require_once RUN_MAINTENANCE_IF_MAIN;
