@@ -1,4 +1,7 @@
 <?php
+
+use MediaWiki\MediaWikiServices;
+
 /**
  * Parse some wikitext.
  *
@@ -46,7 +49,7 @@
  * @file
  * @ingroup Maintenance
  * @author Antoine Musso <hashar at free dot fr>
- * @license GNU General Public License 2.0 or later
+ * @license GPL-2.0-or-later
  */
 
 require_once __DIR__ . '/Maintenance.php';
@@ -68,7 +71,7 @@ class CLIParser extends Maintenance {
 			false,
 			true
 		);
-		$this->addOption( 'tidy', 'Tidy the output' );
+		$this->addOption( 'no-tidy', 'Don\'t tidy the output (deprecated)' );
 		$this->addArg( 'file', 'File containing wikitext (Default: stdin)', false );
 	}
 
@@ -82,7 +85,7 @@ class CLIParser extends Maintenance {
 	 * @return string HTML Rendering
 	 */
 	public function render( $wikitext ) {
-		return $this->parse( $wikitext )->getText();
+		return $this->parse( $wikitext )->getText( [ 'wrapperDivClass' => '' ] );
 	}
 
 	/**
@@ -103,9 +106,7 @@ class CLIParser extends Maintenance {
 	}
 
 	protected function initParser() {
-		global $wgParserConf;
-		$parserClass = $wgParserConf['class'];
-		$this->parser = new $parserClass();
+		$this->parser = MediaWikiServices::getInstance()->getParserFactory()->create();
 	}
 
 	/**
@@ -128,9 +129,10 @@ class CLIParser extends Maintenance {
 	 * @return ParserOutput
 	 */
 	protected function parse( $wikitext ) {
-		$options = new ParserOptions;
-		if ( $this->getOption( 'tidy' ) ) {
-			$options->setTidy( true );
+		$options = ParserOptions::newCanonical();
+		$options->setOption( 'enableLimitReport', false );
+		if ( $this->getOption( 'no-tidy' ) ) {
+			$options->setTidy( false );
 		}
 		return $this->parser->parse(
 			$wikitext,
@@ -140,5 +142,5 @@ class CLIParser extends Maintenance {
 	}
 }
 
-$maintClass = "CLIParser";
+$maintClass = CLIParser::class;
 require_once RUN_MAINTENANCE_IF_MAIN;
