@@ -1,7 +1,7 @@
 /*
  * Javascript for module editWarning
  */
-( function ( mw, $ ) {
+( function () {
 	'use strict';
 
 	$( function () {
@@ -35,8 +35,8 @@
 
 		// Add form submission handler
 		$( '#editform' ).submit( function () {
-			allowCloseWindow();
+			allowCloseWindow.release();
 		} );
 	} );
 
-}( mediaWiki, jQuery ) );
+}() );
