@@ -1,4 +1,4 @@
-( function ( mw, $ ) {
+( function () {
 	'use strict';
 
 	/**
@@ -38,7 +38,12 @@
 		data = data || {};
 
 		if ( data.message === undefined ) {
-			data.message = $.parseHTML( mw.message( 'postedit-confirmation-saved', data.user || mw.user ).escaped() );
+			data.message = $.parseHTML( mw.message(
+				mw.config.get( 'wgEditSubmitButtonLabelPublish' ) ?
+					'postedit-confirmation-published' :
+					'postedit-confirmation-saved',
+				data.user || mw.user
+			).escaped() );
 		}
 
 		$content = $( '<div>' ).addClass( 'postedit-icon postedit-icon-checkmark postedit-content' );
@@ -75,4 +80,4 @@
 		} );
 	}
 
-}( mediaWiki, jQuery ) );
+}() );
