@@ -1,9 +1,10 @@
-( function ( mw ) {
+( function () {
 	/**
 	 * Supported highlight colors.
 	 * Warning: These are also hardcoded in "styles/mw.rcfilters.variables.less"
 	 *
-	 * @type {string[]}
+	 * @member mw.rcfilters
+	 * @property {string[]}
 	 */
 	mw.rcfilters.HighlightColors = [ 'c1', 'c2', 'c3', 'c4', 'c5' ];
-}( mediaWiki ) );
+}() );
