@@ -1,4 +1,4 @@
-( function ( mw ) {
+( function () {
 	/**
 	 * @class
 	 * @singleton
@@ -44,10 +44,6 @@
 
 				return result;
 			}
-		},
-		featureFlags: {
-			liveUpdate: mw.config.get( 'wgStructuredChangeFiltersLiveUpdateSupported' ) &&
-				( mw.user.options.get( 'rcenhancedfilters' ) || new mw.Uri().query.liveupdate )
 		}
 	};
-}( mediaWiki ) );
+}() );
