@@ -1,4 +1,4 @@
-( function ( mw ) {
+( function () {
 	/**
 	 * A widget representing a single toggle filter
 	 *
@@ -6,17 +6,23 @@
 	 *
 	 * @constructor
 	 * @param {mw.rcfilters.Controller} controller RCFilters controller
-	 * @param {mw.rcfilters.dm.FilterItem} model Filter item model
+	 * @param {mw.rcfilters.dm.FiltersViewModel} filtersViewModel
+	 * @param {mw.rcfilters.dm.FilterItem} invertModel
+	 * @param {mw.rcfilters.dm.FilterItem} itemModel Filter item model
+	 * @param {mw.rcfilters.ui.HighlightPopupWidget} highlightPopup Shared highlight color picker popup
 	 * @param {Object} config Configuration object
 	 */
-	mw.rcfilters.ui.FilterMenuOptionWidget = function MwRcfiltersUiFilterMenuOptionWidget( controller, model, config ) {
+	mw.rcfilters.ui.FilterMenuOptionWidget = function MwRcfiltersUiFilterMenuOptionWidget(
+		controller, filtersViewModel, invertModel, itemModel, highlightPopup, config
+	) {
 		config = config || {};
 
 		this.controller = controller;
-		this.model = model;
+		this.invertModel = invertModel;
+		this.model = itemModel;
 
 		// Parent
-		mw.rcfilters.ui.FilterMenuOptionWidget.parent.call( this, controller, model, config );
+		mw.rcfilters.ui.FilterMenuOptionWidget.parent.call( this, controller, filtersViewModel, this.invertModel, itemModel, highlightPopup, config );
 
 		// Event
 		this.model.getGroupModel().connect( this, { update: 'onGroupModelUpdate' } );
@@ -38,9 +44,9 @@
 	/**
 	 * @inheritdoc
 	 */
-	mw.rcfilters.ui.FilterMenuOptionWidget.prototype.onModelUpdate = function () {
+	mw.rcfilters.ui.FilterMenuOptionWidget.prototype.updateUiBasedOnState = function () {
 		// Parent
-		mw.rcfilters.ui.FilterMenuOptionWidget.parent.prototype.onModelUpdate.call( this );
+		mw.rcfilters.ui.FilterMenuOptionWidget.parent.prototype.updateUiBasedOnState.call( this );
 
 		this.setCurrentMuteState();
 	};
@@ -58,7 +64,7 @@
 	mw.rcfilters.ui.FilterMenuOptionWidget.prototype.setCurrentMuteState = function () {
 		if (
 			this.model.getGroupModel().getView() === 'namespaces' &&
-			this.model.isInverted()
+			this.invertModel.isSelected()
 		) {
 			// This is an inverted behavior than the other rules, specifically
 			// for inverted namespaces
@@ -81,4 +87,4 @@
 			} );
 		}
 	};
-}( mediaWiki ) );
+}() );
