@@ -1,4 +1,4 @@
-( function ( mw ) {
+( function () {
 	/**
 	 * List displaying all filter groups
 	 *
@@ -12,6 +12,9 @@
 	 * @param {mw.rcfilters.dm.SavedQueriesModel} savedQueriesModel Saved queries model
 	 * @param {Object} config Configuration object
 	 * @cfg {jQuery} [$overlay] A jQuery object serving as overlay for popups
+	 * @cfg {jQuery} [$wrapper] A jQuery object for the wrapper of the general
+	 *  system. If not given, falls back to this widget's $element
+	 * @cfg {boolean} [collapsed] Filter area is collapsed
 	 */
 	mw.rcfilters.ui.FilterTagMultiselectWidget = function MwRcfiltersUiFilterTagMultiselectWidget( controller, model, savedQueriesModel, config ) {
 		var rcFiltersRow,
@@ -28,8 +31,10 @@
 		this.model = model;
 		this.queriesModel = savedQueriesModel;
 		this.$overlay = config.$overlay || this.$element;
+		this.$wrapper = config.$wrapper || this.$element;
 		this.matchingQuery = null;
 		this.currentView = this.model.getCurrentView();
+		this.collapsed = false;
 
 		// Parent
 		mw.rcfilters.ui.FilterTagMultiselectWidget.parent.call( this, $.extend( true, {
@@ -41,6 +46,8 @@
 			allowReordering: false,
 			$overlay: this.$overlay,
 			menu: {
+				// Our filtering is done through the model
+				filterFromInput: false,
 				hideWhenOutOfView: false,
 				hideOnChoose: false,
 				width: 650,
@@ -85,10 +92,20 @@
 			classes: [ 'mw-rcfilters-ui-filterTagMultiselectWidget-resetButton' ]
 		} );
 
+		this.hideShowButton = new OO.ui.ButtonWidget( {
+			framed: false,
+			flags: [ 'progressive' ],
+			classes: [ 'mw-rcfilters-ui-filterTagMultiselectWidget-hideshowButton' ]
+		} );
+		this.toggleCollapsed( !!config.collapsed );
+
 		if ( !mw.user.isAnon() ) {
 			this.saveQueryButton = new mw.rcfilters.ui.SaveFiltersPopupButtonWidget(
 				this.controller,
-				this.queriesModel
+				this.queriesModel,
+				{
+					$overlay: this.$overlay
+				}
 			);
 
 			this.saveQueryButton.$element.on( 'mousedown', function ( e ) { e.stopPropagation(); } );
@@ -112,12 +129,15 @@
 
 		// Events
 		this.resetButton.connect( this, { click: 'onResetButtonClick' } );
+		this.hideShowButton.connect( this, { click: 'onHideShowButtonClick' } );
 		// Stop propagation for mousedown, so that the widget doesn't
 		// trigger the focus on the input and scrolls up when we click the reset button
 		this.resetButton.$element.on( 'mousedown', function ( e ) { e.stopPropagation(); } );
+		this.hideShowButton.$element.on( 'mousedown', function ( e ) { e.stopPropagation(); } );
 		this.model.connect( this, {
 			initialize: 'onModelInitialize',
 			update: 'onModelUpdate',
+			searchChange: 'onModelSearchChange',
 			itemUpdate: 'onModelItemUpdate',
 			highlightChange: 'onModelHighlightChange'
 		} );
@@ -148,23 +168,18 @@
 		this.viewsSelectWidget = new OO.ui.ButtonSelectWidget( {
 			classes: [ 'mw-rcfilters-ui-filterTagMultiselectWidget-views-select-widget' ],
 			items: [
-				new OO.ui.ButtonOptionWidget( {
-					framed: false,
-					data: '',
-					disabled: true,
-					classes: [ 'mw-rcfilters-ui-filterTagMultiselectWidget-views-select-widget-label' ],
-					label: mw.msg( 'rcfilters-view-advanced-filters-label' )
-				} ),
 				new OO.ui.ButtonOptionWidget( {
 					framed: false,
 					data: 'namespaces',
 					icon: 'article',
+					label: mw.msg( 'namespaces' ),
 					title: mw.msg( 'rcfilters-view-namespaces-tooltip' )
 				} ),
 				new OO.ui.ButtonOptionWidget( {
 					framed: false,
 					data: 'tags',
 					icon: 'tag',
+					label: mw.msg( 'tags-title' ),
 					title: mw.msg( 'rcfilters-view-tags-tooltip' )
 				} )
 			]
@@ -177,6 +192,7 @@
 				.append(
 					$( '<div>' )
 						.addClass( 'mw-rcfilters-ui-row' )
+						.addClass( 'mw-rcfilters-ui-filterTagMultiselectWidget-views' )
 						.append(
 							$( '<div>' )
 								.addClass( 'mw-rcfilters-ui-cell' )
@@ -202,13 +218,25 @@
 
 		// Build the content
 		$contentWrapper.append(
-			title.$element,
-			this.savedQueryTitle.$element,
+			$( '<div>' )
+				.addClass( 'mw-rcfilters-ui-filterTagMultiselectWidget-wrapper-top' )
+				.append(
+					$( '<div>' )
+						.addClass( 'mw-rcfilters-ui-filterTagMultiselectWidget-wrapper-top-title' )
+						.append( title.$element ),
+					$( '<div>' )
+						.addClass( 'mw-rcfilters-ui-filterTagMultiselectWidget-wrapper-top-queryName' )
+						.append( this.savedQueryTitle.$element ),
+					$( '<div>' )
+						.addClass( 'mw-rcfilters-ui-filterTagMultiselectWidget-wrapper-top-hideshow' )
+						.append(
+							this.hideShowButton.$element
+						)
+				),
 			$( '<div>' )
 				.addClass( 'mw-rcfilters-ui-table' )
-				.append(
-					rcFiltersRow
-				)
+				.addClass( 'mw-rcfilters-ui-filterTagMultiselectWidget-wrapper-filters' )
+				.append( rcFiltersRow )
 		);
 
 		// Initialize
@@ -228,6 +256,11 @@
 
 	/* Methods */
 
+	/**
+	 * Override parent method to avoid unnecessary resize events.
+	 */
+	mw.rcfilters.ui.FilterTagMultiselectWidget.prototype.updateIfHeightChanged = function () { };
+
 	/**
 	 * Respond to view select widget choose event
 	 *
@@ -239,20 +272,24 @@
 		this.focus();
 	};
 
+	/**
+	 * Respond to model search change event
+	 *
+	 * @param {string} value Search value
+	 */
+	mw.rcfilters.ui.FilterTagMultiselectWidget.prototype.onModelSearchChange = function ( value ) {
+		this.input.setValue( value );
+	};
+
 	/**
 	 * Respond to input change event
 	 *
 	 * @param {string} value Value of the input
 	 */
 	mw.rcfilters.ui.FilterTagMultiselectWidget.prototype.onInputChange = function ( value ) {
-		var view;
-
-		value = value.trim();
-
-		view = this.model.getViewByTrigger( value.substr( 0, 1 ) );
-
-		this.controller.switchView( view );
+		this.controller.setSearch( value );
 	};
+
 	/**
 	 * Respond to query button click
 	 */
@@ -290,25 +327,24 @@
 		mw.rcfilters.ui.FilterTagMultiselectWidget.parent.prototype.onMenuToggle.call( this );
 
 		if ( isVisible ) {
+			this.focus();
+
 			mw.hook( 'RcFilters.popup.open' ).fire();
 
-			if ( !this.getMenu().getSelectedItem() ) {
+			if ( !this.getMenu().findSelectedItem() ) {
 				// If there are no selected items, scroll menu to top
 				// This has to be in a setTimeout so the menu has time
 				// to be positioned and fixed
 				setTimeout( function () { this.getMenu().scrollToTop(); }.bind( this ), 0 );
 			}
 		} else {
+			this.blur();
+
 			// Clear selection
 			this.selectTag( null );
 
-			// Clear input if the only thing in the input is the prefix
-			if (
-				this.input.getValue().trim() === this.model.getViewTrigger( this.model.getCurrentView() )
-			) {
-				// Clear the input
-				this.input.setValue( '' );
-			}
+			// Clear the search
+			this.controller.setSearch( '' );
 
 			// Log filter grouping
 			this.controller.trackFilterGroupings( 'filtermenu' );
@@ -342,12 +378,25 @@
 		this.input.$input.blur();
 	};
 
+	/**
+	 * @inheritdoc
+	 */
+	mw.rcfilters.ui.FilterTagMultiselectWidget.prototype.onMouseDown = function ( e ) {
+		if ( !this.collapsed && !this.isDisabled() && e.which === OO.ui.MouseButtons.LEFT ) {
+			this.menu.toggle();
+
+			return false;
+		}
+	};
+
 	/**
 	 * @inheritdoc
 	 */
 	mw.rcfilters.ui.FilterTagMultiselectWidget.prototype.onChangeTags = function () {
-		// Parent method
-		mw.rcfilters.ui.FilterTagMultiselectWidget.parent.prototype.onChangeTags.call( this );
+		// If initialized, call parent method.
+		if ( this.controller.isInitialized() ) {
+			mw.rcfilters.ui.FilterTagMultiselectWidget.parent.prototype.onChangeTags.call( this );
+		}
 
 		this.emptyFilterMessage.toggle( this.isEmpty() );
 	};
@@ -407,7 +456,10 @@
 			this.matchingQuery ? this.matchingQuery.getLabel() : ''
 		);
 		this.savedQueryTitle.toggle( !!this.matchingQuery );
-		this.saveQueryButton.toggle( !this.matchingQuery );
+		this.saveQueryButton.setDisabled( !!this.matchingQuery );
+		this.saveQueryButton.setTitle( !this.matchingQuery ?
+			mw.msg( 'rcfilters-savedqueries-add-new-title' ) :
+			mw.msg( 'rcfilters-savedqueries-already-saved' ) );
 
 		if ( this.matchingQuery ) {
 			this.emphasize();
@@ -416,25 +468,26 @@
 
 	/**
 	 * Respond to model itemUpdate event
+	 * fixme: when a new state is applied to the model this function is called 60+ times in a row
 	 *
 	 * @param {mw.rcfilters.dm.FilterItem} item Filter item model
 	 */
 	mw.rcfilters.ui.FilterTagMultiselectWidget.prototype.onModelItemUpdate = function ( item ) {
-		if ( item.getGroupModel().isHidden() ) {
-			return;
-		}
-
-		if (
-			item.isSelected() ||
-			(
-				this.model.isHighlightEnabled() &&
-				item.isHighlightSupported() &&
-				item.getHighlightColor()
-			)
-		) {
-			this.addTag( item.getName(), item.getLabel() );
-		} else {
-			this.removeTagByData( item.getName() );
+		if ( !item.getGroupModel().isHidden() ) {
+			if (
+				item.isSelected() ||
+				(
+					this.model.isHighlightEnabled() &&
+					item.getHighlightColor()
+				)
+			) {
+				this.addTag( item.getName(), item.getLabel() );
+			} else {
+				// Only attempt to remove the tag if we can find an item for it (T198140, T198231)
+				if ( this.findItemFromData( item.getName() ) !== null ) {
+					this.removeTagByData( item.getName() );
+				}
+			}
 		}
 
 		this.setSavedQueryVisibility();
@@ -460,7 +513,7 @@
 		this.controller.toggleFilterSelect( item.model.getName() );
 
 		// Select the tag if it exists, or reset selection otherwise
-		this.selectTag( this.getItemFromData( item.model.getName() ) );
+		this.selectTag( this.findItemFromData( item.model.getName() ) );
 
 		this.focus();
 	};
@@ -482,52 +535,34 @@
 			// Remove capsule widgets if they're not selected
 			highlightedItems.forEach( function ( filterItem ) {
 				if ( !filterItem.isSelected() ) {
-					this.removeTagByData( filterItem.getName() );
+					// Only attempt to remove the tag if we can find an item for it (T198140, T198231)
+					if ( this.findItemFromData( filterItem.getName() ) !== null ) {
+						this.removeTagByData( filterItem.getName() );
+					}
 				}
 			}.bind( this ) );
 		}
+
+		this.setSavedQueryVisibility();
 	};
 
 	/**
 	 * @inheritdoc
 	 */
 	mw.rcfilters.ui.FilterTagMultiselectWidget.prototype.onTagSelect = function ( tagItem ) {
-		var widget = this,
-			menuOption = this.menu.getItemFromModel( tagItem.getModel() ),
-			oldInputValue = this.input.getValue().trim();
+		var menuOption = this.menu.getItemFromModel( tagItem.getModel() );
 
 		this.menu.setUserSelecting( true );
-
-		// Reset input
-		this.input.setValue( '' );
-
-		// Switch view
-		this.controller.switchView( tagItem.getView() );
-
 		// Parent method
 		mw.rcfilters.ui.FilterTagMultiselectWidget.parent.prototype.onTagSelect.call( this, tagItem );
 
-		this.menu.selectItem( menuOption );
+		// Switch view
+		this.controller.resetSearchForView( tagItem.getView() );
+
 		this.selectTag( tagItem );
+		this.scrollToTop( menuOption.$element );
 
-		// Scroll to the item
-		if ( this.model.removeViewTriggers( oldInputValue ) ) {
-			// We're binding a 'once' to the itemVisibilityChange event
-			// so this happens when the menu is ready after the items
-			// are visible again, in case this is done right after the
-			// user filtered the results
-			this.getMenu().once(
-				'itemVisibilityChange',
-				function () {
-					widget.scrollToTop( menuOption.$element );
-					widget.menu.setUserSelecting( false );
-				}
-			);
-		} else {
-			this.scrollToTop( menuOption.$element );
-			this.menu.setUserSelecting( false );
-		}
-
+		this.menu.setUserSelecting( false );
 	};
 
 	/**
@@ -563,7 +598,7 @@
 	 * Respond to click event on the reset button
 	 */
 	mw.rcfilters.ui.FilterTagMultiselectWidget.prototype.onResetButtonClick = function () {
-		if ( this.model.areCurrentFiltersEmpty() ) {
+		if ( this.model.areVisibleFiltersEmpty() ) {
 			// Reset to default filters
 			this.controller.resetToDefaults();
 		} else {
@@ -572,12 +607,51 @@
 		}
 	};
 
+	/**
+	 * Respond to hide/show button click
+	 */
+	mw.rcfilters.ui.FilterTagMultiselectWidget.prototype.onHideShowButtonClick = function () {
+		this.toggleCollapsed();
+	};
+
+	/**
+	 * Toggle the collapsed state of the filters widget
+	 *
+	 * @param {boolean} isCollapsed Widget is collapsed
+	 */
+	mw.rcfilters.ui.FilterTagMultiselectWidget.prototype.toggleCollapsed = function ( isCollapsed ) {
+		isCollapsed = isCollapsed === undefined ? !this.collapsed : !!isCollapsed;
+
+		this.collapsed = isCollapsed;
+
+		if ( isCollapsed ) {
+			// If we are collapsing, close the menu, in case it was open
+			// We should make sure the menu closes before the rest of the elements
+			// are hidden, otherwise there is an unknown error in jQuery as ooui
+			// sets and unsets properties on the input (which is hidden at that point)
+			this.menu.toggle( false );
+		}
+		this.input.setDisabled( isCollapsed );
+		this.hideShowButton.setLabel( mw.msg(
+			isCollapsed ? 'rcfilters-activefilters-show' : 'rcfilters-activefilters-hide'
+		) );
+		this.hideShowButton.setTitle( mw.msg(
+			isCollapsed ? 'rcfilters-activefilters-show-tooltip' : 'rcfilters-activefilters-hide-tooltip'
+		) );
+
+		// Toggle the wrapper class, so we have min height values correctly throughout
+		this.$wrapper.toggleClass( 'mw-rcfilters-collapsed', isCollapsed );
+
+		// Save the state
+		this.controller.updateCollapsedState( isCollapsed );
+	};
+
 	/**
 	 * Reevaluate the restore state for the widget between setting to defaults and clearing all filters
 	 */
 	mw.rcfilters.ui.FilterTagMultiselectWidget.prototype.reevaluateResetRestoreState = function () {
 		var defaultsAreEmpty = this.controller.areDefaultsEmpty(),
-			currFiltersAreEmpty = this.model.areCurrentFiltersEmpty(),
+			currFiltersAreEmpty = this.model.areVisibleFiltersEmpty(),
 			hideResetButton = currFiltersAreEmpty && defaultsAreEmpty;
 
 		this.resetButton.setIcon(
@@ -602,9 +676,7 @@
 		return new mw.rcfilters.ui.MenuSelectWidget(
 			this.controller,
 			this.model,
-			$.extend( {
-				filterFromInput: true
-			}, menuConfig )
+			menuConfig
 		);
 	};
 
@@ -617,6 +689,8 @@
 		if ( filterItem ) {
 			return new mw.rcfilters.ui.FilterTagItemWidget(
 				this.controller,
+				this.model,
+				this.model.getInvertModel(),
 				filterItem,
 				{
 					$overlay: this.$overlay
@@ -682,4 +756,4 @@
 			} );
 		}
 	};
-}( mediaWiki ) );
+}() );
