@@ -1,4 +1,5 @@
-( function ( $, mw ) {
+/* eslint-disable no-restricted-properties */
+( function () {
 
 	/**
 	 * Provides various methods needed for formatting dates and times.
@@ -412,7 +413,7 @@
 		if ( v.normalize ) {
 			v = v.normalize();
 		}
-		re = new RegExp( '^\\s*' + v.replace( /([\\{}()|.?*+\-\^$\[\]])/g, '\\$1' ), 'i' );
+		re = new RegExp( '^\\s*' + v.replace( /([\\{}()|.?*+\-^$\[\]])/g, '\\$1' ), 'i' ); // eslint-disable-line no-useless-escape
 		for ( k in this.values ) {
 			k = +k;
 			if ( !isNaN( k ) && re.test( this.values[ k ] ) ) {
@@ -624,4 +625,4 @@
 		};
 	};
 
-}( jQuery, mediaWiki ) );
+}() );
