@@ -1,4 +1,5 @@
-( function ( $, mw ) {
+/* eslint-disable no-restricted-properties */
+( function () {
 
 	/**
 	 * Provides various methods needed for formatting dates and times. This
@@ -37,25 +38,25 @@
 			config.shortMonthNames = {};
 			$.each( config.fullMonthNames, function ( k, v ) {
 				config.shortMonthNames[ k ] = v.substr( 0, 3 );
-			}.bind( this ) );
+			} );
 		}
 		if ( config.shortDayNames && !config.dayLetters ) {
 			config.dayLetters = [];
 			$.each( config.shortDayNames, function ( k, v ) {
 				config.dayLetters[ k ] = v.substr( 0, 1 );
-			}.bind( this ) );
+			} );
 		}
 		if ( config.fullDayNames && !config.dayLetters ) {
 			config.dayLetters = [];
 			$.each( config.fullDayNames, function ( k, v ) {
 				config.dayLetters[ k ] = v.substr( 0, 1 );
-			}.bind( this ) );
+			} );
 		}
 		if ( config.fullDayNames && !config.shortDayNames ) {
 			config.shortDayNames = {};
 			$.each( config.fullDayNames, function ( k, v ) {
 				config.shortDayNames[ k ] = v.substr( 0, 3 );
-			}.bind( this ) );
+			} );
 		}
 		config = $.extend( {
 			fullMonthNames: this.constructor.static.fullMonthNames,
@@ -672,4 +673,4 @@
 		return ret;
 	};
 
-}( jQuery, mediaWiki ) );
+}() );
