@@ -1,4 +1,4 @@
--- This is a copy of MediaWiki 1.20 schema shared by MySQL and SQLite.
+-- This is a copy of MediaWiki 1.23 schema shared by MySQL and SQLite.
 -- It is used for updater testing. Comments are stripped to decrease
 -- file size, as we don't need to maintain it.
 
@@ -16,20 +16,21 @@ CREATE TABLE /*_*/user (
   user_email_token binary(32),
   user_email_token_expires binary(14),
   user_registration binary(14),
-  user_editcount int
+  user_editcount int,
+  user_password_expires varbinary(14) DEFAULT NULL
 ) /*$wgDBTableOptions*/;
 CREATE UNIQUE INDEX /*i*/user_name ON /*_*/user (user_name);
 CREATE INDEX /*i*/user_email_token ON /*_*/user (user_email_token);
 CREATE INDEX /*i*/user_email ON /*_*/user (user_email(50));
 CREATE TABLE /*_*/user_groups (
   ug_user int unsigned NOT NULL default 0,
-  ug_group varbinary(32) NOT NULL default ''
+  ug_group varbinary(255) NOT NULL default ''
 ) /*$wgDBTableOptions*/;
 CREATE UNIQUE INDEX /*i*/ug_user_group ON /*_*/user_groups (ug_user,ug_group);
 CREATE INDEX /*i*/ug_group ON /*_*/user_groups (ug_group);
 CREATE TABLE /*_*/user_former_groups (
   ufg_user int unsigned NOT NULL default 0,
-  ufg_group varbinary(32) NOT NULL default ''
+  ufg_group varbinary(255) NOT NULL default ''
 ) /*$wgDBTableOptions*/;
 CREATE UNIQUE INDEX /*i*/ufg_user_group ON /*_*/user_former_groups (ufg_user,ufg_group);
 CREATE TABLE /*_*/user_newtalk (
@@ -56,8 +57,10 @@ CREATE TABLE /*_*/page (
   page_is_new tinyint unsigned NOT NULL default 0,
   page_random real unsigned NOT NULL,
   page_touched binary(14) NOT NULL default '',
+  page_links_updated varbinary(14) NULL default NULL,
   page_latest int unsigned NOT NULL,
-  page_len int unsigned NOT NULL
+  page_len int unsigned NOT NULL,
+  page_content_model varbinary(32) DEFAULT NULL
 ) /*$wgDBTableOptions*/;
 CREATE UNIQUE INDEX /*i*/name_title ON /*_*/page (page_namespace,page_title);
 CREATE INDEX /*i*/page_random ON /*_*/page (page_random);
@@ -75,7 +78,9 @@ CREATE TABLE /*_*/revision (
   rev_deleted tinyint unsigned NOT NULL default 0,
   rev_len int unsigned,
   rev_parent_id int unsigned default NULL,
-  rev_sha1 varbinary(32) NOT NULL default ''
+  rev_sha1 varbinary(32) NOT NULL default '',
+  rev_content_model varbinary(32) DEFAULT NULL,
+  rev_content_format varbinary(64) DEFAULT NULL
 ) /*$wgDBTableOptions*/ MAX_ROWS=10000000 AVG_ROW_LENGTH=1024;
 CREATE UNIQUE INDEX /*i*/rev_page_id ON /*_*/revision (rev_page, rev_id);
 CREATE INDEX /*i*/rev_timestamp ON /*_*/revision (rev_timestamp);
@@ -89,6 +94,7 @@ CREATE TABLE /*_*/text (
   old_flags tinyblob NOT NULL
 ) /*$wgDBTableOptions*/ MAX_ROWS=10000000 AVG_ROW_LENGTH=10240;
 CREATE TABLE /*_*/archive (
+  ar_id int unsigned NOT NULL PRIMARY KEY AUTO_INCREMENT,
   ar_namespace int NOT NULL default 0,
   ar_title varchar(255) binary NOT NULL default '',
   ar_text mediumblob NOT NULL,
@@ -104,7 +110,9 @@ CREATE TABLE /*_*/archive (
   ar_len int unsigned,
   ar_page_id int unsigned,
   ar_parent_id int unsigned default NULL,
-  ar_sha1 varbinary(32) NOT NULL default ''
+  ar_sha1 varbinary(32) NOT NULL default '',
+  ar_content_model varbinary(32) DEFAULT NULL,
+  ar_content_format varbinary(64) DEFAULT NULL
 ) /*$wgDBTableOptions*/;
 CREATE INDEX /*i*/name_title_timestamp ON /*_*/archive (ar_namespace,ar_title,ar_timestamp);
 CREATE INDEX /*i*/ar_usertext_timestamp ON /*_*/archive (ar_user_text,ar_timestamp);
@@ -152,6 +160,7 @@ CREATE TABLE /*_*/category (
 CREATE UNIQUE INDEX /*i*/cat_title ON /*_*/category (cat_title);
 CREATE INDEX /*i*/cat_pages ON /*_*/category (cat_pages);
 CREATE TABLE /*_*/externallinks (
+  el_id int unsigned NOT NULL PRIMARY KEY AUTO_INCREMENT,
   el_from int unsigned NOT NULL default 0,
   el_to blob NOT NULL,
   el_index blob NOT NULL
@@ -159,11 +168,6 @@ CREATE TABLE /*_*/externallinks (
 CREATE INDEX /*i*/el_from ON /*_*/externallinks (el_from, el_to(40));
 CREATE INDEX /*i*/el_to ON /*_*/externallinks (el_to(60), el_from);
 CREATE INDEX /*i*/el_index ON /*_*/externallinks (el_index(60));
-CREATE TABLE /*_*/external_user (
-  eu_local_id int unsigned NOT NULL PRIMARY KEY,
-  eu_external_id varchar(255) binary NOT NULL
-) /*$wgDBTableOptions*/;
-CREATE UNIQUE INDEX /*i*/eu_external_id ON /*_*/external_user (eu_external_id);
 CREATE TABLE /*_*/langlinks (
   ll_from int unsigned NOT NULL default 0,
   ll_lang varbinary(20) NOT NULL default '',
@@ -177,7 +181,8 @@ CREATE TABLE /*_*/iwlinks (
   iwl_title varchar(255) binary NOT NULL default ''
 ) /*$wgDBTableOptions*/;
 CREATE UNIQUE INDEX /*i*/iwl_from ON /*_*/iwlinks (iwl_from, iwl_prefix, iwl_title);
-CREATE UNIQUE INDEX /*i*/iwl_prefix_title_from ON /*_*/iwlinks (iwl_prefix, iwl_title, iwl_from);
+CREATE INDEX /*i*/iwl_prefix_title_from ON /*_*/iwlinks (iwl_prefix, iwl_title, iwl_from);
+CREATE INDEX /*i*/iwl_prefix_from_title ON /*_*/iwlinks (iwl_prefix, iwl_from, iwl_title);
 CREATE TABLE /*_*/site_stats (
   ss_row_id int unsigned NOT NULL,
   ss_total_views bigint unsigned default 0,
@@ -186,7 +191,6 @@ CREATE TABLE /*_*/site_stats (
   ss_total_pages bigint default '-1',
   ss_users bigint default '-1',
   ss_active_users bigint default '-1',
-  ss_admins int default '-1',
   ss_images int default 0
 ) /*$wgDBTableOptions*/;
 CREATE UNIQUE INDEX /*i*/ss_row_id ON /*_*/site_stats (ss_row_id);
@@ -238,7 +242,8 @@ CREATE TABLE /*_*/image (
 CREATE INDEX /*i*/img_usertext_timestamp ON /*_*/image (img_user_text,img_timestamp);
 CREATE INDEX /*i*/img_size ON /*_*/image (img_size);
 CREATE INDEX /*i*/img_timestamp ON /*_*/image (img_timestamp);
-CREATE INDEX /*i*/img_sha1 ON /*_*/image (img_sha1);
+CREATE INDEX /*i*/img_sha1 ON /*_*/image (img_sha1(10));
+CREATE INDEX /*i*/img_media_mime ON /*_*/image (img_media_type,img_major_mime,img_minor_mime);
 CREATE TABLE /*_*/oldimage (
   oi_name varchar(255) binary NOT NULL default '',
   oi_archive_name varchar(255) binary NOT NULL default '',
@@ -260,7 +265,7 @@ CREATE TABLE /*_*/oldimage (
 CREATE INDEX /*i*/oi_usertext_timestamp ON /*_*/oldimage (oi_user_text,oi_timestamp);
 CREATE INDEX /*i*/oi_name_timestamp ON /*_*/oldimage (oi_name,oi_timestamp);
 CREATE INDEX /*i*/oi_name_archive_name ON /*_*/oldimage (oi_name,oi_archive_name(14));
-CREATE INDEX /*i*/oi_sha1 ON /*_*/oldimage (oi_sha1);
+CREATE INDEX /*i*/oi_sha1 ON /*_*/oldimage (oi_sha1(10));
 CREATE TABLE /*_*/filearchive (
   fa_id int NOT NULL PRIMARY KEY AUTO_INCREMENT,
   fa_name varchar(255) binary NOT NULL default '',
@@ -282,12 +287,14 @@ CREATE TABLE /*_*/filearchive (
   fa_user int unsigned default 0,
   fa_user_text varchar(255) binary,
   fa_timestamp binary(14) default '',
-  fa_deleted tinyint unsigned NOT NULL default 0
+  fa_deleted tinyint unsigned NOT NULL default 0,
+  fa_sha1 varbinary(32) NOT NULL default ''
 ) /*$wgDBTableOptions*/;
 CREATE INDEX /*i*/fa_name ON /*_*/filearchive (fa_name, fa_timestamp);
 CREATE INDEX /*i*/fa_storage_group ON /*_*/filearchive (fa_storage_group, fa_storage_key);
 CREATE INDEX /*i*/fa_deleted_timestamp ON /*_*/filearchive (fa_deleted_timestamp);
 CREATE INDEX /*i*/fa_user_timestamp ON /*_*/filearchive (fa_user_text,fa_timestamp);
+CREATE INDEX /*i*/fa_sha1 ON /*_*/filearchive (fa_sha1(10));
 CREATE TABLE /*_*/uploadstash (
   us_id int unsigned NOT NULL PRIMARY KEY AUTO_INCREMENT,
   us_user int unsigned NOT NULL,
@@ -298,6 +305,7 @@ CREATE TABLE /*_*/uploadstash (
   us_timestamp varbinary(14) NOT NULL,
   us_status varchar(50) NOT NULL,
   us_chunk_inx int unsigned NULL,
+  us_props blob,
   us_size int unsigned NOT NULL,
   us_sha1 varchar(31) NOT NULL,
   us_mime varchar(255),
@@ -325,8 +333,7 @@ CREATE TABLE /*_*/recentchanges (
   rc_this_oldid int unsigned NOT NULL default 0,
   rc_last_oldid int unsigned NOT NULL default 0,
   rc_type tinyint unsigned NOT NULL default 0,
-  rc_moved_to_ns tinyint unsigned NOT NULL default 0,
-  rc_moved_to_title varchar(255) binary NOT NULL default '',
+  rc_source varchar(16) binary not null default '',
   rc_patrolled tinyint unsigned NOT NULL default 0,
   rc_ip varbinary(40) NOT NULL default '',
   rc_old_len int,
@@ -356,7 +363,7 @@ CREATE TABLE /*_*/searchindex (
   si_page int unsigned NOT NULL,
   si_title varchar(255) NOT NULL default '',
   si_text mediumtext NOT NULL
-) ENGINE=MyISAM;
+) ENGINE=MyISAM DEFAULT CHARSET=utf8;
 CREATE UNIQUE INDEX /*i*/si_page ON /*_*/searchindex (si_page);
 CREATE FULLTEXT INDEX /*i*/si_title ON /*_*/searchindex (si_title);
 CREATE FULLTEXT INDEX /*i*/si_text ON /*_*/searchindex (si_text);
@@ -409,6 +416,8 @@ CREATE INDEX /*i*/times ON /*_*/logging (log_timestamp);
 CREATE INDEX /*i*/log_user_type_time ON /*_*/logging (log_user, log_type, log_timestamp);
 CREATE INDEX /*i*/log_page_id_time ON /*_*/logging (log_page,log_timestamp);
 CREATE INDEX /*i*/type_action ON /*_*/logging (log_type, log_action, log_timestamp);
+CREATE INDEX /*i*/log_user_text_type_time ON /*_*/logging (log_user_text, log_type, log_timestamp);
+CREATE INDEX /*i*/log_user_text_time ON /*_*/logging (log_user_text, log_timestamp);
 CREATE TABLE /*_*/log_search (
   ls_field varbinary(32) NOT NULL,
   ls_value varchar(255) NOT NULL,
@@ -422,8 +431,16 @@ CREATE TABLE /*_*/job (
   job_namespace int NOT NULL,
   job_title varchar(255) binary NOT NULL,
   job_timestamp varbinary(14) NULL default NULL,
-  job_params blob NOT NULL
+  job_params blob NOT NULL,
+  job_random integer unsigned NOT NULL default 0,
+  job_attempts integer unsigned NOT NULL default 0,
+  job_token varbinary(32) NOT NULL default '',
+  job_token_timestamp varbinary(14) NULL default NULL,
+  job_sha1 varbinary(32) NOT NULL default ''
 ) /*$wgDBTableOptions*/;
+CREATE INDEX /*i*/job_sha1 ON /*_*/job (job_sha1);
+CREATE INDEX /*i*/job_cmd_token ON /*_*/job (job_cmd,job_token,job_random);
+CREATE INDEX /*i*/job_cmd_token_id ON /*_*/job (job_cmd,job_token,job_id);
 CREATE INDEX /*i*/job_cmd ON /*_*/job (job_cmd, job_namespace, job_title, job_params(128));
 CREATE INDEX /*i*/job_timestamp ON /*_*/job (job_timestamp);
 CREATE TABLE /*_*/querycache_info (
@@ -451,13 +468,13 @@ CREATE INDEX /*i*/qcc_type ON /*_*/querycachetwo (qcc_type,qcc_value);
 CREATE INDEX /*i*/qcc_title ON /*_*/querycachetwo (qcc_type,qcc_namespace,qcc_title);
 CREATE INDEX /*i*/qcc_titletwo ON /*_*/querycachetwo (qcc_type,qcc_namespacetwo,qcc_titletwo);
 CREATE TABLE /*_*/page_restrictions (
+  pr_id int unsigned NOT NULL PRIMARY KEY AUTO_INCREMENT,
   pr_page int NOT NULL,
   pr_type varbinary(60) NOT NULL,
   pr_level varbinary(60) NOT NULL,
   pr_cascade tinyint NOT NULL,
   pr_user int NULL,
-  pr_expiry varbinary(14) NULL,
-  pr_id int unsigned NOT NULL PRIMARY KEY AUTO_INCREMENT
+  pr_expiry varbinary(14) NULL
 ) /*$wgDBTableOptions*/;
 CREATE UNIQUE INDEX /*i*/pr_pagetype ON /*_*/page_restrictions (pr_page,pr_type);
 CREATE INDEX /*i*/pr_typelevel ON /*_*/page_restrictions (pr_type,pr_level);
@@ -480,6 +497,7 @@ CREATE TABLE /*_*/page_props (
   pp_value blob NOT NULL
 ) /*$wgDBTableOptions*/;
 CREATE UNIQUE INDEX /*i*/pp_page_propname ON /*_*/page_props (pp_page,pp_propname);
+CREATE UNIQUE INDEX /*i*/pp_propname_page ON /*_*/page_props (pp_propname,pp_page);
 CREATE TABLE /*_*/updatelog (
   ul_key varchar(255) NOT NULL PRIMARY KEY,
   ul_value blob
@@ -531,4 +549,32 @@ CREATE TABLE /*_*/module_deps (
   md_deps mediumblob NOT NULL
 ) /*$wgDBTableOptions*/;
 CREATE UNIQUE INDEX /*i*/md_module_skin ON /*_*/module_deps (md_module, md_skin);
--- vim: sw=2 sts=2 et
\ No newline at end of file
+CREATE TABLE /*_*/sites (
+  site_id                    INT UNSIGNED        NOT NULL PRIMARY KEY AUTO_INCREMENT,
+  site_global_key            varbinary(32)       NOT NULL,
+  site_type                  varbinary(32)       NOT NULL,
+  site_group                 varbinary(32)       NOT NULL,
+  site_source                varbinary(32)       NOT NULL,
+  site_language              varbinary(32)       NOT NULL,
+  site_protocol              varbinary(32)       NOT NULL,
+  site_domain                VARCHAR(255)        NOT NULL,
+  site_data                  BLOB                NOT NULL,
+  site_forward              bool                NOT NULL,
+  site_config               BLOB                NOT NULL
+) /*$wgDBTableOptions*/;
+CREATE UNIQUE INDEX /*i*/sites_global_key ON /*_*/sites (site_global_key);
+CREATE INDEX /*i*/sites_type ON /*_*/sites (site_type);
+CREATE INDEX /*i*/sites_group ON /*_*/sites (site_group);
+CREATE INDEX /*i*/sites_source ON /*_*/sites (site_source);
+CREATE INDEX /*i*/sites_language ON /*_*/sites (site_language);
+CREATE INDEX /*i*/sites_protocol ON /*_*/sites (site_protocol);
+CREATE INDEX /*i*/sites_domain ON /*_*/sites (site_domain);
+CREATE INDEX /*i*/sites_forward ON /*_*/sites (site_forward);
+CREATE TABLE /*_*/site_identifiers (
+  si_site                    INT UNSIGNED        NOT NULL,
+  si_type                    varbinary(32)       NOT NULL,
+  si_key                     varbinary(32)       NOT NULL
+) /*$wgDBTableOptions*/;
+CREATE UNIQUE INDEX /*i*/site_ids_type ON /*_*/site_identifiers (si_type, si_key);
+CREATE INDEX /*i*/site_ids_site ON /*_*/site_identifiers (si_site);
+CREATE INDEX /*i*/site_ids_key ON /*_*/site_identifiers (si_key);
