@@ -13,7 +13,7 @@ class AbstractAuthenticationProviderTest extends \MediaWikiTestCase {
 		$provider = $this->getMockForAbstractClass( AbstractAuthenticationProvider::class );
 		$providerPriv = TestingAccessWrapper::newFromObject( $provider );
 
-		$obj = $this->getMockForAbstractClass( 'Psr\Log\LoggerInterface' );
+		$obj = $this->getMockForAbstractClass( \Psr\Log\LoggerInterface::class );
 		$provider->setLogger( $obj );
 		$this->assertSame( $obj, $providerPriv->logger, 'setLogger' );
 
@@ -21,7 +21,7 @@ class AbstractAuthenticationProviderTest extends \MediaWikiTestCase {
 		$provider->setManager( $obj );
 		$this->assertSame( $obj, $providerPriv->manager, 'setManager' );
 
-		$obj = $this->getMockForAbstractClass( 'Config' );
+		$obj = $this->getMockForAbstractClass( \Config::class );
 		$provider->setConfig( $obj );
 		$this->assertSame( $obj, $providerPriv->config, 'setConfig' );
 
